#=============================================================================
# Find script for the ASIFT library
# ASIFT is an fully affine invariant image comparison presented by
# J.M. Morel and G.Yu (See References below)
#
# This script defines the following variables
#  ASIFT_INCLUDE_DIR
#  ASIFT_LIBRARIES match numerics
#  ASIFT_SOURCES 
#  ASIFT_FOUND, true if the above were found correctly
#
#
# The ASIFT library does provide a CMakeLists.txt, 
# but without any proper library definitions or install targets.
# It is therefore not sufficient to link against ASIFT_LIBRARIES.
# 
# Project Homepage: 
#	http://www.cmap.polytechnique.fr/~yu/research/ASIFT/demo.html
# Source Code can be found on the IPOL Journal homepage: 
#	http://www.ipol.im/pub/art/2011/my-asift/
#
#
# Find Script by Moritz Becker (moritz.becker@iwr.uni-heidelberg.de)
# modified by Stephan Meister (stephan.meister@iwr.uni-heidelberg.de)
# University of Heidelberg, 2012
#
# References:
#
# J.M. Morel and G.Yu, ASIFT: 
# A New Framework for Fully Affine Invariant Image Comparison,
# SIAM Journal on Imaging Sciences, vol. 2, issue 2, 2009.
#
# G.Yu and J.M. Morel, ASIFT: 
# An Algorithm for Fully Affine Invariant Comparison, 
# Image Processing On Line, 2011. DOI:10.5201/ipol.2011.my-asift.
#
# G. Yu and J.M. Morel, A Fully Affine Invariant Image Comparison Method, 
# Proc. IEEE International Conference on Acoustics, Speech, and Signal Processing (ICASSP),
# Taipei, 2009.
#
# J.M. Morel and G.Yu, Is SIFT Scale Invariant?, 
# Inverse Problems and Imaging, vol 5, no. 1, Feb., 2011



find_path(ASIFT_INCLUDE_DIR match.h )

IF(ASIFT_INCLUDE_DIR)
	SET(ASIFT_SOURCES
		${ASIFT_INCLUDE_DIR}/compute_asift_keypoints.cpp
		${ASIFT_INCLUDE_DIR}/compute_asift_matches.cpp
		${ASIFT_INCLUDE_DIR}/demo_lib_sift.cpp
		${ASIFT_INCLUDE_DIR}/fproj.cpp
		${ASIFT_INCLUDE_DIR}/numerics1.cpp 
		${ASIFT_INCLUDE_DIR}/frot.cpp 
		${ASIFT_INCLUDE_DIR}/splines.cpp 	
		${ASIFT_INCLUDE_DIR}/library.cpp 
		${ASIFT_INCLUDE_DIR}/flimage.cpp 
		${ASIFT_INCLUDE_DIR}/filter.cpp	
		${ASIFT_INCLUDE_DIR}/demo_lib_sift.cpp 
		${ASIFT_INCLUDE_DIR}/orsa.cpp)
ENDIF(ASIFT_INCLUDE_DIR)

set(ASIFT_MATCH_NAMES Match match)
FIND_LIBRARY(ASIFT_LIBRARY_MATCH
	NAMES ${ASIFT_MATCH_NAMES}
	HINTS ${ASIFT_INCLUDE_DIR}/Build/libMatch/Debug
		${ASIFT_INCLUDE_DIR}/Build/libMatch/Release
		${ASIFT_INCLUDE_DIR}/libMatch/Debug
		${ASIFT_INCLUDE_DIR}/libMatch/Release
)  
set(ASIFT_NUMERICS_NAMES Numerics numerics)
find_library(ASIFT_LIBRARY_NUMERICS 
	NAMES ${ASIFT_NUMERICS_NAMES} 
	HINTS ${ASIFT_INCLUDE_DIR}/Build/libNumerics/Debug
		${ASIFT_INCLUDE_DIR}/Build/libNumerics/Release
		${ASIFT_INCLUDE_DIR}/libNumerics/Debug
		${ASIFT_INCLUDE_DIR}/libNumerics/Release
)  

SET(ASIFT_LIBRARIES ${ASIFT_LIBRARY_MATCH} 
	${ASIFT_LIBRARY_NUMERICS} 
)

find_package_handle_standard_args(ASIFT
                                  REQUIRED_VARS ASIFT_LIBRARY_MATCH ASIFT_LIBRARY_NUMERICS ASIFT_SOURCES ASIFT_INCLUDE_DIR
                                  FAIL_MESSAGE "Could not find ASIFT Library. See cmake/FindASIFT.cmake for details and links.")

mark_as_advanced(ASIFT_INCLUDE_DIR ASIFT_LIBRARIES)
