/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file AddNoiseToPoints.h
 *  Declaration of the parameter class AddNoiseToPoints.
 *  \author Moritz Becker
 *  \date 21.03.2011
 */

#ifndef _ADDNOISETOPOINTS_H_
#define _ADDNOISETOPOINTS_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef addnoisetopoints_EXPORTS
/// Visual C++ specific code
#define addnoisetopoints_DECLDIR __declspec(dllexport)
#else
#define addnoisetopoints_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define addnoisetopoints_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Adds noise to correspondences. 
/** This module adds noise to preferably synthetic correspondences. */
template <typename T>
class addnoisetopoints_DECLDIR AddNoiseToPoints :
		public TemplatedParameteredObject<T> {
public:
	/// Noise in pixels.
	Parameter < float > noise;

	/// Input correspondences to which the noise will be added.
	InputSlot < std::vector<std::vector<std::vector<T> > > > correspondences;
	/// Output correspondences to which a Gaussian noise was added.
	OutputSlot < std::vector<std::vector<std::vector<T> > > > correspondencesOut;
	/// Output correspondences to which a Gaussian noise and also 10% total outliers were added.
	OutputSlot < std::vector<std::vector<std::vector<T> > > > correspondencesOut2;
	

	/// create a new AddNoiseToPoints object
	/// \param name          Instance name
	AddNoiseToPoints(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _ADDNOISETOPOINTS_H_



