/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file AddNoiseToPoints.hxx
 *  Implementation of the parameter class AddNoiseToPoints.
 *  \author Moritz Becker
 *  \date 21.03.2011
 */

#ifndef _ADDNOISETOPOINTS_HXX_
#define _ADDNOISETOPOINTS_HXX_

#include <hekate/AddNoiseToPoints.h>

template <typename T>
AddNoiseToPoints<T>::AddNoiseToPoints(const std::string& name) :
		TemplatedParameteredObject<T>(
				"testAddNoiseToPoints", name,
				"Adds noise to correspondences for test purposes."
		)
			,correspondences(false, false) 
			
			
{
	ParameteredObject::_addParameter (noise, "noise", 
	"noiseInPixels", 
	0.00f);
	
	ParameteredObject::_addInputSlot(correspondences, "correspondences", 
	"correspondences", 
	"vector<vector<vector<T> > >"); 
	ParameteredObject::_addOutputSlot(correspondencesOut, "correspondencesGaussian", 
	"correspondencesGaussian", 
	"vector<vector<vector<T> > >"); 
	ParameteredObject::_addOutputSlot(correspondencesOut2, "correspondencesOutlier", 
	"correspondencesOutlier", 
	"vector<vector<vector<T> > >"); 
	
}

template <typename T>
void AddNoiseToPoints<T>::execute() {
	PARAMETEREDOBJECT_AVOID_REEXECUTION;
	ParameteredObject::execute();

	sout << "Adding noise: " << noise() << std::endl;

	typename std::vector<std::vector<T> >::const_iterator begin =
		this->correspondences()[0].begin();
	typename std::vector<std::vector<T> >::const_iterator begin2 =
		this->correspondences()[1].begin();
	typename std::vector<std::vector<T> >::const_iterator end =
		this->correspondences()[0].end();

	int i=0;
	//calculate 10% of number of correspondences
	int corrSize = 20.;
		
	this->correspondencesOut().resize(2);
	this->correspondencesOut2().resize(2);

	for(; begin != end; begin++, begin2++){

		i++;
	
		//add noise to cimg vector 1
		cimg_library::CImg<T> vec(1,3);
		vec[0] = (T)(*begin)[0];
		vec[1] = (T)(*begin)[1];
		vec[2] = (T)(*begin)[2];
		vec /= vec[2];

		vec.noise(this->noise());
		vec[2] = T(1);

		//sout << (*begin)[0]  << ", " << (*begin)[1] << std::endl;
		//sout << vec[0]  << ", " << vec[1] << std::endl; 

		std::vector<T> stdvec;
		stdvec.push_back(vec[0]);
		stdvec.push_back(vec[1]);
		stdvec.push_back(vec[2]);
		
		this->correspondencesOut()[0].push_back(stdvec);

		//add noise to cimg vector 2
		vec[0] = (T)(*begin2)[0];
		vec[1] = (T)(*begin2)[1];
		vec[2] = (T)(*begin2)[2];
		vec /= vec[2];

		vec.noise(this->noise());
		vec[2] = T(1);

		//sout << (*begin2)[0]  << ", " << (*begin2)[1] << std::endl;
		//sout << vec[0]  << ", " << vec[1] << std::endl; 
		//sout << std::endl; 

		stdvec.clear();
		stdvec.push_back(vec[0]);
		stdvec.push_back(vec[1]);
		stdvec.push_back(vec[2]);
		
		this->correspondencesOut()[1].push_back(stdvec);
		
		//check if point should be an outlier
		if(i % corrSize == 0){
			
			sout << "Outlier at point " << i << std::endl;

			cimg_library::CImg<T> vec2(1,3);
			vec2[0] = (T)(*begin)[0];
			vec2[1] = (T)(*begin)[1];
			vec2[2] = (T)(*begin)[2];
			vec2 /= vec2[2];

			sout << "end " << std::endl;

			std::vector<T> stdvec2;
			stdvec2.push_back(vec2[0]);
			stdvec2.push_back(vec2[1]);
			stdvec2.push_back(vec2[2]);
			this->correspondencesOut2()[0].push_back(stdvec2);
			this->correspondencesOut2()[1].push_back(*begin2);	
			
		}
		else{
			this->correspondencesOut2()[0].push_back(*begin);
			this->correspondencesOut2()[1].push_back(*begin2);	
			
		}
	
	}

}

#endif /* _ADDNOISETOPOINTS_HXX_ */



