/*  Copyright (C) 2010 Mirko Schmidt

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file adjustbase.h
 *  Declaration of the parameter class adjustbase.
 *  \author Mirko Schmidt
 *  \date 13.04.2010
 */

#ifndef _ADJUSTBASE_H_
#define _ADJUSTBASE_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef adjustbase_EXPORTS
/// Visual C++ specific code
#define adjustbase_DECLDIR __declspec(dllexport)
#else
#define adjustbase_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define adjustbase_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Adjust baseline of a given stereo image pair, using a depth map computed before.
template <typename T>
class adjustbase_DECLDIR adjustbase :
		public TemplatedParameteredObject<T> {
public:
	
	
	/// image input  sequence left
	InputSlot < cimg_library::CImgList<T> > inputImagesL;
	/// image input  sequence right
	InputSlot < cimg_library::CImgList<T> > inputImagesR;
	/// dense depth map 
	InputSlot < cimg_library::CImgList<T> > image_disparity_map;
	/// Synthesized stereo images left
	OutputSlot < cimg_library::CImgList<T> > outputImagesL;
	/// Synthesized stereo images right
	OutputSlot < cimg_library::CImgList<T> > outputImagesR;
	/// pointer to the instance of this plugin
	OutputSlot < adjustbase<T>* > outputSlotThis;

	
	// factor of base shift of left image (alpha = 1 => no shift)
	Parameter<float> alpha_l;
	// factor of base shift of right image
	Parameter<float> alpha_r;
	// set inpainting interpolation style
	Parameter<int> interpolation_style;

	bool called;


	/// create a new adjustbase object
	/// \param name          Instance name
	adjustbase(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _ADJUSTBASE_H_



