/*  Copyright (C) 2010 Julian Coordts

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file AdjustBaseDisplay.h
 *  Declaration of the parameter class FileSequenceReader.
 *  \author Julian Coordts
 *  \date 13.04.2010
 */

#ifndef _ADJUSTBASEDISPLAY_H_
#define _ADJUSTBASEDISPLAY_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef adjustbasedisplay_EXPORTS
/// Visual C++ specific code
#define adjustbasedisplay_DECLDIR __declspec(dllexport)
#else
#define adjustbasedisplay_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define adjustbasedisplay_DECLDIR
#endif

#include "AdjustBaseDisplayInterface.h"
#include "AdjustBaseDisplayPluginGui.h"

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include <iostream>
#include <string>
#include <stdio.h>
#include "../AdjustBase/adjustbase.h"

/// Reads a selected pair of images of a sequence of right-left files.
/** To select them a slider has been build in.
 */
template <typename T>
class adjustbasedisplay_DECLDIR AdjustBaseDisplay :
		public TemplatedParameteredObject<T>,
		public AdjustBaseDisplayInterface {
private:
        AdjustBaseDisplayPluginGui* _userInterface;

public:
	/// image input
	InputSlot < cimg_library::CImgList<T> > inputImage;
	InputSlot < adjustbase<T>* > linkedAdjustBase;

	Parameter<int> maxAlphaLeft;
	Parameter<int> maxAlphaRight;

	/// create a new FileSequenceReader object
	/// \param name          Instance name
	AdjustBaseDisplay(const std::string& name);

	virtual ~AdjustBaseDisplay();

	virtual void processNew(float alphaL, float alphaR);
	virtual int getMaxAlphaLeft(){return maxAlphaLeft();}
	virtual int getMaxAlphaRight(){return maxAlphaRight();}

	/// Update object.
	virtual void execute();
};

#endif // _ADJUSTBASEDISPLAY_H_



