/*  Copyright (C) 2010 Julian Coordts

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file AdjustBaseDisplay.hxx
 *  Implementation of the parameter class AdjustBaseDisplay.
 *  \author Julian Coordts
 *  \date 13.04.2010
 */

#ifndef _ADJUSTBASEDISPLAY_HXX_
#define _ADJUSTBASEDISPLAY_HXX_

#include "AdjustBaseDisplay.h"
#include "convToPixmap.hxx"

template <typename T>
AdjustBaseDisplay<T>::AdjustBaseDisplay(const std::string& name) :
		TemplatedParameteredObject<T>("AdjustBaseDisplay", name,
			"Reads a selected pair of images of a sequence of right-left "
			"files. To select them a slider has been build in."),
		_userInterface(0)
{

	ParameteredObject::_addInputSlot(inputImage,
		"imageSequence",
		"the left image", "CImgList<T>");

	ParameteredObject::_addInputSlot(linkedAdjustBase,
		"linkedAdjustBase",
		"the AdjustBase Plugin to be controlled", "adjustbase<T>*");

	ParameteredObject::_addParameter (maxAlphaLeft, "max_alpha_left",
									  "max scaling factor image position left image",
									  10);

	ParameteredObject::_addParameter (maxAlphaRight, "max_alpha_right",
									  "max scaling factor image position right image",
									  10);

}

template <typename T>
void AdjustBaseDisplay<T>::execute() {

	ParameteredObject::execute();

	if (!_userInterface)
		_userInterface= new AdjustBaseDisplayPluginGui(this);

	QPixmap imgLeft, imgRight;
	convToPixmap(this->inputImage()[0],imgLeft) ;
	convToPixmap(this->inputImage()[1],imgRight) ;
	_userInterface->drawPictures(imgLeft,imgRight);

}


template <typename T>
void AdjustBaseDisplay<T>::processNew(float alphaL, float alphaR)
{
	sout << "Process new: " << std::endl;
	this->linkedAdjustBase()->alpha_l()=alphaL;
	this->linkedAdjustBase()->alpha_r()=alphaR;
	//execute again!!!
	this->linkedAdjustBase()->resetExecuted(true);
	const std::set<ParameteredObject*>& targetNodes=this->_getTargetNodes();
	std::set<ParameteredObject*>::const_iterator it;
	 for (it = 	targetNodes.begin();
		  it != targetNodes.end(); it++ )
		 {
			ParameteredObject* curPo= *it;
			sout << "recall: " << curPo->getName() << std::endl;
			curPo->execute();
		}
}

template <typename T>
AdjustBaseDisplay<T>::~AdjustBaseDisplay() {
	if (_userInterface)
		delete _userInterface;
}



#endif /* _ADJUSTBASEDISPLAY_HXX_ */
