/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file BackProject.h
 *  Declaration of the parameter class BackProject.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 27.01.2013
 */

#ifndef _BACKPROJECT_H_
#define _BACKPROJECT_H_

#ifdef _MSC_VER
#ifdef backproject_EXPORTS
/// Visual C++ specific code
#define backproject_DECLDIR __declspec(dllexport)
#else
#define backproject_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define backproject_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>

using namespace hekate;
using namespace cimg_library;

/// Back projects 3D points by cameras
/** Back projects 3D points by cameras
 */
template <typename T>
class backproject_DECLDIR BackProject :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	BackProject(const std::string& name = "");

	/// points3D
	InputSlot< CorrespondenceList<T> > points3D;
	/// Camera matrices P and P'.
	InputSlot < IndexedMatrixList > cameraMatrices;
	/// points2D
	InputSlot< CorrespondenceList<T> > correspondencesIn;
	
	/// correspondences
	OutputSlot< CorrespondenceList<T> > correspondences;


protected:
	/// Update object.
	virtual void execute();

};

#endif // _BACKPROJECT_H_
