/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file BackProjection3D2D.h
 *  Declaration of the parameter class BackProjection3D2D.
 *  \author Moritz Becker
 *  \date 16.02.2011
 */

#ifndef _BACKPROJECTION3D2D_H_
#define _BACKPROJECTION3D2D_H_

#ifdef _MSC_VER
#ifdef backprojection3d2d_EXPORTS
/// Visual C++ specific code
#define backprojection3d2d_DECLDIR __declspec(dllexport)
#else
#define backprojection3d2d_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define backprojection3d2d_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>

using namespace hekate;
using namespace cimg_library;

/// Projects 3D points X to 2D points x,x' via Camera Matrices P,P'.
/** This module back projects 3D points to 2D on the image planes of cameras
	P and P'. Therefore the relation x = PX and x' = P'X is used */
template <typename T>
class backprojection3d2d_DECLDIR BackProjection3D2D :
		public TemplatedParameteredObject<T> {
public:
	
	
	
	/// Maximal back projection error
	Parameter< float > maxBackProjectionError ;	
	/// Remove whole correspondence if one point of it is declared as outlier
	Parameter< bool > removeWholeCorrespondences ;
	/// Minimum number of 3d-2d point correspondences for every view,
	/// If there are less correspondences, a camera is set to degenerated
	Parameter < int > min3d2dCorrs; 

	/// Camera matrices P and P'.
	InputSlot < IndexedMatrixList > cameraMatrices;
	/// 3D points.
	InputSlot < CorrespondenceList<T> > points3D;
	/// Correspondences.
	InputSlot < CorrespondenceList<T> > correspondences;
	/// Back projected 2D points.
	OutputSlot < CorrespondenceList<T> > correspondencesBack;
	/// Corrected input correspondences where 2d points whose 3d point
	/// has a a taller back projection error as defined are marked as outliers
	OutputSlot < CorrespondenceList<T> > correspondencesInliers;
	/// 3D points whos backprojection is smaller than a defined threshold in all views.
	OutputSlot < CorrespondenceList<T> > pointCloudInliers;
	/// Input cameras where those are set to degerenated which do not exhibit
	/// enough 3d-2d correspondences
	OutputSlot < IndexedMatrixList > cameraMatricesOut;
	
	
	/// create a new BackProjection3D2D object
	/// \param name          Instance name
	BackProjection3D2D(const std::string& name = "");

	/// Update object.
	virtual void execute();
};

#endif // _BACKPROJECTION3D2D_H_



