/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Bilateral.h
 *  Declaration of the parameter class Bilateral.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 28.10.2013
 */

#ifndef _BILATERAL_H_
#define _BILATERAL_H_

#ifdef _MSC_VER
#ifdef bilateral_EXPORTS
/// Visual C++ specific code
#define bilateral_DECLDIR __declspec(dllexport)
#else
#define bilateral_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define bilateral_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// The bilateral function of CImg
/** The bilateral function of CImg
 */
template <typename T>
class bilateral_DECLDIR Bilateral :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	Bilateral(const std::string& name = "");

	/// input images
	InputSlot< cimg_library::CImgList<T> > in;

	/// Output images
	OutputSlot< cimg_library::CImgList<T> > out;

	/// Amount of blur along the XYZ-axes
	Parameter< float > blurXYZ;

	/// Amount of blur along the value axis
	Parameter< float > blurR;


protected:
	/// Update object.
	virtual void execute();
};

#endif // _BILATERAL_H_
