/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Bilateral.hxx
 *  Implementation of the parameter class Bilateral.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 28.10.2013
 */

#ifndef _BILATERAL_HXX_
#define _BILATERAL_HXX_

#include <charon-core/ParameteredObject.hxx>
#include "Bilateral.h"

template <typename T>
Bilateral<T>::Bilateral(const std::string& name) :
		TemplatedParameteredObject<T>(
			"Bilateral", name,
			"<h2>The bilateral function of CImg</h2><br>"
			"The bilateral function of CImg"
		)
{
	ParameteredObject::_addInputSlot(
		in, "in",
		"input images",
		"CImgList<T>");

	ParameteredObject::_addOutputSlot(
		out, "out",
		"Output images",
		"CImgList<T>");
	
	ParameteredObject::_addParameter< float >(
		blurXYZ, "blurXYZ",
		"Amount of blur along the XYZ-axes. ",
		5, "float");
	
	ParameteredObject::_addParameter< float >(
		blurR, "blurR",
		"Amount of blur along the value axis. ",
		5, "float");

}

template <typename T>
void Bilateral<T>::execute() {
	
	for( size_t i = 0; i < this->in().size(); i++){
		cimg_library::CImg<float> blur = in()[i];
		blur.blur_bilateral(blurXYZ,10);
		this->out().push_back( blur );
	}

}

#endif /* _BILATERAL_HXX_ */
