/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file CImgListFromPath.h
 *  Declaration of the parameter class CImgListFromPath.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 19.10.2012
 */

#ifndef _CIMGLISTFROMPATH_H_
#define _CIMGLISTFROMPATH_H_

#ifdef _MSC_VER
#ifdef cimglistfrompath_EXPORTS
/// Visual C++ specific code
#define cimglistfrompath_DECLDIR __declspec(dllexport)
#else
#define cimglistfrompath_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define cimglistfrompath_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <charon-utils/FileReader.h>

using namespace cimg_library;

/// Reads in images from path
/** Reads in images from path as CImgList
 */
template <typename T>
class cimglistfrompath_DECLDIR CImgListFromPath :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	CImgListFromPath(const std::string& name = "");

	/// paths
	InputSlot< std::vector<std::string> > paths;

	/// images
	OutputSlot< cimg_library::CImgList<T> > images;

	/// Read flo files from string list
	Parameter < bool > readFlo;	

	/// Create black images for images without known file string
	Parameter < bool > fillEmptyImages;	
	
	/// Update object.
	virtual void execute();
};

#endif // _CIMGLISTFROMPATH_H_
