/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file CalibrationFromIndexedMatrixList.hxx
 *  Implementation of the parameter class CalibrationFromIndexedMatrixList.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 25.02.2013
 */

#ifndef _CALIBRATIONFROMINDEXEDMATRIXLIST_HXX_
#define _CALIBRATIONFROMINDEXEDMATRIXLIST_HXX_

#include <charon-core/ParameteredObject.hxx>
#include <hekate/CalibrationFromIndexedMatrixList.h>
#include <charon-utils/ImgTool.hxx>

template <typename T>
CalibrationFromIndexedMatrixList<T>::CalibrationFromIndexedMatrixList(const std::string& name) :
		TemplatedParameteredObject<T>(
			"CalibrationFromIndexedMatrixList", name,
			"<h2>Reads out calibration matrixes from "
			"indexedmatrixlist</h2><br>"
			"Reads out calibration matrices from indexedmatrixlist."
		)
{
	ParameteredObject::_addInputSlot(
		indexedMatrixList, "indexedMatrixList",
		"indexedMatrixList",
		"IndexedMatrixList");

	ParameteredObject::_addOutputSlot(
		calibrations, "calibrations",
		"Calibration matrices generated by focal length parameter (f,0,0,0,f,0,0,0,1)",
		"CImgList<T>");

	ParameteredObject::_setTags("hekate") ;
}

template <typename T>
void CalibrationFromIndexedMatrixList<T>::execute() {
	
	for( size_t i = 0; i<this->indexedMatrixList().size(); i++)
	{
		const IndexedMatrix &mat = this->indexedMatrixList()[i];
		CImg<T> cal(3,3);
		cal.fill( T(0) );
		cal(0,0) = mat.f;
		cal(1,1) = mat.f;		
		cal(2,2) = T(1);
		this->calibrations().push_back( cal );
		//ImgTool::printInfo(sout,cal,"cali");
	}

}

#endif /* _CALIBRATIONFROMINDEXEDMATRIXLIST_HXX_ */
