/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file calibrationMatrices.h
 *  Declaration of the parameter class calibrationMatrices.
 *  \author Moritz Becker
 *  \date 14.01.2011
 */

#ifndef _CALIBRATIONMATRICES_H_
#define _CALIBRATIONMATRICES_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef calibrationmatrices_EXPORTS
/// Visual C++ specific code
#define calibrationmatrices_DECLDIR __declspec(dllexport)
#else
#define calibrationmatrices_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define calibrationmatrices_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Creates CImgList of two CImg calibration matrices by parameter input.
/** Create two calibration matrices saved in a CImgList. Those calibration
	matrices can used to reconstruct scenes where the calibration of used
	cameras is known but also to estimate calibrations by the image size
	of an optional related image. */
template <typename T>
class calibrationmatrices_DECLDIR calibrationMatrices :
		public TemplatedParameteredObject<T> {
public:
	

	/// Calibration parameters of camera 1.
	ParameterList < double > calibration1;
	/// Calibration parameters of camera 2.
	ParameterList < double > calibration2;

	/// Optional image from which the calibration is estimated from.
	InputSlot < cimg_library::CImgList<T> > image;

	/// Output calibration matrices.
	OutputSlot < cimg_library::CImgList<T> > calibration;

	/// Inverted output calibration matrices.
	OutputSlot < cimg_library::CImgList<T> > invertedCalibration;	


	/// Create new calibrationMatrices object
	/// \param name          Instance name
	calibrationMatrices(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _CALIBRATIONMATRICES_H_



