/*  Copyright (C) 2014 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file CalibrationMatrixGenerator.h
 *  Declaration of the parameter class CalibrationMatrixGenerator.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 15.01.2014
 */

#ifndef _CALIBRATIONMATRIXGENERATOR_H_
#define _CALIBRATIONMATRIXGENERATOR_H_

#ifdef _MSC_VER
#ifdef calibrationmatrixgenerator_EXPORTS
/// Visual C++ specific code
#define calibrationmatrixgenerator_DECLDIR __declspec(dllexport)
#else
#define calibrationmatrixgenerator_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define calibrationmatrixgenerator_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Generates a calibration matrix
/** Generates a calibration matrix using given focal length and pricipal point
 */
template <typename T>
class calibrationmatrixgenerator_DECLDIR CalibrationMatrixGenerator :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	CalibrationMatrixGenerator(const std::string& name = "");

	/// Calibration matrix
	OutputSlot< cimg_library::CImgList<T> > calibration;

	/// Focal length
	Parameter< float > f;
	/// Principal point X coordinate. The principle point can be assumed as
	/// the middle of the image
	Parameter< int > px;
	/// Principal point Y coordinate. The principle point can be assumed as
	/// the middle of the image
	Parameter< int > py;
	/// Use the given milimeter focal length and the given sensor size
	///	to determine the focal length
	Parameter< bool > useMilimeterValues;
	/// width of the image
	Parameter< int > imageWidth;
	/// focal length in milimeters
	Parameter< int > focalLengthMM;
	/// Sensor width in milimeters
	Parameter< float > sensorWidthMM;

protected:
	/// Update object.
	virtual void execute();
};

#endif // _CALIBRATIONMATRIXGENERATOR_H_
