/*  Copyright (C) 2009 SasiKanth

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file cameramatrixgenerator.h
 *  Declaration of the parameter class cameramatrixgenerator.
 *  \author SasiKanth
 *  \date 02.06.2010
 */

#ifndef _CAMERAMATRIXGENERATOR_H_
#define _CAMERAMATRIXGENERATOR_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef cameramatrixgenerator_EXPORTS
/// Visual C++ specific code
#define cameramatrixgenerator_DECLDIR __declspec(dllexport)
#else
#define cameramatrixgenerator_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define cameramatrixgenerator_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>

using namespace hekate;

/// This plugin generates a set of camera matrices from the F matrix given as the input (Hartley pg. 256).
/** This plugin generates a set of camera matrices. The cameras are projective, if only a fundamental
	matrix without calibration is known or metric, if an essential matrix (fundamental matrix with 
	calibration) is known. */
template <typename T>
class cameramatrixgenerator_DECLDIR cameramatrixgenerator :
		public TemplatedParameteredObject<T> {
public:
	
	/// Try to transform subsequent cams to same coordinate system.
	Parameter < bool > transformCam;
	/// Flag to use the input matrix as essential matrix for metric cameras.
	Parameter < bool > useEssentialMatrix;
	/// Correspondences to determine depth of points to check which of the calculated cameras is the correct one.
	InputSlot < typename hekate::CorrespondenceList<T> > correspondences;

	/// Calibration matrix which is necessary for metric cameras.
	InputSlot < cimg_library::CImgList<T> > calibration;
	/// The fundamental matrix.
	InputSlot < cimg_library::CImgList<T> > f_matrix;
	/// The two output camera matrices. 
	OutputSlot < cimg_library::CImgList<T> > p_matrices;

	/// create a new cameramatrixgenerator object
	/// \param name          Instance name
	cameramatrixgenerator(const std::string& name);

	/// Update object.
	virtual void execute();

	double getDepth(cimg_library::CImg<T> P, cimg_library::CImg<T> X);

	cimg_library::CImg<T> getTriangulation(
		cimg_library::CImg<T> P1, cimg_library::CImg<T> P2, 
		cimg_library::CImg<T> x1, cimg_library::CImg<T> x2);
};

#endif // _CAMERAMATRIXGENERATOR_H_
