/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file CannyEdges.h
 *  Declaration of the parameter class CannyEdges.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 17.09.2012
 */

#ifndef _CANNYEDGES_H_
#define _CANNYEDGES_H_

#ifdef _MSC_VER
#ifdef cannyedges_EXPORTS
/// Visual C++ specific code
#define cannyedges_DECLDIR __declspec(dllexport)
#else
#define cannyedges_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define cannyedges_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <opencv/cv.h>
#include <hekate/openCvUtil.hxx>

using namespace cimg_library;
using namespace cv ;
using namespace openCvUtil ;

/// Detects edges by Canny algorithm
/** Computes Canny edges
 */
template <typename T>
class cannyedges_DECLDIR CannyEdges :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	CannyEdges(const std::string& name = "");
	
	/// Input images
	InputSlot< cimg_library::CImgList<T> > inputImages;
	/// Input mask images
	InputSlot< cimg_library::CImgList<T> > probability;

	/// Edge images
	OutputSlot< cimg_library::CImgList<T> > edgeImages;

	/// First threshold for the hysteresis procedure.
	Parameter< int > threshold1;
	/// Second threshold for the hysteresis procedure.
	Parameter< int > threshold2;
	/// Aperture size for the Sobel() operator.
	Parameter< int > apertureSize;
	/// Limit to edges masked by mask images.
	Parameter< bool > l2gradient;
	
	/// channel hue
	Parameter< bool > H;
	/// channel saturation
	Parameter< bool > S;
	/// channel brightness
	Parameter< bool > V;
	/// channel red
	Parameter< bool > R;
	/// channel green
	Parameter< bool > G;
	/// channel blue
	Parameter< bool > B;
	
	/// sum different channels to discrete edge value
	Parameter< bool > discrete;	
	
	/// blur before canny
	Parameter< float > blur;
	
	/// factor for probabilites on threshold 1
	Parameter< float > sigma1;
	/// factor for probabilites on threshold 2
	Parameter< float > sigma2;

	/// Start frame
	Parameter< int > inFrame;
	/// End frame
	Parameter< int > outFrame;

	/// Update object.
	virtual void execute();
};

#endif // _CANNYEDGES_H_
