/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file CompleteEdges.h
 *  Declaration of the parameter class CompleteEdges.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 06.04.2013
 */

#ifndef _COMPLETEEDGES_H_
#define _COMPLETEEDGES_H_

#ifdef _MSC_VER
#ifdef completeedges_EXPORTS
/// Visual C++ specific code
#define completeedges_DECLDIR __declspec(dllexport)
#else
#define completeedges_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define completeedges_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Define a none edge pixel as an edge pixel, if X nearest neighbours are
/// edge pixels
/** Define a none edge pixel as an edge pixel, if X nearest neighbours are
 *  edge pixels
 */
template <typename T>
class completeedges_DECLDIR CompleteEdges :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	CompleteEdges(const std::string& name = "");

	/// discrete two value images
	InputSlot< cimg_library::CImgList<T> > images;

	/// Corrected edge images
	OutputSlot< cimg_library::CImgList<T> > imagesOut;
	
	/// Remove pixels without (8) nearest neighbour edges
	Parameter< bool > removeSinglePixels;

	/// The number of nearest neighbours to define a none edge pixel as an
	/// edge pixel.
	Parameter< int > numberOfNearestNeighbours;

	/// The number of nearest neighbours to define an edge pixel as a none edge pixel
	Parameter< int > numberOfNearestNeighbours2;
	
	/// Number of iterations for every image
	Parameter< int > numberOfIterations;

	/// Threshold which defines an edge (for non discrete 2 value images)
	Parameter< float > threshold;

protected:
	/// Update object.
	virtual void execute();
};

#endif // _COMPLETEEDGES_H_
