/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file CompleteMask.h
 *  Declaration of the parameter class CompleteMask.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 06.09.2013
 */

#ifndef _COMPLETEMASK_H_
#define _COMPLETEMASK_H_

#ifdef _MSC_VER
#ifdef completemask_EXPORTS
/// Visual C++ specific code
#define completemask_DECLDIR __declspec(dllexport)
#else
#define completemask_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define completemask_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

using namespace cimg_library;

/// Repairs a broken mask where a consisten edge is not available
/** Repairs a broken mask where a consisten edge is not available. A maximal
 *  distance which defines the maximal length of an repair edge has to be
 *  defined.
 */
template <typename T>
class completemask_DECLDIR CompleteMask :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	CompleteMask(const std::string& name = "");

	/// Mask images which are putatively broken.
	InputSlot< cimg_library::CImgList<T> > masks;

	/// The repaired masks
	OutputSlot< cimg_library::CImgList<T> > masksCompleted;

	/// Maximal distance between two broken segments.
	Parameter< int > maxDistance;

	/// Minimal allowed number of pixels of a sub edge to be considered for completion
	Parameter< int > minSize;

	/// Maximal allowed number of pixels of black hole segments to be removed
	Parameter< int > maxSizeBlack;



protected:
	/// Update object.
	virtual void execute();
};

#endif // _COMPLETEMASK_H_
