/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file ConvertVecToCImg.h
 *  Declaration of the parameter class ConvertVecToCImg.
 *  \author Moritz Becker
 *  \date 02.03.2011
 */

#ifndef _CONVERTVECTOCIMG_H_
#define _CONVERTVECTOCIMG_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef convertvectocimg_EXPORTS
/// Visual C++ specific code
#define convertvectocimg_DECLDIR __declspec(dllexport)
#else
#define convertvectocimg_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define convertvectocimg_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Converts 3D and 2D vector containers into CImgLists and the other way around.
template <typename T>
class convertvectocimg_DECLDIR ConvertVecToCImg :
		public TemplatedParameteredObject<T> {
public:
	
	/// 3D vector which will be converted to a 3D CImgList
	InputSlot < std::vector<std::vector<T> > > points3DVec;
	/// 3D CImgList which will be converted to a 3D vector
	InputSlot < cimg_library::CImgList<T> > points3DCImg;
	/// 2D vector which will be converted to a 2D CImgList
	InputSlot < std::vector<std::vector<std::vector<T> > > > points2DVec;
	/// 2D CImgList which will be converted to a 2D vector
	InputSlot < cimg_library::CImgList<T> > points2DCImg;
	/// Stereo disp image which contains rectified correspondences as CImg
	InputSlot < cimg_library::CImgList<T> > pointsStereo;
	/// Converted 3D CImgList from the 3D input vector
	OutputSlot < cimg_library::CImgList<T> > points3DCImgOut;
	/// Converted 3D vector from the 3D input CImgList
	OutputSlot < std::vector<std::vector<T> > > points3DVecOut;
	/// Converted 2D CImgList from the 2D input vector
	OutputSlot < cimg_library::CImgList<T> > points2DCImgOut;
	/// Converted 2D vector from the 3D input CImgList
	OutputSlot < std::vector<std::vector<std::vector<T> > > > points2DVecOut;
	/// Stereo disp image which contains rectified correspondences as std::vector
	OutputSlot < std::vector<std::vector<std::vector<T> > > > pointsStereoOut;
	

	/// create a new ConvertVecToCImg object
	/// \param name          Instance name
	ConvertVecToCImg(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _CONVERTVECTOCIMG_H_



