/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file correctCorrespondences.h
 *  Declaration of the parameter class correctCorrespondences.
 *  \author Moritz Becker
 *  \date 22.10.2010
 */

#ifndef _CORRECTCORRESPONDENCES_H_
#define _CORRECTCORRESPONDENCES_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef correctcorrespondences_EXPORTS
/// Visual C++ specific code
#define correctcorrespondences_DECLDIR __declspec(dllexport)
#else
#define correctcorrespondences_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define correctcorrespondences_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
// Include vigra windows.h to avoid errors
#include <vigra/windows.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>

using namespace hekate;
using namespace cimg_library;

/// Corrects correspondences by the epipolar constraint; The points are shifted to the nearest point of the epipolar line. (See Hartley p 318)
template <typename T>
class correctcorrespondences_DECLDIR correctCorrespondences :
		public TemplatedParameteredObject<T> {
public:
	


	/// Maximal allowed distance of the corrected point from the original point.
	Parameter < T > maxDistance;	

	/// Input correspondences.
	InputSlot < typename CorrespondenceList<T> > inputCorrespondences;
	/// Output correspondences.
	OutputSlot < typename CorrespondenceList<T> > outputCorrespondences;
	/// Fudamental matrix. 
	InputSlot < typename IndexedMatrixList > fmatrix;
	

	/// create a new correctCorrespondences object
	/// \param name          Instance name
	correctCorrespondences(const std::string& name);

	void getEpipoles(
			const CImg<double>& fmatrix,
			CImg<double>& epipole);
	double getS(double &t, CImg<double> &F, 
			CImg<double> &e1, CImg<double> &e2);
	bool getT(std::vector<double> &t, CImg<double> &F, 
			CImg<double> &e1, CImg<double> &e2);

	/// Update object.
	virtual void execute();
};

#endif // _CORRECTCORRESPONDENCES_H_



