/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file CorrectCorrespondencesOpenCV.h
 *  Declaration of the parameter class CorrectCorrespondencesOpenCV.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 21.08.2012
 */

#ifndef _CORRECTCORRESPONDENCESOPENCV_H_
#define _CORRECTCORRESPONDENCESOPENCV_H_

#ifdef _MSC_VER
#ifdef correctcorrespondencesopencv_EXPORTS
/// Visual C++ specific code
#define correctcorrespondencesopencv_DECLDIR __declspec(dllexport)
#else
#define correctcorrespondencesopencv_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define correctcorrespondencesopencv_DECLDIR
#endif

#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>
#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

using namespace hekate;

/// Corrects correspondences by epipolar geometry
/** Corrects correspondences by epipolar geometry using the fundamental
 *  matrices.
 */
template <typename T>
class correctcorrespondencesopencv_DECLDIR CorrectCorrespondencesOpenCV :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	CorrectCorrespondencesOpenCV(const std::string& name = "");

	/// Correspondences to correct.
	InputSlot< hekate::CorrespondenceList<T> > correspondences;
	/// Fundamental Matrices for epipolar geometry
	InputSlot< IndexedMatrixList > fundamentalMatrix;
	/// initial frames (optional)
	InputSlot< std::vector<int> > initialFrames;

	/// Corrected correspondences
	OutputSlot< hekate::CorrespondenceList<T> > correctedCorrespondences;
	
	/// Initial Frame 1
	Parameter< int > initFrame1 ;
	
	/// Initial Frame 2
	Parameter< int > initFrame2 ;
	
	/// Initial Frame 3
	Parameter< int > initFrame3 ;

	/// Update object.
	virtual void execute();
};

#endif // _CORRECTCORRESPONDENCESOPENCV_H_
