/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Correspondence2FlowMap.h
 *  Declaration of the parameter class Correspondence2FlowMap.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 07.03.2013
 */

#ifndef _CORRESPONDENCE2FLOWMAP_H_
#define _CORRESPONDENCE2FLOWMAP_H_

#ifdef _MSC_VER
#ifdef correspondence2flowmap_EXPORTS
/// Visual C++ specific code
#define correspondence2flowmap_DECLDIR __declspec(dllexport)
#else
#define correspondence2flowmap_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define correspondence2flowmap_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>

using namespace hekate;
using namespace cimg_library;

/// Writes correspondences to a images which can be used as an initial guess
/// for optical flow
/** This plugin converts correspondences to:
 *  1) a mask where all pixels which are part of the correspondence are 1,
 *  otherwise 0
 *  2) two images with the x and y shift value for the masked pixels to shift
 *  the point to its correspondence in the subsequent frame
 */
template <typename T>
class correspondence2flowmap_DECLDIR Correspondence2FlowMap :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	Correspondence2FlowMap(const std::string& name = "");
	
	/// Correspondences to generate the mask and images
	InputSlot< CorrespondenceList<T> > correspondences;
	/// Source images to read out width and heigth of every image
	InputSlot< std::vector<std::string> > images;

	/// Masks which mask correspondence pixels
	OutputSlot< cimg_library::CImgList<T> > masks;
	/// Translation in x direction of the correspondence
	OutputSlot< cimg_library::CImgList<T> > xTranslation;
	/// Translation in y direction of the correspondence
	OutputSlot< cimg_library::CImgList<T> > yTranslation;


protected:
	/// Update object.
	virtual void execute();
};

#endif // _CORRESPONDENCE2FLOWMAP_H_
