/*  Copyright (C) 2011 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.

	-------------------------------------------------------------------------
	This plugin uses the LIBSVM:
	Copyright (c) 2000-2011 Chih-Chung Chang and Chih-Jen Lin
	All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer.

	2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in the
	documentation and/or other materials provided with the distribution.

	3. Neither name of copyright holders nor the names of its contributors
	may be used to endorse or promote products derived from this software
	without specific prior written permission.


	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
	``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
	A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
	LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
	NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	-----------------------------------------------------------------------


*/
/** \file CorrespondenceOutlierElimination.h
 *  Declaration of the parameter class CorrespondenceOutlierElimination.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 21.09.2011
 */

#ifndef _CORRESPONDENCEOUTLIERELIMINATION_H_
#define _CORRESPONDENCEOUTLIERELIMINATION_H_

#ifdef _MSC_VER
#ifdef correspondenceoutlierelimination_EXPORTS
/// Visual C++ specific code
#define correspondenceoutlierelimination_DECLDIR __declspec(dllexport)
#else
#define correspondenceoutlierelimination_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define correspondenceoutlierelimination_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>

/// Eliminates correspondence outliers by SVM regression method
/** Eliminates correspondence outliers by SVM regression method. This plugin
 *  uses <a href="http://www.csie.ntu.edu.tw/~cjlin/libsvm/">LIBSVM</a>.
 */
template <typename T>
class correspondenceoutlierelimination_DECLDIR CorrespondenceOutlierElimination :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	CorrespondenceOutlierElimination(const std::string& name = "");

	/// correspondences
	InputSlot< typename hekate::CorrespondenceList<T> > correspondences;

	/// correspondencesOut
	OutputSlot< typename hekate::CorrespondenceList<T> > correspondencesOut;

	///Confidence level for the standard deviation (stdev) of
	/// sub set correspondence views (CV). 
	/** The ratio between stdev of initial CV and stdev of sub set CV has
	 *	to be taller than subsetThreshold.
	 */
	Parameter< float > threshold1;
	/// Confidence level for outliers
	/** To detect an outlier, Err(x)^2*Stdev(x)^2+Err(y)^2*Stdev(y)^2
	 *	has to be taller than outlierThreshold.
	 */
	Parameter< float > threshold2;
	/// Confidence level for influentals. 
	/** The error of influentals has to be taller than 
	 *  influentalThreshold*sigma where as 
	 * sigma is calculated automatically.
	 */
	Parameter< float > threshold3;
	/// Maximal relative amount of influentals which can be removed 
	///to estimate a correspondence view
	Parameter< float > threshold4;
	

	/// Update object.
	virtual void execute();


};

#endif // _CORRESPONDENCEOUTLIERELIMINATION_H_
