/*  Copyright (C) 2011 Stephan Meister

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file CorrespondenceReadWrite.h
 *  Declaration of the parameter class CorrespondenceReadWrite.
 *  \author <a href="mailto:stephan.meister@iwr.uni-heidelberg.de">
 *      Stephan Meister</a>
 *  \date 20.09.2011
 */

#ifndef _CORRESPONDENCEREADWRITE_H_
#define _CORRESPONDENCEREADWRITE_H_

#ifdef _MSC_VER
#ifdef correspondencereadwrite_EXPORTS
/// Visual C++ specific code
#define correspondencereadwrite_DECLDIR __declspec(dllexport)
#else
#define correspondencereadwrite_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define correspondencereadwrite_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <hekate/Correspondence.hxx>

/// read list of objects of type Correspondence from file
/** supported file structure
 *  x y z w i1 i2 #x,y,z,w coordinates, point Index, img index
 */
template <typename T>
class correspondencereadwrite_DECLDIR CorrespondenceReadWrite :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	CorrespondenceReadWrite(const std::string& name = "");

	/// filename
	Parameter< std::string > _filename;

	Parameter<bool> _verbose ;

	Parameter<bool> _pass ;

	/// correspondences
	InputSlot< typename hekate::CorrespondenceList<T> > _in;

	/// correspondences
	OutputSlot< typename hekate::CorrespondenceList<T> > _out;

	size_t _sizeAll;

	/// Update object.
	virtual void execute();
};

#endif // _CORRESPONDENCEREADWRITE_H_
