/*  Copyright (C) 2014 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file CorrespondenceWriter.h
 *  Declaration of the parameter class CorrespondenceWriter.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 13.01.2014
 */

#ifndef _CORRESPONDENCEWRITER_H_
#define _CORRESPONDENCEWRITER_H_

#ifdef _MSC_VER
#ifdef correspondencewriter_EXPORTS
/// Visual C++ specific code
#define correspondencewriter_DECLDIR __declspec(dllexport)
#else
#define correspondencewriter_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define correspondencewriter_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>

/// Writes CorrespondenceList to txt file
/** Writes CorrespondenceList to txt file
 */
template <typename T>
class correspondencewriter_DECLDIR CorrespondenceWriter :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	CorrespondenceWriter(const std::string& name = "");

	/// CorrespondenceList to write to file
	InputSlot< typename hekate::CorrespondenceList<T> > correspondence;

	/// File to write to
	Parameter< std::string > filename;


protected:
	/// Update object.
	virtual void execute();
};

#endif // _CORRESPONDENCEWRITER_H_
