/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file CreateTrackingMatrix.h
 *  Declaration of the parameter class CreateTrackingMatrix.
 *  \author Moritz Becker
 *  \date 29.08.2011
 */

#ifndef _CREATETRACKINGMATRIX_H_
#define _CREATETRACKINGMATRIX_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef createtrackingmatrix_EXPORTS
/// Visual C++ specific code
#define createtrackingmatrix_DECLDIR __declspec(dllexport)
#else
#define createtrackingmatrix_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define createtrackingmatrix_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Compares subsequent feature coordinates and uses them to create a tracking matrix.
template <typename T>
class createtrackingmatrix_DECLDIR CreateTrackingMatrix :
		public TemplatedParameteredObject<T> {
public:
	/// Path where the tracking matrix is saved. 
	Parameter < std::string > filename;
	
	
	/// Optimized tracking matrix. 
	OutputSlot < cimg_library::CImgList<T> > trackingMatrixOut;


	/// Input correspondences. 
	InputSlot < std::vector<std::vector<std::vector<T> > > > correspondences;
	

	/// create a new CreateTrackingMatrix object
	/// \param name          Instance name
	CreateTrackingMatrix(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _CREATETRACKINGMATRIX_H_



