/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file DepthEdgeEvaluation.h
 *  Declaration of the parameter class DepthEdgeEvaluation.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 27.03.2013
 */

#ifndef _DEPTHEDGEEVALUATION_H_
#define _DEPTHEDGEEVALUATION_H_

#ifdef _MSC_VER
#ifdef depthedgeevaluation_EXPORTS
/// Visual C++ specific code
#define depthedgeevaluation_DECLDIR __declspec(dllexport)
#else
#define depthedgeevaluation_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define depthedgeevaluation_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

using namespace cimg_library;

/// Compares predicted depth edges with a given ground truth to evaluate
/// results
/** The module determines which pixels are predicted correctly and which
 *  pixels are wrongly identified as depth edges.
 */
template <typename T>
class depthedgeevaluation_DECLDIR DepthEdgeEvaluation :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	DepthEdgeEvaluation(const std::string& name = "");

	/// Ground truth images
	InputSlot< cimg_library::CImgList<T> > groundTruth;
	/// Predicted pixels
	InputSlot< cimg_library::CImgList<T> > prediction;
	/// A mask to limit ground truth pixels and predicted pixels to the masked pixels.
	/**
		This is useful, since a manually given ground truth and the predicted pixels can 
		be reduced to pixels, lying on an edge. The drawback is, that the edge masks not only 
		reduces given values to edges but also may remove values from actual edges, when the mask 
		is wrong. Check the quality of the mask to classify the evaluation
	*/
	InputSlot< cimg_library::CImgList<T> > mask;

	/// Evaluated pixels separated in RGB channels (R=false negative,G=positive,B=false positive)
	OutputSlot< cimg_library::CImgList<T> > predictionEvaluated;
	/// Statistically evaluated values
	OutputSlot< cimg_library::CImgList<T> > statistic;
	

	/// If continuous probability images are given, a threshold for depth edges has to be defined. 
	/** This is only relevant, if descreteValues is not set */
	Parameter< float > edgeThreshold;

	/// Use index list to assign indices to ground truth images
	/** If no index list is given, image pairs are assigned in increasing order */
	Parameter< bool > useGtIndices;

	/// List of ground truth indices
	ParameterList< unsigned int > indices;

protected:
	/// Update object.
	virtual void execute();
};

#endif // _DEPTHEDGEEVALUATION_H_
