/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file DepthEdgeMergeFeatures.h
 *  Declaration of the parameter class DepthEdgeMergeFeatures.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 12.11.2013
 */

#ifndef _DEPTHEDGEMERGEFEATURES_H_
#define _DEPTHEDGEMERGEFEATURES_H_

#ifdef _MSC_VER
#ifdef depthedgemergefeatures_EXPORTS
/// Visual C++ specific code
#define depthedgemergefeatures_DECLDIR __declspec(dllexport)
#else
#define depthedgemergefeatures_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define depthedgemergefeatures_DECLDIR
#endif

#include <charon-utils/CImg.h>
#include <charon-core/ParameteredObject.h>

using namespace cimg_library;

/// Merges two CImgLists containing depth edge features
/** Merges two CImgLists containing depth edge features
 */
template <typename T>
class depthedgemergefeatures_DECLDIR DepthEdgeMergeFeatures :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	DepthEdgeMergeFeatures(const std::string& name = "");
	
	/// First feature list to merge
	InputSlot< std::vector<std::string> > features1;
	/// Second feature list to merge
	InputSlot< std::vector<std::string> > features2;

	/// Merged feature lists
	OutputSlot< cimg_library::CImgList<T> > merged;


protected:
	/// Update object.
	virtual void execute();
};

#endif // _DEPTHEDGEMERGEFEATURES_H_
