/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file DepthEdgeMergeFeatures.hxx
 *  Implementation of the parameter class DepthEdgeMergeFeatures.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 12.11.2013
 */

#ifndef _DEPTHEDGEMERGEFEATURES_HXX_
#define _DEPTHEDGEMERGEFEATURES_HXX_

#include <charon-core/ParameteredObject.hxx>
#include "DepthEdgeMergeFeatures.h"

template <typename T>
DepthEdgeMergeFeatures<T>::DepthEdgeMergeFeatures(const std::string& name) :
		TemplatedParameteredObject<T>(
			"DepthEdgeMergeFeatures", name,
			"<h2>Merges two CImgLists containing depth edge "
			"features</h2><br>"
			"Merges two CImgLists containing depth edge features"
		)
{	
	ParameteredObject::_addInputSlot(
		features1, "features1",
		"First feature list to merge", "vector<string>");
	ParameteredObject::_addInputSlot(
		features2, "features2",
		"Second feature list to merge", "vector<string>");

	ParameteredObject::_addOutputSlot(
		merged, "merged",
		"Second feature list to merge", "vector<CImgList>");

	ParameteredObject::_setTags("Hekate;Edges") ;
}

template <typename T>
void DepthEdgeMergeFeatures<T>::execute() {
	
	if( this->features1().size() != this->features2().size() )
		throw( std::range_error("DepthEdgeMergeFeatures<T>::execute(): features1.size != features2.size") );

	for( int i = 0; i < this->features1().size(); i++ ){
	

		CImgList< T > feat1( this->features1()[ i ].c_str() );
		CImgList< T > feat2( this->features2()[ i ].c_str() );
		feat1.push_back( feat2 );

		this->merged().push_back( feat1 );
	
	}

}

#endif /* _DEPTHEDGEMERGEFEATURES_HXX_ */
