#ifndef _DEPTHEDGEMODEL_HXX_
#define _DEPTHEDGEMODEL_HXX_

#include <vigra/windows.h>
#include <vigra/hdf5impex.hxx>
#include <vigra/random_forest.hxx>
#include <vigra/random_forest_hdf5_impex.hxx>

using namespace vigra;
using namespace vigra::detail;
using namespace rf;

typedef std::vector< vigra::RandomForest<int> >	RF_list;
typedef std::vector< int >						RF_ids;

namespace hekate {

	/// Container class which includes one ore more random forests, its image ids, 
	class DepthEdgeModel
	{
	public:

		/// List of random forests imgId( rfId( rfFileName ) )
		RF_list randomForests;

		/// image ids of the used images
		RF_ids imageIds;

		/// default constructor
		explicit DepthEdgeModel(){}

		/// default constructor
		explicit DepthEdgeModel( RF_list randomForests){}

		/// default constructor
		explicit DepthEdgeModel( RF_list randomForests, RF_ids imageIds)
		{
			randomForests = randomForests;
			imageIds = imageIds;
		}
			
		virtual ~DepthEdgeModel() {}

		void clear()
		{
			randomForests.clear();
			imageIds.clear();
		}

		/// Save model with index. 
		/** If index already exists, overwrite old model. */
		void saveModel( const int index, const vigra::RandomForest<int> &rf ){

			int position;
			if( exists(index,position) ){
				sout << "A model for frame " << index << " already exists "
					<< "at position " << position << std::endl;
				sout << "Overwriting model with new one..." << std::endl;
				randomForests[position] = rf;
				return;
			}

			randomForests.push_back( rf );
			imageIds.push_back( index );
			sout << "The random forest for frame " << index << " was saved to model list" << std::endl;

		}

		/// Checks if a randon forest with index is available and return position of entry
		bool exists( const int index, int &position ) const{

			if( !imageIds.size() )
				return false;

			for( int i = 0; i < imageIds.size(); i++ ){
				if( imageIds[i] == index ){
					position = i;
					return 1;
				}
			}

			return 0;
		}

		/// Checks if a randon forest with index is available and return position of entry
		bool exists( const int index ) const{
			int position;
			return exists( index, position );
		}

	} ;



} ; /*namespace hekate */


#endif /* #ifndef _DEPTHEDGEMODEL_HXX_ */