/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file DepthEdgeWarp.h
 *  Declaration of the parameter class DepthEdgeWarp.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 20.11.2013
 */

#ifndef _DEPTHEDGEWARP_H_
#define _DEPTHEDGEWARP_H_

#ifdef _MSC_VER
#ifdef depthedgewarp_EXPORTS
/// Visual C++ specific code
#define depthedgewarp_DECLDIR __declspec(dllexport)
#else
#define depthedgewarp_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define depthedgewarp_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Warp ground truthmasks foward and backward to get a warped mask image for
/// every frame
/** Warp ground truthmasks foward and backward to get a warped mask image for
 *  every frame
 */
template <typename T>
class depthedgewarp_DECLDIR DepthEdgeWarp :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	DepthEdgeWarp(const std::string& name = "");

	/// Depth Edge Masks which should be warped to the other frames
	InputSlot< cimg_library::CImgList<T> > masks;
	/// Ids of the given masks
	InputSlot< std::vector<int> > ids;
	/// Flow to warp masks. CImgList dimension != 2 for x and y flow and the
	/// channel dimension contains the different frame values.
	InputSlot< cimg_library::CImgList<T> > flow;

	/// Warped masks
	OutputSlot< cimg_library::CImgList<T> > warped;


protected:
	/// Update object.
	virtual void execute();
	
	/// Get ground truth mask next to this frame.
	/** id_sequence is the id according to the sequence size.
	*   id_vector is the according entry of this->ids() and this->masks().
	*/
	void getNextMaskId(const int index, int &id_sequence, int &id_vector);

	/// Check if current reference frame did not change
	bool checkReference( const int id, const int id_reference );
	
	/// Warp frame of id-1 to frame id using forward flow if forward == true and otherwise fram id+1
	void getWarp(const int id, const bool forward);

};

#endif // _DEPTHEDGEWARP_H_
