/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file DepthEdgesRFReader.h
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 16.07.2013
 *  Declaraion of parameter class DepthEdgesRFReader.
 */
#ifndef _DEPTHEDGESRFREADER_H_
#define _DEPTHEDGESRFREADER_H_

#ifdef _MSC_VER
#ifdef depthedgesrfreader_EXPORTS
/// Visual C++ specific code
#define depthedgesrfreader_DECLDIR __declspec(dllexport)
#else
#define depthedgesrfreader_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define depthedgesrfreader_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>

#ifdef _MSC_VER
# pragma warning (disable : 4244)
#endif

#include <vigra/windows.h>
#include <vigra/random_forest.hxx>
#include <vigra/random_forest_deprec.hxx>
#include <vigra/multi_math.hxx>
#include <vigra/hdf5impex.hxx>
#include <vigra/random_forest.hxx>
#include <vigra/random_forest_hdf5_impex.hxx>

#include <hekate/DepthEdgeModel.hxx>

using namespace vigra;
using namespace vigra::detail;
using namespace rf;

/// Reads in random forests saved by DepthEdgesRFWriter
/** Reads in a depth edge random forest project including a index.txt file and
 *  hdf5 random forest files.
 *  The index file thereby contains the hdf5 file names, its corresponding
 *  frame ids and also the coordinates of corresponding image patches.
 */
class depthedgesrfreader_DECLDIR DepthEdgesRFReader : public ParameteredObject {
public:
	/// default constructor
	/// \param name             instance name
	DepthEdgesRFReader(const std::string& name = "");

	/// The depth edge model which keeps random forests, its patch coordinates
	///	and its ids and the feature setup which was chosen at learning runtime
	OutputSlot< hekate::DepthEdgeModel > depthEdgeModel;

	/// Path to the index file of the random forests
	Parameter< std::string > filename;


protected:
	/// Update object.
	virtual void execute();
};

#endif /* _DEPTHEDGESRFREADER_H_ */
