/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file DepthEdgesRFWriter.h
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 08.07.2013
 *  Declaraion of parameter class DepthEdgesRFWriter.
 */
#ifndef _DEPTHEDGESRFWRITER_H_
#define _DEPTHEDGESRFWRITER_H_

#ifdef _MSC_VER
#ifdef depthedgesrfwriter_EXPORTS
/// Visual C++ specific code
#define depthedgesrfwriter_DECLDIR __declspec(dllexport)
#else
#define depthedgesrfwriter_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define depthedgesrfwriter_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>

#include <vigra/windows.h>
#include <vigra/random_forest.hxx>
#include <vigra/random_forest_deprec.hxx>
#include <vigra/multi_math.hxx>
#include <vigra/hdf5impex.hxx>
#include <vigra/random_forest.hxx>
#include <vigra/random_forest_hdf5_impex.hxx>

#include <hekate/DepthEdgeModel.hxx>

using namespace vigra;
using namespace vigra::detail;
using namespace rf;

/// Writes random forests learned by DepthEdgesLearn to a local file
/** Saves random forests and corresponding image ids to a seperate file
 */
class depthedgesrfwriter_DECLDIR DepthEdgesRFWriter : public ParameteredObject {
public:
	/// default constructor
	/// \param name             instance name
	DepthEdgesRFWriter(const std::string& name = "");

	/// The depth edge model which keeps random forests, its patch coordinates
	///	and its ids and the feature setup which was chosen at learning runtime
	InputSlot< hekate::DepthEdgeModel > depthEdgeModel;

	/// Path to write files to.
	Parameter< std::string > filepath;
	/// Delete old HDF5 file
	Parameter< bool > deleteHDF5;


protected:
	/// Update object.
	virtual void execute();
};

#endif /* _DEPTHEDGESRFWRITER_H_ */
