/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file DepthMapFrom3DPointCloud.h
 *  Declaration of the parameter class DepthMapFrom3DPointCloud.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 06.03.2013
 */

#ifndef _DEPTHMAPFROM3DPOINTCLOUD_H_
#define _DEPTHMAPFROM3DPOINTCLOUD_H_

#ifdef _MSC_VER
#ifdef depthmapfrom3dpointcloud_EXPORTS
/// Visual C++ specific code
#define depthmapfrom3dpointcloud_DECLDIR __declspec(dllexport)
#else
#define depthmapfrom3dpointcloud_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define depthmapfrom3dpointcloud_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>

using namespace hekate;
using namespace cimg_library;

/// Creates a depth map from a 3d point cloud and given cameras
/** Creates a depth map from a 3d point cloud and given cameras
 */
template <typename T>
class depthmapfrom3dpointcloud_DECLDIR DepthMapFrom3DPointCloud :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	DepthMapFrom3DPointCloud(const std::string& name = "");

	/// 3D point cloud to calculate depths of points
	InputSlot< CorrespondenceList<T> > pointCloud;
	/// 2D points corresponding with the 3D point cloud. The depth map is
	/// calculated by the 2d position and the 3d depth.
	InputSlot< CorrespondenceList<T> > correspondences;
	/// Camera matrices
	InputSlot< IndexedMatrixList > cameras;
	
	/// Calculated depth maps vizualised by pixel values of [0,255]
	OutputSlot< cimg_library::CImgList<T> > depthMaps;
	
	/// Width of back projected depth map
	Parameter< int > width;
	/// Height of back projected depth map
	Parameter< int > height;
	/// Number of maximal images to backproject
	Parameter< int > range;
	/// Depth map color of average depth
	Parameter< int > averageDepthColor;
	/// Background color if not every pixel has a depth
	Parameter< int > backgroundColor;
	/// Amount of putative inliers
	Parameter< float > confidence;
	/// Sort out farest points by confidence level to prevent outliers
	Parameter< bool > sortOutFarestPoints;
	/// Sort out nearest points by confidence level to prevent outliers
	Parameter< bool > sortOutNearestPoints;
	/// Back projects all 3D points without checking for occlusions
	Parameter< bool > useAllPoints;
	/// Back projects 3D points from neighbouring frames without checking for occlusions
	Parameter< bool > useNeighbourFramePoints;

protected:
	/// Update object.
	virtual void execute();
	
	/// Calculate depth between x and principle plane of P.
	double getDepth(const CImg<T> &P, const CImg<T> &X);
};

#endif // _DEPTHMAPFROM3DPOINTCLOUD_H_
