/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file DepthMapProjection.h
 *  Declaration of the parameter class DepthMapProjection.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 08.04.2013
 */

#ifndef _DEPTHMAPPROJECTION_H_
#define _DEPTHMAPPROJECTION_H_

#ifdef _MSC_VER
#ifdef depthmapprojection_EXPORTS
/// Visual C++ specific code
#define depthmapprojection_DECLDIR __declspec(dllexport)
#else
#define depthmapprojection_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define depthmapprojection_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>

/// Projects a depth map with given cameras
/** Projects a depth map with given cameras
 */
template <typename T>
class depthmapprojection_DECLDIR DepthMapProjection :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	DepthMapProjection(const std::string& name = "");

	/// DepthMaps given as CImgList
	InputSlot< cimg_library::CImgList<T> > depthMaps;
	/// Cameras according to depth maps
	InputSlot< IndexedMatrixList > cameras;

	/// Projected point cloud
	OutputSlot< hekate::CorrespondenceList<T> > pointCloud;
	
	/// Mapping factor
	Parameter < float > mappingFactor;
	/// Step size for pixel selection. Choose a higher value for 
	/// a sparse point cloud and faster processing time
	Parameter < int > stepSize;

protected:
	/// Update object.
	virtual void execute();
};

#endif // _DEPTHMAPPROJECTION_H_
