/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file DepthTransformer.h
 *  Declaration of the parameter class DepthTransformer.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 06.11.2012
 */

#ifndef _DEPTHTRANSFORMER_H_
#define _DEPTHTRANSFORMER_H_

#ifdef _MSC_VER
#ifdef depthtransformer_EXPORTS
/// Visual C++ specific code
#define depthtransformer_DECLDIR __declspec(dllexport)
#else
#define depthtransformer_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define depthtransformer_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>

using namespace hekate;
using namespace cimg_library;

/// Transforms Depth Map by Optical Flow and Match Move
/** Transforms Depth Map by Optical Flow and Match Move
 */
template <typename T>
class depthtransformer_DECLDIR DepthTransformer :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	DepthTransformer(const std::string& name = "");

	/// 2d corrs
	InputSlot< CorrespondenceList<T> > corrs;
	/// 3d
	InputSlot< CorrespondenceList<T> > points3d;
	/// cameras
	InputSlot< IndexedMatrixList > cameras;

	/// 3d out
	OutputSlot< CorrespondenceList<T> > points3dOut;

	/// Update object.
	virtual void execute();
};

#endif // _DEPTHTRANSFORMER_H_
