/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file DepthTransformer.hxx
 *  Implementation of the parameter class DepthTransformer.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 06.11.2012
 */

#ifndef _DEPTHTRANSFORMER_HXX_
#define _DEPTHTRANSFORMER_HXX_

#include <charon-core/ParameteredObject.hxx>
#include <hekate/DepthTransformer.h>

template <typename T>
DepthTransformer<T>::DepthTransformer(const std::string& name) :
		TemplatedParameteredObject<T>(
			"DepthTransformer", name,
			"<h2>Transforms Depth Map by Optical Flow and Match "
			"Move</h2><br>"
			"Transforms Depth Map by Optical Flow and Match Move"
		)
{
	ParameteredObject::_addInputSlot(
		corrs, "corrs",
		"2d corrs",
		"CorrespondenceList");
	ParameteredObject::_addInputSlot(
		points3d, "points3d",
		"3d",
		"CorrespondenceList");
	ParameteredObject::_addInputSlot(
		cameras, "cameras",
		"cameras",
		"IndexedMatrixList");

	ParameteredObject::_addOutputSlot(
		points3dOut, "points3dOut",
		"3d out",
		"CorrespondenceList");

}

template <typename T>
void DepthTransformer<T>::execute() {
	PARAMETEREDOBJECT_AVOID_REEXECUTION;
	ParameteredObject::execute();

	/*

	x' = P' X'
	x = P X

	x' = H1 x
	X' = H2 X
	
	x' = P' H2 X

	H1 x = H1 P X = P' X' = P' H2 X  <==>  H1 P = P' H2
	P = H1 P' H2  :=  b = A Y

	H1 P' := P''  -->  P = P'' H2



	*/

}

#endif /* _DEPTHTRANSFORMER_HXX_ */
