/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file DynamicSuperpixels.h
 *  Declaration of the parameter class DynamicSuperpixels.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 04.12.2012
 */

#ifndef _DYNAMICSUPERPIXELS_H_
#define _DYNAMICSUPERPIXELS_H_

#ifdef _MSC_VER
#ifdef dynamicsuperpixels_EXPORTS
/// Visual C++ specific code
#define dynamicsuperpixels_DECLDIR __declspec(dllexport)
#else
#define dynamicsuperpixels_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define dynamicsuperpixels_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Calculates segmentation of an image sequence
/** This module is kind of a wrapper of "DynamicSuperpixels" from Goethe
 *  University, Frankfurt (http://www.vsi.cs.uni-frankfurt.de/). It calculates
 *  segmentations of an image sequence and optionally propagates subsequent
 *  segmentations, so the segmentation of frame 1 is used as initial guess for
 *  frame 2.
 */
template <typename T>
class dynamicsuperpixels_DECLDIR DynamicSuperpixels :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	DynamicSuperpixels(const std::string& name = "");

	/// Input image
	InputSlot< std::vector< std::string > > imagePath;
	
	/// segmentation label ids
	OutputSlot< cimg_library::CImgList<T> > labels;
	/// Segmented images
	OutputSlot< cimg_library::CImgList<T> > segmentation;
	/// countour
	OutputSlot< cimg_library::CImgList<T> > countour;
	/// countourOverlay
	OutputSlot< cimg_library::CImgList<T> > countourOverlay;

	/// Use subsequent frames to propagate segmentations
	Parameter< bool > propagateSegmentation;
	/// Widths of blocks used for initial segments
	Parameter< int > blockWidth;
	/// Heights of blocks used for initial segments
	Parameter< int > blockHeight;
	/// Markov clique cost for horizontal and vertical cliques
	Parameter< double > directCliqueCost;
	/// Weight for the compactness feature, optional, default value = 0.045
	/// for color images, 0.015 for grayscale images
	Parameter< double > compactnessWeight;


protected:
	/// Update object.
	virtual void execute();
};

#endif // _DYNAMICSUPERPIXELS_H_
