/*  Copyright (C) 2014 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file EigenValuesSchaefer.h
 *  Declaration of the parameter class EigenValuesSchaefer.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 11.03.2014
 */

#ifndef _EIGENVALUESSCHAEFER_H_
#define _EIGENVALUESSCHAEFER_H_

#ifdef _MSC_VER
#ifdef eigenvaluesschaefer_EXPORTS
/// Visual C++ specific code
#define eigenvaluesschaefer_DECLDIR __declspec(dllexport)
#else
#define eigenvaluesschaefer_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define eigenvaluesschaefer_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <math.h>
#include <ImgTool.hxx>

using namespace cimg_library;

/// An eigenvalue of structure tensor implementation
/** An eigenvalue of structure tensor implementation originally written by
 *  Henrik Schfer from HCI, Heidelberg, Germany.
 */
template <typename T>
class eigenvaluesschaefer_DECLDIR EigenValuesSchaefer :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	EigenValuesSchaefer(const std::string& name = "");

	/// Images to process
	InputSlot< cimg_library::CImgList<T> > images;

	/// Eigenvalues saved in z-component
	OutputSlot< cimg_library::CImgList<T> > eigenvalues;
	/// Draws the scale space of maximal eigen values for each pixel
	OutputSlot< cimg_library::CImgList<T> > eigenvaluesScaleSpaces;
	/// Multiplication of eigenvalue 1 and 2
	OutputSlot< cimg_library::CImgList<T> > eigenvaluesMul;
	/// Eigenvector perpendicular to an edge. Components saved in Z-Axis.
	OutputSlot< cimg_library::CImgList<T> > eigenvector1;
	/// Eigenvector parallel to an edge. Components saved in Z-Axis.
	OutputSlot< cimg_library::CImgList<T> > eigenvector2;
	/// Angle perpendicular to an edge
	OutputSlot< cimg_library::CImgList<T> > angle;
	
	/// Gaussian variance for gradient estimation.
	Parameter< float > gradientSigma;
	/// Gaussian variance for gradient estimation in edge direction.
	Parameter< float > gradientSigma2;
	/// Width of the gradient filter.
	Parameter< int > gradientFilterWidth;
	/// Gaussian variance for structure tensor estimation.
	Parameter< float > structureTensorSigma1;
	/// Second gaussian variance for structure tensor estimation. This one is
	/// used, if the filter is weighted for a specific direction. In this
	/// case, sigma2 is used to smooth in gradient direction while sigma1
	/// smooths perpendicular to gradient direction
	Parameter< float > structureTensorSigma2;
	/// Width of the gaussian filter for structure tensor estimation
	Parameter< int > structureTensorWidth;
	/// Numer of scale spaces for which, eigen values are estimated. While the
	/// results of the output slot rely on the given parameters above, this
	/// value can be used to evaluate correlations between eigen values of
	/// different scale spaces.
	Parameter< int > scaleSpaces;
	/// Activate evaluation using the parameter scaleSpaces for different
	/// scale spaces.
	Parameter< bool > evaluateScaleSpaces;
	/// Use oriented filters determined by initial structure tensor values
	Parameter< bool > useOrientedFilters;


protected:
	/// Update object.
	virtual void execute();
};

#endif // _EIGENVALUESSCHAEFER_H_
