/*  Copyright (C) 2009 SasiKanth

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file epipolarrectification.h
 *  Declaration of the parameter class epipolarrectification.
 *  \author SasiKanth
 *  \date 18.06.2010
 */

#ifndef _EPIPOLARRECTIFICATION_H_
#define _EPIPOLARRECTIFICATION_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef epipolarrectification_EXPORTS
/// Visual C++ specific code
#define epipolarrectification_DECLDIR __declspec(dllexport)
#else
#define epipolarrectification_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define epipolarrectification_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
/// This plugin warps the images taking the fundamental matix as the input so that, the epipolar lines in both the cases become horizontal and Images are effectively rectified (Hartley pg. 304-307)
template <typename T>
class epipolarrectification_DECLDIR epipolarrectification :
		public TemplatedParameteredObject<T> {
public:
	/// The threshold for confident inlier determination
	Parameter < double > inlier_confidence;
	

	

	/// The set of input images 
	InputSlot < cimg_library::CImgList<T> > Input_Images;
	/// The fundamental matrix 
	InputSlot < cimg_library::CImgList<T> > fmatrix;
	// The correspondences input
	InputSlot < std::vector<std::vector<std::pair<T,T > > > > correspondences_input;
	/// The warping matrices for the two images 
	OutputSlot < cimg_library::CImgList<T> > Warping_Matrices;
	

	/// create a new epipolarrectification object
	/// \param name          Instance name
	epipolarrectification(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _EPIPOLARRECTIFICATION_H_



