/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Equalize.h
 *  Declaration of the parameter class Equalize.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 16.10.2013
 */

#ifndef _EQUALIZE_H_
#define _EQUALIZE_H_

#ifdef _MSC_VER
#ifdef equalize_EXPORTS
/// Visual C++ specific code
#define equalize_DECLDIR __declspec(dllexport)
#else
#define equalize_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define equalize_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Equalize function of CIMG
/** Equalize function of CIMG
 */
template <typename T>
class equalize_DECLDIR Equalize :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	Equalize(const std::string& name = "");

	/// images to equalize
	InputSlot< cimg_library::CImgList<T> > images;

	/// Equalized images
	OutputSlot< cimg_library::CImgList<T> > out;

	/// Number of pixels to consider for equalization
	Parameter< float > pixels;


protected:
	/// Update object.
	virtual void execute();
};

#endif // _EQUALIZE_H_
