/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Equalize.hxx
 *  Implementation of the parameter class Equalize.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 16.10.2013
 */

#ifndef _EQUALIZE_HXX_
#define _EQUALIZE_HXX_

#include <charon-core/ParameteredObject.hxx>
#include "Equalize.h"

template <typename T>
Equalize<T>::Equalize(const std::string& name) :
		TemplatedParameteredObject<T>(
			"Equalize", name,
			"<h2>Equalize function of CIMG</h2><br>"
			"Equalize function of CIMG"
		)
{
	ParameteredObject::_addInputSlot(
		images, "images",
		"images to equalize",
		"CImgList<T>");

	ParameteredObject::_addOutputSlot(
		out, "out",
		"Equalized images",
		"CImgList<T>");

	ParameteredObject::_addParameter< float >(
		pixels, "percentageOfPixels",
		"Number of pixels to consider for equalization",
		1, "float");

}

template <typename T>
void Equalize<T>::execute() {
	
	this->out() = this->images();
	for( size_t i = 0; i < this->images().size(); i++ )
	{
		
		int levels = out()[i].max() - out()[i].min();
		int first=0, last=0;
		if( pixels() < 1 && pixels() > 0 ){
			int sum = out()[i].width() * out()[i].height();
			cimg_library::CImg<float> hist = out()[i].get_histogram(256);
			// get lower value
			for(size_t j = 0; j < hist.width(); j++){
				first += hist(j);
				if(first > pixels() * sum){
					first = j - 1;
					break;
				}
			}
			// get upper value
			for(size_t j = hist.width()-1; j >= 0; j--){
				last += hist(j);
				if(last > pixels() * sum){
					last = j + 1;
					break;
				}
			}
			//levels = last-first;
			sout << "Taking lower value: " << first << std::endl;
			sout << "Taking upper value: " << last << std::endl;
		}

		//out()[i].cut(20,245);
		out()[i].equalize(255,0,255);//,first,last);
	}

}

#endif /* _EQUALIZE_HXX_ */
