/*  Copyright (C) 2011 Stephan Meister

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file EstimateCamMatrixOCV.h
 *  Declaration of the parameter class EstimateCamMatrixOCV.
 *  \author <a href="mailto:stephan.meister@iwr.uni-heidelberg.de">
 *      Stephan Meister</a>
 *  \date 31.10.2011
 */

#ifndef _ESTIMATECAMMATRIXOCV_H_
#define _ESTIMATECAMMATRIXOCV_H_

#ifdef _MSC_VER
#ifdef estimatecammatrixocv_EXPORTS
/// Visual C++ specific code
#define estimatecammatrixocv_DECLDIR __declspec(dllexport)
#else
#define estimatecammatrixocv_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define estimatecammatrixocv_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <vector>

/// Estimate internal camera matrix from several images of checkerboard
/// pattern using OpenCV
/** see above
 */
template <typename T>
class estimatecammatrixocv_DECLDIR EstimateCamMatrixOCV :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	EstimateCamMatrixOCV(const std::string& name = "");

	InputSlot< cimg_library::CImgList<T> > initialCamMatrix ;
	
	/// cameraMatrix
	OutputSlot< cimg_library::CImgList<T> > camMatrix;
	/// distortion coefficients
	OutputSlot< cimg_library::CImgList<T> > distortionCoeffs;
	/// input filenames
	ParameterList< std::string > filenames ;

	/// number of tiles in x direction
	Parameter< int > gridRangeX;
	/// number of tiles in y direction
	Parameter< int > gridRangeY;
	/// distance between checkerboard tiles
	Parameter< double > gridDist;


	/// type of grid
	Parameter< std::string > gridType ;
	/// verbose console output
	Parameter < bool > verbose ;

	/// try to find grid with switched gridRangeX/Y Parameters
	Parameter < bool > tryRotatedGrid ;

	Parameter < bool > aggresiveFeatureSearch ;

	/// Update object.
	virtual void execute();
};

#endif // _ESTIMATECAMMATRIXOCV_H_
