/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file EvaluationCorrespondences3D.h
 *  Declaration of the parameter class EvaluationCorrespondences3D.
 *  \author Moritz Becker
 *  \date 16.02.2011
 */

#ifndef _EVALUATIONCORRESPONDENCES3D_H_
#define _EVALUATIONCORRESPONDENCES3D_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef evaluationcorrespondences3d_EXPORTS
/// Visual C++ specific code
#define evaluationcorrespondences3d_DECLDIR __declspec(dllexport)
#else
#define evaluationcorrespondences3d_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define evaluationcorrespondences3d_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Evaluates distance between 3D correspondences.
template <typename T>
class evaluationcorrespondences3d_DECLDIR EvaluationCorrespondences3D :
		public TemplatedParameteredObject<T> {
public:
	
	
    /// Filename to write evaluation results to.
    Parameter<std::string> filename;
    /// filename to write point evaluation results to
    Parameter<std::string> filename2;
	/// Gives out an extended log with point information.
	Parameter<bool> extendedLog ;

	/// Evaluation 3D point cloud.
	InputSlot < std::vector<std::vector<T> > > points3D1;
	/// Reference 3D point cloud. These points are assumed to be without errors.
	InputSlot < std::vector<std::vector<T> > > points3D2;

	/// Evaluation 2D point cloud.
	InputSlot < std::vector<std::vector<std::vector<T> > > > points2D1;
	/// Reference 2D point cloud. These points are assumed to be without errors.
	InputSlot < std::vector<std::vector<std::vector<T> > > > points2D2;
	

	/// create a new EvaluationCorrespondences3D object
	/// \param name          Instance name
	EvaluationCorrespondences3D(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _EVALUATIONCORRESPONDENCES3D_H_



