/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file ExifReader.h
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 13.07.2012
 *  Declaraion of parameter class ExifReader.
 */
#ifndef _EXIFREADER_H_
#define _EXIFREADER_H_

#ifdef _MSC_VER
#ifdef exifreader_EXPORTS
/// Visual C++ specific code
#define exifreader_DECLDIR __declspec(dllexport)
#else
#define exifreader_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define exifreader_DECLDIR
#endif

#include <CImg.h>
#include <charon-core/ParameteredObject.h>

/// Reads EXIF data from images
/** The plugin takes a list of file paths and reads out the according exif
 *  data.
 */
class exifreader_DECLDIR ExifReader : public ParameteredObject {
public:
	/// default constructor
	/// \param name             instance name
	ExifReader(const std::string& name = "");

	/// Paths to the images
	InputSlot< std::vector<std::string> > imagePaths;

	/// Exif data
	/** The inner vector contains: width, height, focalLength(mm), sensor resolution in x direction(px/mm),
	* sensor resolution in y direction(px/mm)
	*
	*/
	OutputSlot< std::vector<std::vector<float> > > exifData;

	/// Calibration matrices calculated by EXIF data
	OutputSlot< cimg_library::CImgList<double> > calibration;

	/// Update object.
	virtual void execute();
};

#endif /* _EXIFREADER_H_ */
