/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file ExtendSparseEdges.h
 *  Declaration of the parameter class ExtendSparseEdges.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 19.10.2012
 */

#ifndef _EXTENDSPARSEEDGES_H_
#define _EXTENDSPARSEEDGES_H_

#ifdef _MSC_VER
#ifdef extendsparseedges_EXPORTS
/// Visual C++ specific code
#define extendsparseedges_DECLDIR __declspec(dllexport)
#else
#define extendsparseedges_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define extendsparseedges_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

using namespace cimg_library;

/// Extends sparse edges by a given dense edge image
/** Extends edges of source image by additional edges given by second image.
 *  This is primarily used to densify sparse depth edges obtained by
 *  LearnDepthEdgeByMask. If a given percentage (threshold1) of sparse edges
 *  ly on an edge provided by the second edge image, the sparse edge is
 *  extended by the provided edge.
 *  The plugin can be used to only refine specific edges.
 */
template <typename T>
class extendsparseedges_DECLDIR ExtendSparseEdges :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	ExtendSparseEdges(const std::string& name = "");

	/// Image of edges which should be extended
	InputSlot< cimg_library::CImgList<T> > sourceEdges;
	/// Image of edges used to extend source edges
	InputSlot< cimg_library::CImgList<T> > extensionEdges;

	/// Extended edges
	OutputSlot< cimg_library::CImgList<T> > extendedEdges;

	/// Lower threshold for source pixel values to belong to an putative edge
	Parameter< int > iterations;

	/// Lower threshold for source pixel values to belong to an putative edge
	Parameter< float > threshold;

	/// Threshold which defines source pixels to be edges
	Parameter< float > threshold2;

	/// How much percent of source pixels have to ly on a neighbouring edge, if pixel should be extended
	Parameter< float > intersection;

	/// Tolerance for edge segments
	Parameter< float > tolerance;

	/// Scale factor for sub segments 
	/** All connected segments/edges are
	*	divided to smaller segments for better matches between sparse
	*	predicted edges and the dense extension edges. If this parameter
	*	is 1.0, no subsegments are generated.
	*/
	Parameter< float > subSegmentScale;
	
	/// Minimum sub edge length
	Parameter< int > minSubEdgeLength;

	/// Update object.
	virtual void execute();


};

#endif // _EXTENDSPARSEEDGES_H_
