/*  Copyright (C) 2009 Julian Coordts

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file FileSequenceReader.h
 *  Declaration of the parameter class FileSequenceReader.
 *  \author Julian Coordts
 *  \date 13.04.2010
 */

#ifndef _FILESEQUENCEREADER_H_
#define _FILESEQUENCEREADER_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef filesequencereader_EXPORTS
/// Visual C++ specific code
#define filesequencereader_DECLDIR __declspec(dllexport)
#else
#define filesequencereader_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define filesequencereader_DECLDIR
#endif

#include "FileSequenceReaderInterface.h"
//#include "PluginGui.h"

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include <iostream>
#include <string>
#include <stdio.h>

/// Reads a selected pair of images of a sequence of right-left files.
/** To select them a slider has been build in.
 */
template <typename T>
class filesequencereader_DECLDIR FileSequenceReader :
		public TemplatedParameteredObject<T>,
		public FileSequenceReaderInterface {
private:
		///the gui
        PluginGui* _userInterface;

public:
	/// the selected image pair(left first, second 
	OutputSlot < cimg_library::CImgList<T> > current_image_sequence;

	///default path for right images
	Parameter<std::string> right_path;
	///default path for left images
	Parameter<std::string> left_path;
	///default name for right images
	Parameter<std::string> right_name;
	///default name for left images
	Parameter<std::string> left_name;
	///allowed size of number places in filename
	Parameter<int> number_places_count;
	/// create a new FileSequenceReader object
	/// \param name          Instance name
	FileSequenceReader(const std::string& name);

	///standart Destructor
	virtual ~FileSequenceReader();

	///reexecutes the workflow with the choosen image pair
	virtual void sendNewImagePair(
			std::string fileLeft,
			std::string fileRight);

	/// Update object.
	virtual void execute();
};

#endif // _FILESEQUENCEREADER_H_



