/*  Copyright (C) 2009 Julian Coordts

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file FileSequenceReader.hxx
 *  Implementation of the parameter class FileSequenceReader.
 *  \author Julian Coordts
 *  \date 13.04.2010
 */

#ifndef _FILESEQUENCEREADER_HXX_
#define _FILESEQUENCEREADER_HXX_

#include "FileSequenceReader.h"

template <typename T>
FileSequenceReader<T>::FileSequenceReader(const std::string& name) :
		TemplatedParameteredObject<T>("FileSequenceReader", name,
			"Reads a selected pair of images of a sequence of right-left "
			"files. To select them a slider has been build in."),
		_userInterface(0)
{

	ParameteredObject::_addParameter (right_path, "right_path",
			"Path to right image sequence", "path");
	ParameteredObject::_addParameter (left_path, "left_path",
			"Path to left image sequence", "path");
	ParameteredObject::_addParameter (right_name, "right_name",
			"template of right images");
	ParameteredObject::_addParameter (left_name, "left_name",
			"template of left images");
	ParameteredObject::_addParameter (number_places_count, "number_places_count",
			"count of number places", 4);


	ParameteredObject::_addOutputSlot(current_image_sequence,
		"current_image_sequence",
		"the selected image pair(left first, second", "CImgList<T>");
}

template <typename T>
void FileSequenceReader<T>::execute() {

	if (!this->executed())
	{
		ParameteredObject::execute();

		if (!_userInterface)
			_userInterface= new PluginGui(this,
										  this->right_path(),
										  this->left_path(),
										  this->right_name(),
										  this->left_name(),
										  this->number_places_count());

		current_image_sequence().assign(1,1,1);
	}
}

template <typename T>
void FileSequenceReader<T>::sendNewImagePair(std::string fileLeft,
											 std::string fileRight)
{
	current_image_sequence().clear();
	current_image_sequence().assign(2);
	sout << "execute with image left: " << fileLeft << std::endl <<
			"file right: " << fileRight << std::endl;
	current_image_sequence()[0].load(fileLeft.c_str());
	current_image_sequence()[1].load(fileRight.c_str());


	//execute again!!!
	const std::set<ParameteredObject*>& targetNodes=this->_getTargetNodes();
	std::set<ParameteredObject*>::const_iterator it;



	 for (it = 	targetNodes.begin();
		  it != targetNodes.end(); it++ )
	 {
		 ParameteredObject* curPo= *it;
		if (curPo->getName() != this->getName())
		 {
			sout << "recall: " << curPo->getName();
			curPo->execute();
		}
	}




}

template <typename T>
FileSequenceReader<T>::~FileSequenceReader() {
	if (_userInterface)
		delete _userInterface;
}

#endif /* _FILESEQUENCEREADER_HXX_ */
