/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file filewriterWRL.h
 *  Declaration of the parameter class filewriterWRL.
 *  \author Moritz Becker
 *  \date 09.06.2010
 */

#ifndef _FILEWRITERWRL_H_
#define _FILEWRITERWRL_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef filewriterwrl_EXPORTS
/// Visual C++ specific code
#define filewriterwrl_DECLDIR __declspec(dllexport)
#else
#define filewriterwrl_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define filewriterwrl_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>

using namespace hekate;

/// Saves *.wrl files for 3D point clouds (e.g. can be opened with blender)
template <typename T>
class filewriterwrl_DECLDIR filewriterWRL :
		public TemplatedParameteredObject<T> {
public:
	
    /// filename to write point cloud to
    Parameter<std::string> filename;
    /// X position of virtual camera
    Parameter<float> camX;
    /// Y position of virtual camera
    Parameter<float> camY;
    /// Z position of virtual camera
    Parameter<float> camZ;
	/// X coordinate of camera rotation axis
	Parameter<float> viewRotAxisX ;
	/// Y coordinate of camera rotation axis
	Parameter<float> viewRotAxisY ;
	/// Z coordinate of camera rotation axis
	Parameter<float> viewRotAxisZ ;
	/// angle of camera rotation [rad]
	Parameter<float> viewRotAngle ;
	
	/// camera Field-Of-View [rad]
	Parameter<float> viewFov ;
	/// withFaceSet
	Parameter<bool> withFaceSet ;

	/// camera matrices
	InputSlot < IndexedMatrixList > cameraMatrices;
	/// calibration matrix
	InputSlot < cimg_library::CImgList<T> > calibrationMatrix;
	/// point_index to connect points via lines 
	InputSlot < std::vector<std::vector<int> > > pointIndex;
	/// point_cloud 
	InputSlot < typename hekate::CorrespondenceList<T> > pointCloud;
	/// colors according to index
	//InputSlot < std::vector<std::vector<T> > > colors;
	/// image 
	InputSlot < std::vector< std::string > > imagePath;
	InputSlot < std::vector<std::vector<T> > > pointCloud2D;
	

	/// create a new filewriterWRL object
	/// \param name          Instance name
	filewriterWRL(const std::string& name = "");

	/// Update object.
	virtual void execute();

private:

	static const float _defaultColorR ;
	static const float _defaultColorG ;
	static const float _defaultColorB ;
};

#endif // _FILEWRITERWRL_H_



