/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file filewriterWRL.h
 *  Declaration of the parameter class filewriterWRL.
 *  \author Moritz Becker
 *  \date 09.06.2010
 */

#ifndef _FILEWRITERWRLHOMEST_H_
#define _FILEWRITERWRLHOMEST_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef filewriterwrlhomest_EXPORTS
/// Visual C++ specific code
#define filewriterwrlhomest_DECLDIR __declspec(dllexport)
#else
#define filewriterwrlhomest_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define filewriterwrlhomest_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Saves *.wrl files for 3D point cloud correspondences (e.g. can be opened with blender)
/** Generates a *.wrl file with lines between the input correspondences. 
	This plugin can process the output correspondences of the homest (homography estimation) plugin.*/
template <typename T>
class filewriterwrlhomest_DECLDIR filewriterWRLhomest :
		public TemplatedParameteredObject<T> {
public:
	
    /// Filename to write the *.wrl file to.
    Parameter<std::string> filename;

	/// Input 3D point cloud.
	InputSlot < std::vector<std::vector<std::vector<T> > > > pointCloud;

	/// create a new filewriterWRLhomest object
	/// \param name          Instance name
	filewriterWRLhomest(const std::string& name);

	/// Update object.
	virtual void execute();

private:

	static const float _defaultColorR ;
	static const float _defaultColorG ;
	static const float _defaultColorB ;
};

#endif // _FILEWRITERWRLHOMEST_H_



