/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file filewriterWRL.hxx
 *  Implementation of the parameter class filewriterWRL.
 *  \author Moritz Becker, Stephan Meister
 *  \date 09.06.2010
 */

#ifndef _FILEWRITERWRLHOMEST_HXX_
#define _FILEWRITERWRLHOMEST_HXX_

#include <hekate/FilewriterWRLhomest.h>

//set static member vars

template <typename T>
const float filewriterWRLhomest<T>::_defaultColorR = 1.0 ;
template <typename T>
const float filewriterWRLhomest<T>::_defaultColorG = 1.0 ;
template <typename T>
const float filewriterWRLhomest<T>::_defaultColorB = 1.0 ;


template <typename T>
filewriterWRLhomest<T>::filewriterWRLhomest(const std::string& name) :
		TemplatedParameteredObject<T>("filewriterWRLhomest", name,
			"Generates a *.wrl file with lines between the input correspondences. "
			"This plugin can process the output correspondences of the homest (homography estimation) plugin."),
		pointCloud(false, false)
{

	ParameteredObject::_addParameter(filename, "filename", "Filename to write point cloud to.",
			"filewrite");
	ParameteredObject::_addInputSlot(pointCloud, "point_cloud", "Point correspondences.", "vector<vector<vector<T> > >"); 
	
}

template <typename T>
void filewriterWRLhomest<T>::execute() {
	PARAMETEREDOBJECT_AVOID_REEXECUTION;
	ParameteredObject::execute();

    std::string inFile = this->filename() ;
	//check if input string already contains file type .wrl
	if(inFile.substr(inFile.size()-4,4)!=".wrl") inFile.append(".wrl");

	FILE* fp; 
	fp = fopen(inFile.c_str(),"w");
	if(fp == NULL)
	{	throw std::runtime_error(std::string("Could not open file ") + std::string(inFile)) ;	}
	//write HEADER
	fprintf(fp, "#VRML V2.0 utf8 Hekate filewriterWRL plugin\n") ;


	//write point data
	
	//create Shape node to hold the geometry data
	fprintf(fp, "\nShape {\n");

	fprintf(fp, "\tgeometry IndexedLineSet {\n");

	fprintf(fp, "\t\tcoord Coordinate {\n");
	fprintf(fp, "\t\t\tpoint [\n");

	// write point coords to wrl file
	typename std::vector<std::vector<std::vector<T> > >::const_iterator it =
		pointCloud().begin() ;
	typename std::vector<std::vector<std::vector<T> > >::const_iterator end =
		pointCloud().end() ;

	for(;it != end ; it++)
	{
		fprintf(fp, "\t\t\t%f %f %f,\n",
			(double)(*it)[0][0],
			(double)(*it)[0][1],
			(double)(*it)[0][2]) ;
		fprintf(fp, "\t\t\t%f %f %f,\n",
			(double)(*it)[1][0],
			(double)(*it)[1][1],
			(double)(*it)[1][2]) ;
	}

	//close indentations
	fprintf(fp, "\t\t\t]\n\t\t}\n");
	fprintf(fp, "coordIndex [\n");

	for(unsigned int i = 0; i<pointCloud().size()*2; i=i+2)
	{
		fprintf(fp, "\t\t\t") ;
		fprintf(fp, "%u, ", i) ;
		fprintf(fp, "%u, ", i+1) ;
		fprintf(fp, "-1,\n") ;
	}

	fprintf(fp, "\t\t\t]\n");		
	
	fprintf(fp, "\t}\n}");
	fclose(fp);
}

#endif /* _FILEWRITERWRL_HXX_ */



