/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Flow2Correspondences.h
 *  Declaration of the parameter class Flow2Correspondences.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 18.07.2012
 */

#ifndef _FLOW2CORRESPONDENCES_H_
#define _FLOW2CORRESPONDENCES_H_

#ifdef _MSC_VER
#ifdef flow2correspondences_EXPORTS
/// Visual C++ specific code
#define flow2correspondences_DECLDIR __declspec(dllexport)
#else
#define flow2correspondences_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define flow2correspondences_DECLDIR
#endif

#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>
#include <charon-core/ParameteredObject.h>

using namespace hekate;
using namespace cimg_library;

/// Converts a flow image to Correspondences
/** Converts a flow image to Correspondences<T>
 */
template <typename T>
class flow2correspondences_DECLDIR Flow2Correspondences :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	Flow2Correspondences(const std::string& name = "");

	/// flow image
	InputSlot< cimg_library::CImgList<T> > flowImage;

	/// color images
	InputSlot< cimg_library::CImgList<T> > colorImages;

	/// A mask to select only areas of the given flow
	InputSlot< cimg_library::CImgList<T> > flowMask;

	/// correspondences
	OutputSlot< hekate::CorrespondenceList<T> > correspondences;

	/// Step size for choosing points (1 to take every points -> slow).
	Parameter< int > stepSize ;

	/// Start a new correspondence every "stepSizeNewCorrs"th frame
	Parameter< int > stepSizeNewCorrs ;

	/// Length of a correspondence (number of frames)
	Parameter< int > length ;

	/// List of image indices represented by the flow images
	ParameterList< int > indicesFlow;

	/// Update object.
	virtual void execute();

	/// Check inout consistency
	bool checkInputConsistency();

};

#endif // _FLOW2CORRESPONDENCES_H_
