/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Flow2Correspondences.h
 *  Declaration of the parameter class Flow2Correspondences.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 18.07.2012
 */

#ifndef _FLOW2CORRESPONDENCESKITTI_H_
#define _FLOW2CORRESPONDENCESKITTI_H_

#ifdef _MSC_VER
#ifdef flow2correspondenceskitti_EXPORTS
/// Visual C++ specific code
#define flow2correspondenceskitti_DECLDIR __declspec(dllexport)
#else
#define flow2correspondenceskitti_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define flow2correspondenceskitti_DECLDIR
#endif

#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>
#include <charon-core/ParameteredObject.h>

/// Converts a flow image to Correspondences
/** Converts a flow image to Correspondences<T>
 */
template <typename T>
class flow2correspondenceskitti_DECLDIR Flow2CorrespondencesKitti :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	Flow2CorrespondencesKitti(const std::string& name = "");

	/// flow image
	InputSlot< cimg_library::CImgList<T> > flowImage;

	/// color images
	InputSlot< cimg_library::CImgList<T> > colorImages;

	/// correspondences
	OutputSlot< hekate::CorrespondenceList<T> > correspondences;

	/// correspondences
	OutputSlot< cimg_library::CImgList<T> > flow;

	/// Step size for choosing points (1 to take every points -> slow).
	Parameter< int > stepSize ;

	/// Update object.
	virtual void execute();
};

#endif // _FLOW2CORRESPONDENCESKITTI_H_
