/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file FlowConfidence.h
 *  Declaration of the parameter class FlowConfidence.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 15.11.2013
 */

#ifndef _FLOWCONFIDENCE_H_
#define _FLOWCONFIDENCE_H_

#ifdef _MSC_VER
#ifdef flowconfidence_EXPORTS
/// Visual C++ specific code
#define flowconfidence_DECLDIR __declspec(dllexport)
#else
#define flowconfidence_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define flowconfidence_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

using namespace cimg_library;

/// Confidence measure for a given flow field
/** Confidence measure for a given flow field
 */
template <typename T>
class flowconfidence_DECLDIR FlowConfidence :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	FlowConfidence(const std::string& name = "");

	/// Flow to measure confidence for
	InputSlot< std::vector<std::string> > flow;

	/// Weights according to the measured confidence
	OutputSlot< cimg_library::CImgList<T> > confidence;

	/// Size of blocks to evaluate
	Parameter< int > blockSize;

	/// Number of dimension for PCA projection
	Parameter< int > projectDimensions;

protected:
	/// Update object.
	virtual void execute();
	
	/// Get vector of u and v flow at area (_x,_y) and its neighbourhood
	bool getVec(CImg<T> &vec, const CImg<float> &u, const CImg<float> &v, const int index, const int _x, const int _y, 
		const int _bs, const int _w, const int _h, const float _mean);

};

#endif // _FLOWCONFIDENCE_H_
