/*  Copyright (C) 2011 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file FundamentalMatrix.h
 *  Declaration of the parameter class FundamentalMatrix.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 14.10.2011
 */

#ifndef _FUNDAMENTALMATRIX_H_
#define _FUNDAMENTALMATRIX_H_

#ifdef _MSC_VER
#ifdef fundamentalmatrix_EXPORTS
/// Visual C++ specific code
#define fundamentalmatrix_DECLDIR __declspec(dllexport)
#else
#define fundamentalmatrix_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define fundamentalmatrix_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>

using namespace hekate;

/// Calculates a fundamental matrix using the OpenCV implementation
/** This module calculates a fundamental matrix based on OpenCV.
 */
template <typename T>
class fundamentalmatrix_DECLDIR FundamentalMatrix :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	FundamentalMatrix(const std::string& name = "");

	/// A vector list of corresponding feature point.
	InputSlot< typename CorrespondenceList<T> > correspondences;

	/// The calculated fundamental matrix
	OutputSlot< typename IndexedMatrixList > fundamentalMatrix;
	/// A correspondence list, where outliers have been sorted out by epipolar
	/// condition.
	OutputSlot< typename CorrespondenceList<T> > correspondenceInlier;

	/// Numer of maximal frames which will be considered
	Parameter< int > maxFrames;
	/// Parameter used for RANSAC. It is the maximum distance from a point to
	/// an epipolar line in pixels, beyond which the point is considered an
	/// outlier and is not used for computing the final fundamental matrix. It
	/// can be set to something like 1-3, depending on the accuracy of the
	/// point localization, image resolution, and the image noise.
	Parameter< double > paramRANSAC;
	/// Parameter used for the RANSAC or LMedS methods only. It specifies a
	/// desirable level of confidence (probability) that the estimated matrix
	/// is correct.
	Parameter< double > paramConfidence;
	/// Prints details of calculation to log file.
	Parameter< bool > logDetails;

	/// Update object.
	virtual void execute();

	int calcMax, calcCurrent;

};

#endif // _FUNDAMENTALMATRIX_H_
