/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file GetIdsOfNonEmtyImages.hxx
 *  Implementation of the parameter class GetIdsOfNonEmtyImages.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 21.08.2013
 */

#ifndef _GETIDSOFNONEMTYIMAGES_HXX_
#define _GETIDSOFNONEMTYIMAGES_HXX_

#include <charon-core/ParameteredObject.hxx>
#include "GetIdsOfNonEmtyImages.h"

template <typename T>
GetIdsOfNonEmtyImages<T>::GetIdsOfNonEmtyImages(const std::string& name) :
		TemplatedParameteredObject<T>(
			"GetIdsOfNonEmtyImages", name,
			"<h2>Reads an image sequence and determines ids of non empty "
			"images</h2><br>"
			"Reads an image sequence and determines ids of non empty images"
		)
{
	ParameteredObject::_addInputSlot(
		images, "images",
		"Images with putative emptyness",
		"CImgList<T>");

	ParameteredObject::_addOutputSlot(
		indexList, "indexList",
		"List of ids of non empty images",
		"vector<int>");

}

template <typename T>
void GetIdsOfNonEmtyImages<T>::execute() {

	sout << "Read image ids of non-empty images: " ;

	for( size_t i = 0; i < this->images().size(); i++ )
	{
		// Check if current image is not black
		if( this->images()[i].get_channel(0).sum() > 0 )
		{
			this->indexList().push_back( i ) ;
			sout << i << ", " ;
		}
	}

	sout << std::endl;


}

#endif /* _GETIDSOFNONEMTYIMAGES_HXX_ */
