/*  Copyright (C) 2009 SasiKanth

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file guidedmatchingssd.h
 *  Declaration of the parameter class guidedmatchingssd.
 *  \author SasiKanth
 *  \date 16.06.2010
 */

#ifndef _GUIDEDMATCHINGSSD_H_
#define _GUIDEDMATCHINGSSD_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef guidedmatchingssd_EXPORTS
/// Visual C++ specific code
#define guidedmatchingssd_DECLDIR __declspec(dllexport)
#else
#define guidedmatchingssd_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define guidedmatchingssd_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// This plugin performs a guided matching of correspondences based on the Sum of Squared Differences (SSD)
/// .
template <typename T>
class guidedmatchingssd_DECLDIR guidedmatchingssd :
		public TemplatedParameteredObject<T> {
public:
	/// The size of the window to be searched around the epipolar line's points
	Parameter < int > window_size;
	/// The matt size for computation of SSD
	Parameter < int > matt_size;

	

	/// The stereo image pair 
	InputSlot < cimg_library::CImgList<T> > Input_Images;
	/// The computed f-matrix of the stereo image pair 
	InputSlot < cimg_library::CImgList<T> > fmatrix;
	/// The set of correspondences after guided matching 
	OutputSlot < std::vector<std::vector<std::pair<T, T> > > > correspondences_output;
	

	/// create a new guidedmatchingssd object
	/// \param name          Instance name
	guidedmatchingssd(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _GUIDEDMATCHINGSSD_H_



