/*  Copyright (C) 2009 SasiKanth

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file guidedmatchingssd.hxx
 *  Implementation of the parameter class guidedmatchingssd.
 *  \author SasiKanth
 *  \date 16.06.2010
 */

#ifndef _GUIDEDMATCHINGSSD_HXX_
#define _GUIDEDMATCHINGSSD_HXX_

#include <hekate/GuidedMatchingSsd.h>

template <typename T>
guidedmatchingssd<T>::guidedmatchingssd(const std::string& name) :
		TemplatedParameteredObject<T>(
				"guidedmatchingssd", name,
				"This plugin performs a guided matching of correspondences based on the Sum of Squared Differences (SSD)"
			"."
		)
			,Input_Images(false, false) 
			,fmatrix(false, false) 
			
			
{
	ParameteredObject::_addParameter (window_size, "window_size", 
	"The size of the window to be searched around the epipolar line's points", 
	25);
	ParameteredObject::_addParameter (matt_size, "matt_size", 
	"The matt size for computation of SSD", 
	5);
	

	

	ParameteredObject::_addInputSlot(Input_Images, "Input_Images", 
	"The stereo image pair", 
	"CImgList<T>"); 
	ParameteredObject::_addInputSlot(fmatrix, "fmatrix", 
	"The computed f-matrix of the stereo image pair", 
	"CImgList<T>"); 
	ParameteredObject::_addOutputSlot(correspondences_output, "correspondences_output", 
	"The set of correspondences after guided matching", 
	"std::vector<std::vector<std::pair<T, T> > >"); 
	
}

template <typename T>
void guidedmatchingssd<T>::execute() {
	PARAMETEREDOBJECT_AVOID_REEXECUTION;
	ParameteredObject::execute();

	// Beginning of Plugin Code

	int i, j; // Indices used to traverse the left image completely
	int x, y; // Indices of the right image generated from the epipolar geometry
	int si, sj; // Indices used to traverse the windows
	int mi, mj; // Indices used to cover the Matt
	int ssd=0, min_ssd=RAND_MAX; // SSD variables
	int temp_min[2]; // Temporarily stores the minimum coordinates
	std::vector<std::pair<T, T> > coords_set;
	std::pair<T,T> leftpair, rightpair;
	cimg_library::CImg<T> xleft(1,3), lright(1,3);

	for(i=0; i<Input_Images()[0].width(); i++)
	{
		for(j=0; j<Input_Images()[0].height(); j++)
		{
			sout << "(" << i << "," << j << ")" << std::endl;
			
			// Assign the left point set

			xleft(0,0) = i;
			xleft(0,1) = j;
			xleft(0,2) = 1;

			// Generate the epipolar line

			lright = fmatrix()[0]*xleft;

			// Traversing the right image

			for(x=0; x<Input_Images()[1].width(); x++)
			{
				// Generating the y-coordinate

				y = -((lright(0)*x+lright(2))/(lright(1)));

				// Computing the SSD

				ssd =0;

				for(si=std::max(0,x-window_size/2); si<std::min(Input_Images()[1].width(),x+window_size/2); si++)
				{
					for(sj=std::max(0,y-window_size/2); sj<std::min(Input_Images()[1].height(),y+window_size/2); sj++)
					{
						for(mi=std::max(0,si-matt_size/2); mi<std::min(Input_Images()[1].width(),si+matt_size/2); mi++)
						{
							for(mj=std::max(0,sj-matt_size/2); mj<std::min(Input_Images()[1].height(),sj+matt_size/2); mj++)
							{
								ssd = ssd + (Input_Images()[0](i,j)-Input_Images()[1](mi,mj))*(Input_Images()[0](i,j)-Input_Images()[1](mi,mj));
							}
						}
						
						// Checking if SSD is minimum
	
						if(ssd < min_ssd)
						{
							min_ssd = ssd;
							temp_min[0] = x;
							temp_min[1] = y;
						}
					}
				}
			}

			// Pushing Coordinate pair to the output

			leftpair.first = i;
			leftpair.second = j;
			rightpair.first = temp_min[0];
			rightpair.second = temp_min[1];

			coords_set.push_back(leftpair);
			coords_set.push_back(rightpair);

			correspondences_output().push_back(coords_set);

		}
	}
}

#endif /* _GUIDEDMATCHINGSSD_HXX_ */



