/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file HoughTransformation.h
 *  Declaration of the parameter class HoughTransformation.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 12.11.2012
 */

#ifndef _HOUGHTRANSFORMATION_H_
#define _HOUGHTRANSFORMATION_H_

#ifdef _MSC_VER
#ifdef houghtransformation_EXPORTS
/// Visual C++ specific code
#define houghtransformation_DECLDIR __declspec(dllexport)
#else
#define houghtransformation_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define houghtransformation_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Extracts edges by hough transformation
/** Extracts edges by hough transformation
 */
template <typename T>
class houghtransformation_DECLDIR HoughTransformation :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	HoughTransformation(const std::string& name = "");

	/// paths to image folder
	InputSlot< vector<string> > imagePaths;

	/// Edges
	OutputSlot< cimg_library::CImgList<T> > edges;

	/// Update object.
	virtual void execute();
};

#endif // _HOUGHTRANSFORMATION_H_
