/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Image2Roi.h
 *  Declaration of the parameter class Image2Roi.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 05.12.2013
 */

#ifndef _IMAGE2ROI_H_
#define _IMAGE2ROI_H_

#ifdef _MSC_VER
#ifdef image2roi_EXPORTS
/// Visual C++ specific code
#define image2roi_DECLDIR __declspec(dllexport)
#else
#define image2roi_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define image2roi_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <charon-utils/Roi.h>

/// Converts image dimensions to the ROI format
/** Converts image dimensions to the ROI format. This is used to avoid
 *  adjustments of the ROI plugin if the whole image is needed anyway.
 */
template <typename T>
class image2roi_DECLDIR Image2Roi :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	Image2Roi(const std::string& name = "");

	/// Image with dimensions of interest
	InputSlot< cimg_library::CImgList<T> > image;

	/// ROI object
	OutputSlot< Roi<int>* > roi;


protected:
	/// Update object.
	virtual void execute();
};

#endif // _IMAGE2ROI_H_
