/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Image2Roi.hxx
 *  Implementation of the parameter class Image2Roi.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 05.12.2013
 */

#ifndef _IMAGE2ROI_HXX_
#define _IMAGE2ROI_HXX_

#include <charon-core/ParameteredObject.hxx>
#include "Image2Roi.h"

template <typename T>
Image2Roi<T>::Image2Roi(const std::string& name) :
		TemplatedParameteredObject<T>(
			"Image2Roi", name,
			"<h2>Converts image dimensions to the ROI format</h2><br>"
			"Converts image dimensions to the ROI format. This is used to "
			"avoid adjustments of the ROI plugin if the whole image is "
			"needed anyway."
		)
{
	ParameteredObject::_addInputSlot(
		image, "image",
		"Image with dimensions of interest",
		"CImgList<T>");

	ParameteredObject::_addOutputSlot(
		roi, "roi",
		"ROI object",
		"Roi<int>*");

}

template <typename T>
void Image2Roi<T>::execute() {

	if( this->roi() == NULL) 
	{
		this->roi() = new Roi<int>();
		this->roi()->initialize();
	}

	if( this->image().size() > 0) 
	{
		int x = this->image()[0].width();
		int y = this->image()[0].height();
		int z = this->image()[0].depth();

		int n = this->image().size();
			
		sout << "Write image dimension to ROI" << std::endl;
		sout << "Image dimension is " << x << "," << y << std::endl;
		this->roi()->assign( 0,x, 
							 0,y,
							 0,z,
							 0, 1 );
	}
}

#endif /* _IMAGE2ROI_HXX_ */
