/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file ImagePathReader.h
 *  Declaration of the parameter class ImagePathReader.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 21.02.2012
 */

#ifndef _IMAGEPATHREADER_H_
#define _IMAGEPATHREADER_H_

#ifdef _MSC_VER
#ifdef imagepathreader_EXPORTS
/// Visual C++ specific code
#define imagepathreader_DECLDIR __declspec(dllexport)
#else
#define imagepathreader_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define imagepathreader_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Reads in all image paths of a specific image format
/** Reads in all image paths of a specific image format. Suppose one selects
 *  an JPG image inside a folder, the plugin will save every path of a JPG
 *  image which is inside that folder to a vector<string>
 */
template <typename T>
class imagepathreader_DECLDIR ImagePathReader :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	ImagePathReader(const std::string& name = "");
	
	/// Paths to images
	OutputSlot< std::vector<std::string> > imagePaths;

	/// Image path of an image
	Parameter< std::string > imageDir;

	/// ID of first frame to read in
	Parameter< int > startFrame;

	/// ID of last frame to read in
	Parameter< int > endFrame;

	/// Ignore number pattern for read in
	Parameter< bool > ignorePattern;

	/// Throw exception if error while read
	Parameter< bool > throw_error;

	/// Print log messages
	Parameter< bool > verbose;

	/// Update object.
	virtual void execute();

protected:
	std::vector<std::string> filenames;
	std::string filepath;
	size_t numberOfFiles;

};

#endif // _IMAGEPATHREADER_H_
