/*  Copyright (C) 2009 Julian Coordts

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file images2orderedsequence.h
 *  Declaration of the parameter class images2orderedsequence.
 *  \author Julian Coordts
 *  \date 31.03.2010
 */

#ifndef _IMAGES2ORDEREDSEQUENCE_H_
#define _IMAGES2ORDEREDSEQUENCE_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef images2orderedsequence_EXPORTS
/// Visual C++ specific code
#define images2orderedsequence_DECLDIR __declspec(dllexport)
#else
#define images2orderedsequence_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define images2orderedsequence_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// generates a sorted (choosen by parameter) sequence of the 2 input imagelist
template <typename T>
class images2orderedsequence_DECLDIR images2orderedsequence :
		public TemplatedParameteredObject<T> {
public:
	/// sort order: left before right if true, right before left if false 
	Parameter < bool > leftbeforeright;
	

	/// the left image(s) 
	InputSlot < cimg_library::CImgList<T> > images_left;
	/// the right image(s) 
	InputSlot < cimg_library::CImgList<T> > images_right;
	/// the merged image sequence 
	OutputSlot < cimg_library::CImgList<T> > image_sequence;
	

	/// create a new images2orderedsequence object
	/// \param name          Instance name
	images2orderedsequence(const std::string& name);

	/// Update object.
	virtual void execute();
	///boolean to avoid multiple calls
	bool called;
};

#endif // IMAGES2ORDEREDSEQUENCE_H_



