/*  Copyright (C) 2009 Julian Coordts

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file images2orderedsequence.hxx
 *  Implementation of the parameter class images2orderedsequence.
 *  \author Julian Coordts
 *  \date 31.03.2010
 */

#ifndef _IMAGES2ORDEREDSEQUENCE_HXX_
#define _IMAGES2ORDEREDSEQUENCE_HXX_

#include <hekate/Images2OrderedSequence.h>

template <typename T>
images2orderedsequence<T>::images2orderedsequence(const std::string& name) :
		TemplatedParameteredObject<T>("images2orderedsequence", name,
			"generates a sorted (choosen by parameter) sequence of the 2 input imagelist")
{
	ParameteredObject::_addParameter (leftbeforeright, "leftbeforeright", "sort order: left before right if true, right before left if false", true);
	

	ParameteredObject::_addInputSlot(images_left, "images_left", "the left image(s)", "CImgList<T>");
	ParameteredObject::_addInputSlot(images_right, "images_right", "the right image(s)", "CImgList<T>");
	ParameteredObject::_addOutputSlot(image_sequence, "image_sequence", "the merged image sequence", "CImgList<T>");

	ParameteredObject::_setTags("Hekate") ;

	called=false;
}
template <typename T>
void images2orderedsequence<T>::execute() {
	PARAMETEREDOBJECT_AVOID_REEXECUTION;
	ParameteredObject::execute();

	this->image_sequence().assign(2,images_left()[0].width(),images_left()[0].height());
	if (this->leftbeforeright())
	{
		this->image_sequence()[0] = images_left()[0];
		this->image_sequence()[1] = images_right()[0];
	}
	else
	{
		this->image_sequence()[0] = images_right()[0];
		this->image_sequence()[1] = images_left()[0];
	}


}

#endif /* _images2orderedsequence_HXX_ */



