#ifndef _INDEXEDMATRIX_HXX_
#define _INDEXEDMATRIX_HXX_

#include <CImg.h>
#include <stdio.h>
#include <vector>


	/// Class to save CImg matrices together with two indeces
	class IndexedMatrix : public cimg_library::CImg<double>
	{
	public:

		/// default constructor
		IndexedMatrix(): degenerated(false), f(0), radial(0), distortion_type(0) {}
		IndexedMatrix(unsigned int a, unsigned int b) : 
				cimg_library::CImg<double>(a,b), degenerated(false), 
				f(0), radial(0), distortion_type(0) {}
		IndexedMatrix(const cimg_library::CImg<double>& a) : 
				cimg_library::CImg<double>(a), degenerated(false),
				f(0), radial(0), distortion_type(0) {}
		IndexedMatrix(const cimg_library::CImg<double>& a, const size_t id1, const size_t id2) : 
				cimg_library::CImg<double>(a), degenerated(false),
				f(0), radial(0), distortion_type(0), index1(id1), index2(id2) {}
		
		const size_t getIndex1() const;
		const size_t getIndex2() const;

		bool degenerated;
		
		void setIndex(size_t id1, size_t id2);
		
		void setIndex(size_t id1);

		float f;

		float radial;

		int distortion_type;

	private:
		/// Index of image 1
		size_t index1 ;

		/// Index of image 2
		size_t index2 ;
			
	} ;
	

	
	/// Class which keeps IndexedMatrix classes together with a () operator
	class IndexedMatrixList : public std::vector< IndexedMatrix >
	{
		public:
		IndexedMatrix& operator()(size_t i, size_t j);
		const IndexedMatrix& operator()(size_t i, size_t j) const;
		IndexedMatrix& operator()(size_t i);
		const IndexedMatrix& operator()(size_t i) const;
		const size_t sizeNonDegenerated() const;
		const IndexedMatrixList getNonDegeneratedList() const;
		bool exists(size_t i) const;
		bool exists(size_t i, size_t j) const;
	};




#endif /* #ifndef _INDEXEDMATRIX_HXX_ */