/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file IndexedMatrixListToCImgList.hxx
 *  Implementation of the parameter class IndexedMatrixListToCImgList.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 27.02.2013
 */

#ifndef _INDEXEDMATRIXLISTTOCIMGLIST_HXX_
#define _INDEXEDMATRIXLISTTOCIMGLIST_HXX_

#include <charon-core/ParameteredObject.hxx>
#include <hekate/IndexedMatrixListToCImgList.h>

template <typename T>
IndexedMatrixListToCImgList<T>::IndexedMatrixListToCImgList(const std::string& name) :
		TemplatedParameteredObject<T>(
			"IndexedMatrixListToCImgList", name,
			"<h2>Converts IndexedMatrixList to CImgList</h2><br>"
			"Converts IndexedMatrixList to CImgList"
		)
{
	ParameteredObject::_addInputSlot(
		indexedMatrixList, "indexedMatrixList",
		"Indexed matrix list",
		"IndexedMatrixList");

	ParameteredObject::_addOutputSlot(
		cimgList, "cimgList",
		"CImgList",
		"CImgList<T>");

}

template <typename T>
void IndexedMatrixListToCImgList<T>::execute() {

	for( size_t i = 0; i < this->indexedMatrixList().size(); i++ )
	{
		cimg_library::CImg<T> matrix = this->indexedMatrixList()[i] ;
		this->cimgList().push_back( matrix );
	}

}

#endif /* _INDEXEDMATRIXLISTTOCIMGLIST_HXX_ */
