/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file InitialCameraMatrices.h
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 22.08.2012
 *  Declaraion of parameter class InitialCameraMatrices.
 */
#ifndef _INITIALCAMERAMATRICES_H_
#define _INITIALCAMERAMATRICES_H_

#ifdef _MSC_VER
#ifdef initialcameramatrices_EXPORTS
/// Visual C++ specific code
#define initialcameramatrices_DECLDIR __declspec(dllexport)
#else
#define initialcameramatrices_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define initialcameramatrices_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/IndexedMatrix.h>

/// Generates two initial camera matrices
/** Generates two initial camera matrices
 */
class initialcameramatrices_DECLDIR InitialCameraMatrices : public ParameteredObject {
public:
	/// default constructor
	/// \param name             instance name
	InitialCameraMatrices(const std::string& name = "");

	/// Camera matrices
	OutputSlot< IndexedMatrixList > cameraMatrices;

	/// Camera 1: K*(R|t)
	ParameterList< double > camera1;
	/// Camera 2: K2*(R2|t2)
	ParameterList< double > camera2;	
	/// initial Index 1
	Parameter< int > index1;
	/// initial Index 2
	Parameter< int > index2;

	/// Update object.
	virtual void execute();
};

#endif /* _INITIALCAMERAMATRICES_H_ */
