/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file InitialFrameSelection.h
 *  Declaration of the parameter class InitialFrameSelection.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 27.07.2012
 */

#ifndef _INITIALFRAMESELECTION_H_
#define _INITIALFRAMESELECTION_H_

#ifdef _MSC_VER
#ifdef initialframeselection_EXPORTS
/// Visual C++ specific code
#define initialframeselection_DECLDIR __declspec(dllexport)
#else
#define initialframeselection_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define initialframeselection_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <hekate/Correspondence.hxx>

using namespace hekate;

/// Selects initial frames of an image sequence for SfM Reconstruction
/** Selects three initial frames from an image sequence. Selection criterion
 *  is the number of correspondences between those frames, but also the mean
 *  distance between correspondences.
 */
template <typename T>
class initialframeselection_DECLDIR InitialFrameSelection :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	InitialFrameSelection(const std::string& name = "");

	/// correspondences
	InputSlot< hekate::CorrespondenceList<T> > correspondences;

	/// Vector of three initial frames used for SfM
	OutputSlot< std::vector<int> > initialFrames;

	/// Distribution of correspondences between images
	OutputSlot< cimg_library::CImgList<T> > distributionCorrs;

	/// Distribution of disparity between correspondences between images
	OutputSlot< cimg_library::CImgList<T> > distributionDisparity;

	/// Number of correspondences between frame 1 and 2
	Parameter< int > nCorrs1;
	/// Number of correspondences between frame 1 and 3
	Parameter< int > nCorrs2;
	/// Number of correspondences between frame 2 and 3
	Parameter< int > nCorrs3;

	/// Update object.
	virtual void execute();
};

#endif // _INITIALFRAMESELECTION_H_
