/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file JPG2HSV.h
 *  Declaration of the parameter class JPG2HSV.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 08.04.2013
 */

#ifndef _JPG2HSV_H_
#define _JPG2HSV_H_

#ifdef _MSC_VER
#ifdef jpg2hsv_EXPORTS
/// Visual C++ specific code
#define jpg2hsv_DECLDIR __declspec(dllexport)
#else
#define jpg2hsv_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define jpg2hsv_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Converts jpg image type to hsv
/** Converts jpg image type to hsv
 */
template <typename T>
class jpg2hsv_DECLDIR JPG2HSV :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	JPG2HSV(const std::string& name = "");

	/// JPG images
	InputSlot< cimg_library::CImgList<T> > imagesJPG;

	/// HSV images
	OutputSlot< cimg_library::CImgList<T> > imagesHSV;

	/// Export channel H
	Parameter< bool > channelH;
	/// Export channel S
	Parameter< bool > channelS;
	/// Export channel V
	Parameter< bool > channelV;


protected:
	/// Update object.
	virtual void execute();
};

#endif // _JPG2HSV_H_
