/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file JPG2HSV.hxx
 *  Implementation of the parameter class JPG2HSV.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 08.04.2013
 */

#ifndef _JPG2HSV_HXX_
#define _JPG2HSV_HXX_

#include <charon-core/ParameteredObject.hxx>
#include "JPG2HSV.h"

template <typename T>
JPG2HSV<T>::JPG2HSV(const std::string& name) :
		TemplatedParameteredObject<T>(
			"JPG2HSV", name,
			"<h2>Converts jpg image type to hsv</h2><br>"
			"Converts jpg image type to hsv"
		)
{
	ParameteredObject::_addInputSlot(
		imagesJPG, "imagesJPG",
		"JPG images",
		"CImgList<T>");

	ParameteredObject::_addOutputSlot(
		imagesHSV, "imagesHSV",
		"HSV images",
		"CImgList<T>");

	ParameteredObject::_addParameter< bool >(
		channelH, "channelH",
		"Export channel H",
		1, "bool");
	ParameteredObject::_addParameter< bool >(
		channelS, "channelS",
		"Export channel S",
		1, "bool");
	ParameteredObject::_addParameter< bool >(
		channelV, "channelV",
		"Export channel V",
		1, "bool");

}

template <typename T>
void JPG2HSV<T>::execute() {
	
	for( size_t i = 0; i < this->imagesJPG().size(); i++ )
	{

		cimg_library::CImg<T> img = this->imagesJPG()[i];
		sout << img.spectrum() << std::endl;

		img.RGBtoHSV();
		sout << img.spectrum() << std::endl;

		cimg_library::CImg<T> imgOut(img.width(),img.height(),1,1);
		if( img.spectrum() == 3 )
		{
			if(this->channelH())
				imgOut.assign(img.channel(0),'c');
			if(this->channelS())
				imgOut.assign(img.channel(0),'c');
			if(this->channelV())
				imgOut.assign(img.channel(0),'c');
		}
		else
			imgOut.assign(img);

		sout << imgOut.spectrum() << std::endl;

		this->imagesHSV().push_back(img);

	}

}

#endif /* _JPG2HSV_HXX_ */
