/*  Copyright (C) 2009 Julian Coordts

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file lsqnonlinsolver.h
 *  Declaration of the parameter class lsqnonlinsolver.
 *  \author Julian Coordts
 *  \date 22.03.2010
 */

#ifndef _LSQNONLINSOLVER_H_
#define _LSQNONLINSOLVER_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef lsqnonlinsolver_EXPORTS
/// Visual C++ specific code
#define lsqnonlinsolver_DECLDIR __declspec(dllexport)
#else
#define lsqnonlinsolver_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define lsqnonlinsolver_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <levmar/levmar.h>

/// Baseclass for least squared not-linear solvers
template <typename T>
class lsqnonlinsolver_DECLDIR lsqnonlinsolver :
		public TemplatedParameteredObject<T> {
public:
	/// pointer to instance
	OutputSlot < lsqnonlinsolver<T>* > out;

	/// create a new lsqnonlinsolver object
	lsqnonlinsolver(const std::string & pluginName, const std::string& name,
			const std::string & doc);

	virtual ~lsqnonlinsolver();

	/// Update object.
	virtual int solve(
			void costfunct(double *p, double *hx, int m, int n, void *adata),
			double *costinput,
			double *costresults,
			int inputsize,
			int outputsize,
			double *lowerbound,
			double *upperbound,
			int maxIterations,
			double opts[5],
			double info[LM_INFO_SZ],
			double *work,
			double *covar,
			void *data) = 0;
};

#endif // _LSQNONLINSOLVER_H_



