/*  Copyright (C) 2009 Julian Coordts

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file levmarSolverBC.h
 *  Declaration of the parameter class levmarSolverBC.
 *  \author Julian Coordts
 *  \date 19.03.2010
 */

#ifndef _LEVMARSOLVERBC_H_
#define _LEVMARSOLVERBC_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef levmarsolverbc_EXPORTS
/// Visual C++ specific code
#define levmarsolverbc_DECLDIR __declspec(dllexport)
#else
#define levmarsolverbc_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define levmarsolverbc_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <hekate/LSqNonLinSolver.h>

/// solver using levmar(double precision, with box constraints)
template <typename T>
class levmarsolverbc_DECLDIR levmarSolverBC :
		public lsqnonlinsolver<T> {
public:
	/// Number of maximum iteration count 
	Parameter < int > maxIterations;

	/* I: opts[0-4] = minim. options [\mu, \epsilon1, \epsilon2, \epsilon3, \delta]. Respectively the
	                       * scale factor for initial \mu, stopping thresholds for ||J^T e||_inf, ||Dp||_2 and ||e||_2 and
	                       * the step used in difference approximation to the Jacobian. Set to NULL for defaults to be used.
	                       * If \delta<0, the Jacobian is approximated with central differences which are more accurate
	                       * (but slower!) compared to the forward differences employed by default.
	                       */
	/// docu unknown 
	Parameter < double > opt0;
	/// docu unknown 
	Parameter < double > opt1;
	/// docu unknown 
	Parameter < double > opt2;
	/// docu unknown 
	Parameter < double > opt3;
	/// docu unknown 
	Parameter < double > opt4;
	

	/// create a new levmarSolverBC object
	///  @param name          Instance name
    levmarSolverBC(const std::string& name);
    levmarSolverBC();
    virtual ~levmarSolverBC();

	int solve(void costfunct(double *p, double *hx, int m, int n, void *adata),
				double *costinput,
			    double *costresults,
			    int inputsize,
			    int outputsize,
			    double *lowerbound,
			    double *upperbound,
			    int maxIt,
			    double opts[5],
			    double info[LM_INFO_SZ],
			    double *work,
			    double *covar,
			    void *data);

};

#endif // _LEVMARSOLVERBC_H_



