/*  Copyright (C) 2009 Julian Coordts

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file levmarSolverBC.hxx
 *  Implementation of the parameter class levmarSolverBC.
 *  \author Julian Coordts
 *  \date 19.03.2010
 */

#ifndef _LEVMARSOLVERBC_HXX_
#define _LEVMARSOLVERBC_HXX_

#include <hekate/LevmarSolverBC.h>
#include <levmar/levmar.h>
#include <hekate/LSqNonLinSolver.h>
#include <hekate/LSqNonLinSolver.hxx>

template <typename T>
levmarSolverBC<T>::levmarSolverBC(const std::string& name) :
	lsqnonlinsolver<T>("levmarSolverBC", name,
			"solver using levmar(double precision, with box constraints)")
{
	ParameteredObject::_addParameter (maxIterations, "maxIterations",
			"Number of maximum iteration count", 1000);
	ParameteredObject::_addParameter (opt0, "opt0",
			"docu unknown", LM_INIT_MU);
	ParameteredObject::_addParameter (opt1, "opt1",
			"docu unknown", 1E-15);
	ParameteredObject::_addParameter (opt2, "opt2",
			"docu unknown", 1E-15);
	ParameteredObject::_addParameter (opt3, "opt3",
			"docu unknown", 1E-20);
	ParameteredObject::_addParameter (opt4, "opt4",
			"docu unknown", LM_DIFF_DELTA);
}

template <typename T>
int levmarSolverBC<T>::solve(void costfunct(double *p, double *hx, int m, int n, void *adata),
			double *costinput,
		    double *costresults,
		    int inputsize,
		    int outputsize,
		    double *lowerbound,
		    double *upperbound,
		    int maxIt,
		    double opts[5],
		    double info[LM_INFO_SZ],
		    double *work,
		    double *covar,
		    void *data)
{
	//check if opts are set, if not set them according to the parameters
	if (maxIt==0)
	{
		sout << "\tlevmarSolverbc: maxIt not set. Choose Parameter maxIterations" << std::endl;
				maxIt=maxIterations;
	}
	if (opts[0]==0){
		sout << "\tlevmarSolverbc: opts[0] not set. Choose Parameter opt0" << std::endl;
		opts[0]= this->opt0();
	}
	if (opts[1]==0){
		sout << "\tlevmarSolverbc: opts[1] not set. Choose Parameter opt1" << std::endl;
		opts[1]= this->opt1();
	}
	if (opts[2]==0){
		sout << "\tlevmarSolverbc: opts[2] not set. Choose Parameter opt2" << std::endl;
		opts[2]= this->opt2();
	}
	if (opts[3]==0){
		sout << "\tlevmarSolverbc: opts[3] not set. Choose Parameter opt3" << std::endl;
		opts[3]= this->opt3();
	}
	if (opts[4]==0){
		sout << "\tlevmarSolverbc: opts[4] not set. Choose Parameter opt4" << std::endl;
		opts[4]= this->opt4();
	}

	sout << "levmarSolverBC: solve..." <<  std::endl;
	return dlevmar_bc_dif(costfunct,costinput,costresults,inputsize,outputsize,
			lowerbound,upperbound,maxIt,
			opts,info,work,covar,data);
}

template <typename T>
levmarSolverBC<T>::~levmarSolverBC() {
}

template <typename T>
levmarSolverBC<T>::levmarSolverBC() :
	levmarSolverBC<T>("levmarSolverBC", "", "levmarSolver BC") {
}

#endif /* _LEVMARSOLVERBC_HXX_ */
