/*  Copyright (C) 2011 Daniel Kondermann

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file ManualCorrespondences.h
 *  Declaration of the parameter class ManualCorrespondences.
 *  \author <a href="mailto:daniel@kondermann.de">
 *      Daniel Kondermann</a>
 *  \date 11.10.2011
 */

#ifndef _MANUALCORRESPONDENCES_H_
#define _MANUALCORRESPONDENCES_H_

#ifdef _MSC_VER
#ifdef manualcorrespondences_EXPORTS
/// Visual C++ specific code
#define manualcorrespondences_DECLDIR __declspec(dllexport)
#else
#define manualcorrespondences_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define manualcorrespondences_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>

using namespace std;
using namespace cimg_library;

/// Tool to annotate manual correspondences in stereo sequences
/** Works both from left to right as well as through time. Run workflow
 *  threaded in tuchulcha to see the console output.
 */
template <typename T>
class manualcorrespondences_DECLDIR ManualCorrespondences :
		public TemplatedParameteredObject<T> {
private:
	void drawFrame(int frame, CImg<T> &img);
public:
	/// default constructor
	/// \param name          Instance name
	ManualCorrespondences(const std::string& name = "");

	/// Left sequence
	InputSlot< cimg_library::CImgList<T> > left;
	/// Right sequence.
	InputSlot< cimg_library::CImgList<T> > right;

	OutputSlot< typename hekate::CorrespondenceList<T> > correspondences;

	/// Update object.
	virtual void execute();
};

#endif // _MANUALCORRESPONDENCES_H_
