/*  Copyright (C) 2011 Julian Wintermayr

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file ManualUpgrade.h
 *  Declaration of the parameter class ManualUpgrade.
 *  \author Julian Wintermayr
 *  \date 27.03.2011
 */

#ifndef _MANUALUPGRADE_H_
#define _MANUALUPGRADE_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef manualupgrade_EXPORTS
/// Visual C++ specific code
#define manualupgrade_DECLDIR __declspec(dllexport)
#else
#define manualupgrade_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define manualupgrade_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

#include "drawingarea.h"

/// Define the plane at infinity, parallel lines and orthographic lines to receive much better results!
template <typename T>
class manualupgrade_DECLDIR ManualUpgrade :
		public TemplatedParameteredObject<T> {
public:

	/// image 
	InputSlot < cimg_library::CImgList<T> > imageSequence;
	/// vectors 
	OutputSlot < std::vector<DrawingArea::drawnLines> > pointsVector;
	OutputSlot < std::vector<DrawingArea::drawnLines> > parallelLinesVector;
	OutputSlot < std::vector<DrawingArea::drawnLines> > orthographicLinesVector;	

	/// create a new ManualUpgrade object
	/// \param name          Instance name
	ManualUpgrade(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _MANUALUPGRADE_H_



