/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file MergeCorrespondences.hxx
 *  Implementation of the parameter class MergeCorrespondences.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 07.03.2013
 */

#ifndef _MERGECORRESPONDENCES_HXX_
#define _MERGECORRESPONDENCES_HXX_

#include <charon-core/ParameteredObject.hxx>
#include <hekate/MergeCorrespondences.h>

template <typename T>
MergeCorrespondences<T>::MergeCorrespondences(const std::string& name) :
		TemplatedParameteredObject<T>(
			"MergeCorrespondences", name,
			"<h2>Merges multiple CorrespondenceLists</h2><br>"
			"Merges multiple CorrespondenceLists"
		),
		correspondences(false,true)
{
	ParameteredObject::_addInputSlot(
		correspondences, "correspondences",
		"Multiple correspondence lists to merge",
		"CorrespondenceList");

	ParameteredObject::_addOutputSlot(
		correspodencesMerged, "correspodencesMerged",
		"Merged correspondences",
		"CorrespondenceList");

}

template <typename T>
void MergeCorrespondences<T>::execute() {

	CorrespondenceList<T> &list = this->correspodencesMerged();

	size_t indexOffset = 0;

	// iterate over multislot
	for( size_t i = 0; i < this->correspondences.size(); i++ )
	{
		
		auto it = this->correspondences[i].begin();
		auto itEnd = this->correspondences[i].end();
		for( ; it != itEnd; it++ )
		{
			Correspondence<T> corr( it->point, it->pointIndex + indexOffset, it->imgIndex, 0, it->R, it->G, it->B );
			list.push_back( corr );
		}
	
		int cloudSize = this->correspondences[i].size();
		indexOffset += cloudSize;
		sout << "Added point cloud with " << cloudSize << " points" << std::endl;

	}

	sout << "Number of merged points: " << this->correspodencesMerged().size() << std::endl;

}

#endif /* _MERGECORRESPONDENCES_HXX_ */
