/*  Copyright (C) 2013 Michael Mueller

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file MergePointClouds.h
 *  Declaration of the parameter class MergePointClouds.
 *  \author <a href="mailto:michael@fds-team.de">
 *      Michael Mueller</a>
 *  \date 28.02.2013
 */

#ifndef _MERGEPOINTCLOUDS_H_
#define _MERGEPOINTCLOUDS_H_

#ifdef _MSC_VER
#ifdef mergepointclouds_EXPORTS
/// Visual C++ specific code
#define mergepointclouds_DECLDIR __declspec(dllexport)
#else
#define mergepointclouds_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define mergepointclouds_DECLDIR
#endif

//#include <pcl/io/pcd_io.h>
//#include <pcl/point_types.h>
#include <hekate/Correspondence.hxx>
#include <charon-core/ParameteredObject.h>


#include <hekate/IndexedMatrix.h>
#include <charon-utils/CImg.h>

/// Merges several point clouds in a single one
/** Merges several point clouds in a single one
 */
template <typename T>
class mergepointclouds_DECLDIR MergePointClouds :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	MergePointClouds(const std::string& name = "");

	/// serveral input point clouds
	InputSlot< typename std::vector<hekate::CorrespondenceList<T> > > pointCloudsIn;

	/// one or more input point clouds
	OutputSlot< typename std::vector<hekate::CorrespondenceList<T> > > pointCloudsOut;

	InputSlot  < std::vector<IndexedMatrixList> > cameraMatricesIn;
	OutputSlot < std::vector<IndexedMatrixList> > cameraMatricesOut;

	OutputSlot< typename hekate::CorrespondenceList<T> >  	maxPointCloud;
	OutputSlot< IndexedMatrixList > 						maxCameras;

	Parameter < int > 		numCor;
	Parameter < int > 		maxIterations;
	Parameter < double > 	inlierThreshold;
	Parameter < int > 		minMergeCount;


protected:
	/// Update object.
	virtual void execute();

private:

	//pcl::PointCloud<pcl::PointXYZRGBA>::Ptr convertPointCloud(hekate::CorrespondenceList<T> input);
	bool mergeTwoPointClouds(	hekate::CorrespondenceList<T> &first, 	hekate::CorrespondenceList<T> &second,
								IndexedMatrixList &firstCameras,		IndexedMatrixList &secondCameras);

	cimg_library::CImgList<double> accumulateCentroids(	std::vector< hekate::Correspondence<T> > &pointCloud1, 
														std::vector< hekate::Correspondence<T> > &pointCloud2, bool evMedian);

	cimg_library::CImg<double> getPointSetHomography( std::vector< hekate::Correspondence<T> > &pointCloud1, 
										std::vector< hekate::Correspondence<T> > &pointCloud2, 
										const cimg_library::CImg<double> T1, 
										const cimg_library::CImg<double> T2);

};

template <typename T>
bool pointCloudCompareSize(const hekate::CorrespondenceList<T> &a, const hekate::CorrespondenceList<T> &b);

#endif // _MERGEPOINTCLOUDS_H_
