/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file MeshPainter.h
 *  Declaration of the parameter class MeshPainter.
 *  \author Moritz Becker
 *  \date 08.12.2010
 */

#ifndef _MESHPAINTER_H_
#define _MESHPAINTER_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef meshpainter_EXPORTS
/// Visual C++ specific code
#define meshpainter_DECLDIR __declspec(dllexport)
#else
#define meshpainter_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define meshpainter_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Paints back meshes and depths to the original image using indices and features from delaunay plugin.
template <typename T>
class meshpainter_DECLDIR MeshPainter :
		public TemplatedParameteredObject<T> {
public:
	/// withMesh 
	Parameter < bool > withMesh;
	/// withDepth 
	Parameter < bool > withDepth;	

	/// FeaturesWithDepth 
	InputSlot < std::vector<std::vector<T> > > featuresWithDepth;
	/// Indices 
	InputSlot < std::vector<std::vector<int> > > indices;
	/// image 
	InputSlot < cimg_library::CImgList<T> > image;
	/// outputImage 
	OutputSlot < cimg_library::CImgList<T> > outputImage;
	

	/// create a new MeshPainter object
	/// \param name          Instance name
	MeshPainter(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _MESHPAINTER_H_



