/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file MeshPainter.hxx
 *  Implementation of the parameter class MeshPainter.
 *  \author Moritz Becker
 *  \date 08.12.2010
 */

#ifndef _MESHPAINTER_HXX_
#define _MESHPAINTER_HXX_

#include <hekate/MeshPainter.h>

template <typename T>
MeshPainter<T>::MeshPainter(const std::string& name) :
		TemplatedParameteredObject<T>(
				"MeshPainter", name,
				"Paints back meshes and depths to the original image using indices and features from delaunay plugin."
		)
			,featuresWithDepth(false, false) 
			,indices(false, false) 
			,image(false, false) 
			
			
{
	ParameteredObject::_addParameter (withMesh, "withMesh", 
	"withMesh", 
	true);
	ParameteredObject::_addParameter (withDepth, "withDepth", 
	"withDepth", 
	false);
	

	

	ParameteredObject::_addInputSlot(featuresWithDepth, "featuresWithDepth", 
	"FeaturesWithDepth", 
	"vector<vector<T> >"); 
	ParameteredObject::_addInputSlot(indices, "indices", 
	"Indices", 
	"vector<vector<int> >"); 
	ParameteredObject::_addInputSlot(image, "image", 
	"image", 
	"CImgList<T>"); 
	ParameteredObject::_addOutputSlot(outputImage, "outputImage", 
	"outputImage", 
	"CImgList<T>"); 
	
}

template <typename T>
void MeshPainter<T>::execute() {
	PARAMETEREDOBJECT_AVOID_REEXECUTION;
	ParameteredObject::execute();

	// create copy of input image
	cimg_library::CImg<T> img, img2;
	img = this->image()[0];
	img2 = this->image()[1];

	// convert input point coord. to
	// original values since they where
	// normalized by homogeneous coord.
	cimg_library::CImgList<T> points;
	for(unsigned i=0; i<this->featuresWithDepth().size(); i++){
		cimg_library::CImg<T> ptc(1,3);
		ptc[0] = this->featuresWithDepth()[i][0];
		ptc[1] = this->featuresWithDepth()[i][1];
		ptc[2] = this->featuresWithDepth()[i][2];
		points.push_back(ptc);
	}

	// define colors
	const unsigned char red[] = { 255,0,0 };

	if(this->withMesh){
		for(unsigned int i=0; i<this->indices().size(); i++){
			img.draw_line(points[indices()[i][0]][0],points[indices()[i][0]][1],
				points[indices()[i][1]][0],points[indices()[i][1]][1],red);

			img.draw_line(points[indices()[i][1]][0],points[indices()[i][1]][1],
				points[indices()[i][2]][0],points[indices()[i][2]][1],red);

			img.draw_line(points[indices()[i][2]][0],points[indices()[i][2]][1],
				points[indices()[i][0]][0],points[indices()[i][0]][1],red);
		
		}
	}

	this->outputImage().push_back(img);


}

#endif /* _MESHPAINTER_HXX_ */



