/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file MeshTriangulationPCL.h
 *  Declaration of the parameter class MeshTriangulationPCL.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 15.03.2012
 */

#ifndef _MESHTRIANGULATIONPCL_H_
#define _MESHTRIANGULATIONPCL_H_

#ifdef _MSC_VER
#ifdef meshtriangulationpcl_EXPORTS
/// Visual C++ specific code
#define meshtriangulationpcl_DECLDIR __declspec(dllexport)
#else
#define meshtriangulationpcl_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define meshtriangulationpcl_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>
#include <pcl/point_types.h>
#include <pcl/io/pcd_io.h>
#include <pcl/kdtree/kdtree_flann.h>
#include <pcl/features/normal_3d.h>
#include <pcl/surface/gp3.h>
#include <pcl/io/vtk_io.h>

using namespace hekate;
using namespace cimg_library;

/// Generates a mesh using the PCL
/** Generates a mesh by fast triangulation of unordered point cloud, using the
 *  point cloud library.
 */
template <typename T>
class meshtriangulationpcl_DECLDIR MeshTriangulationPCL :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	MeshTriangulationPCL(const std::string& name = "");

	/// A 3D point cloud
	InputSlot< hekate::CorrespondenceList<T> > pointCloud;

	/// Camera matrices
	InputSlot< IndexedMatrixList > cameraMatrices;

	/// Indices of point mesh
	OutputSlot< std::vector<std::vector<int>> > pointIndex;

	/// Update object.
	virtual void execute();


};

#endif // _MESHTRIANGULATIONPCL_H_
