/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file MultiPointCloudExtractor.h
 *  Declaration of the parameter class MultiPointCloudExtractor.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 29.04.2013
 */

#ifndef _MULTIPOINTCLOUDEXTRACTOR_H_
#define _MULTIPOINTCLOUDEXTRACTOR_H_

#ifdef _MSC_VER
#ifdef multipointcloudextractor_EXPORTS
/// Visual C++ specific code
#define multipointcloudextractor_DECLDIR __declspec(dllexport)
#else
#define multipointcloudextractor_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define multipointcloudextractor_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>

/// Extracts specific index entry of multipointcloud and multicameramtrixlist
/** Extracts specific index entry of multipointcloud and multicameramtrixlist
 */
template <typename T>
class multipointcloudextractor_DECLDIR MultiPointCloudExtractor :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	MultiPointCloudExtractor(const std::string& name = "");

	/// multiCameras
	InputSlot< std::vector<IndexedMatrixList> > multiCameras;
	/// multiPointClouds
	InputSlot< std::vector<hekate::CorrespondenceList<T>> > multiPointClouds;

	/// cameras of given index
	OutputSlot< IndexedMatrixList > cameras;
	/// pointCloud of given index
	OutputSlot< hekate::CorrespondenceList<T> > pointCloud;

	/// Index of set to extract
	Parameter< unsigned int > index;


protected:
	/// Update object.
	virtual void execute();
};

#endif // _MULTIPOINTCLOUDEXTRACTOR_H_
