/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file MultiPointCloudExtractor.hxx
 *  Implementation of the parameter class MultiPointCloudExtractor.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 29.04.2013
 */

#ifndef _MULTIPOINTCLOUDEXTRACTOR_HXX_
#define _MULTIPOINTCLOUDEXTRACTOR_HXX_

#include <charon-core/ParameteredObject.hxx>
#include "MultiPointCloudExtractor.h"

template <typename T>
MultiPointCloudExtractor<T>::MultiPointCloudExtractor(const std::string& name) :
		TemplatedParameteredObject<T>(
			"MultiPointCloudExtractor", name,
			"<h2>Extracts specific index entry of multipointcloud and "
			"multicameramtrixlist</h2><br>"
			"Extracts specific index entry of multipointcloud and "
			"multicameramtrixlist"
		)
{
	ParameteredObject::_addInputSlot(
		multiCameras, "multiCameras",
		"multiCameras",
		"vector<IndexedMatrixList>");
	ParameteredObject::_addInputSlot(
		multiPointClouds, "multiPointClouds",
		"multiPointClouds",
		"vector<CorrespondenceList>");

	ParameteredObject::_addOutputSlot(
		cameras, "cameras",
		"cameras of given index",
		"IndexedMatrixList");
	ParameteredObject::_addOutputSlot(
		pointCloud, "pointCloud",
		"pointCloud of given index",
		"CorrespondenceList");

	ParameteredObject::_addParameter< unsigned int >(
		index, "index",
		"Index of set to extract",
		0, "uint");

}

template <typename T>
void MultiPointCloudExtractor<T>::execute() {

	this->cameras() = this->multiCameras()[ this->index() ];
	this->pointCloud() = this->multiPointClouds()[ this->index() ];

}

#endif /* _MULTIPOINTCLOUDEXTRACTOR_HXX_ */
