/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file NVMReader.h
 *  Declaration of the parameter class NVMReader.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 19.02.2013
 */

#ifndef _NVMREADER_H_
#define _NVMREADER_H_

#ifdef _MSC_VER
#ifdef nvmreader_EXPORTS
/// Visual C++ specific code
#define nvmreader_DECLDIR __declspec(dllexport)
#else
#define nvmreader_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define nvmreader_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>
#include <pba/pba.h>
#include <pba/util.h>
#include <pba/DataInterface.h>

using namespace hekate;
using namespace cimg_library;

/// Reads NVM files from Visual SfM
/** Read NVM files from Visual SfM and forwards camera matrices and point
 *  lists.
 */
template <typename T>
class nvmreader_DECLDIR NVMReader :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	NVMReader(const std::string& name = "");

	/// image paths in increasing name order
	InputSlot< std::vector<std::string> > imagePathSource;

	/// cameras
	OutputSlot< IndexedMatrixList > cameras;
	/// correspondences
	OutputSlot< hekate::CorrespondenceList<T> > correspondences;
	/// points3D
	OutputSlot< hekate::CorrespondenceList<T> > points3D;
	
	/// filename to read NVM from
	Parameter<std::string> filename;
	Parameter<int> width;
	Parameter<int> height;

protected:
	/// Update object.
	virtual void execute();
};

#endif // _NVMREADER_H_
