/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file NormalizeLocal.h
 *  Declaration of the parameter class NormalizeLocal.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 16.10.2013
 */

#ifndef _NORMALIZELOCAL_H_
#define _NORMALIZELOCAL_H_

#ifdef _MSC_VER
#ifdef normalizelocal_EXPORTS
/// Visual C++ specific code
#define normalizelocal_DECLDIR __declspec(dllexport)
#else
#define normalizelocal_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define normalizelocal_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Normalizes image by local histograms
/** This module takes local histograms of a chosen size and accordingly
 *  normalizes the image.
 */
template <typename T>
class normalizelocal_DECLDIR NormalizeLocal :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	NormalizeLocal(const std::string& name = "");

	/// Images to normalize
	InputSlot< cimg_library::CImgList<T> > images;

	/// Normalized images
	OutputSlot< cimg_library::CImgList<T> > normalized;
	
	/// Width and height of the patches used for local histograms.
	Parameter< int > patchSize;
	/// Blur before normalize
	Parameter< float > blur;
	/// Weight pixels in normalization step by its distance to center pixel
	Parameter< bool > weightByDistance;
	/// Weight for pixels with maximal distance
	Parameter< float > weight;
	/// Normalize channels seperately
	Parameter< bool > useSingleChannels;
	/// Convert to HSV, normalize and convert back to RGB
	Parameter< bool > useHSV;

protected:
	/// Update object.
	virtual void execute();
};

#endif // _NORMALIZELOCAL_H_
