/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file PCLViewer.h
 *  Declaration of the parameter class PCLViewer.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 05.04.2012
 */

#ifndef _PCLVIEWER_H_
#define _PCLVIEWER_H_

#ifdef _MSC_VER
#ifdef pclviewer_EXPORTS
/// Visual C++ specific code
#define pclviewer_DECLDIR __declspec(dllexport)
#else
#define pclviewer_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define pclviewer_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <pcl/visualization/cloud_viewer.h>
#include <hekate/Correspondence.hxx>
#include <iostream>
#include <pcl/io/io.h>
#include <pcl/io/pcd_io.h>

using namespace hekate;
using namespace cimg_library;

/// Takes a CorrepondenceList object and draws a 3d point cloud
/** Takes a CorrepondenceList object and draws a 3d point cloud. Plugin is
 *  based on PCL.
 */
template <typename T>
class pclviewer_DECLDIR PCLViewer :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	PCLViewer(const std::string& name = "");

	/// A 3d point cloud of the class CorrespondenceList<T>
	InputSlot< CorrespondenceList<T> > pointCloud;

	/// Draw outliers to 3d point cloud
	Parameter< bool > drawOutliers;

	/// Update object.
	virtual void execute();
};

#endif // _PCLVIEWER_H_
