/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file PCLViewer.hxx
 *  Implementation of the parameter class PCLViewer.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 05.04.2012
 */

#ifndef _PCLVIEWER_HXX_
#define _PCLVIEWER_HXX_

#include <charon-core/ParameteredObject.hxx>
#include "PCLViewer.h"

template <typename T>
PCLViewer<T>::PCLViewer(const std::string& name) :
		TemplatedParameteredObject<T>(
			"PCLViewer", name,
			"<h2>Takes a CorrepondenceList object and draws a 3d point "
			"cloud</h2><br>"
			"Takes a CorrepondenceList object and draws a 3d point cloud. "
			"Plugin is based on PCL."
		)
{
	ParameteredObject::_addInputSlot(
		pointCloud, "pointCloud",
		"A 3d point cloud of the class CorrespondenceList<T>",
		"CorrespondenceList");

	ParameteredObject::_addParameter(
		drawOutliers, "drawOutliers",
		"Draw outliers to 3d point cloud",
		"bool");

}

template <typename T>
void PCLViewer<T>::execute() {

	pcl::PointCloud<pcl::PointXYZRGBA>::Ptr cloud (new pcl::PointCloud<pcl::PointXYZRGBA>);
	auto it = this->pointCloud().begin();
	auto itEnd = this->pointCloud().end();
	for(; it != itEnd; it++)
	{
		if(!it->outlier)
		{
			pcl::PointXYZRGBA pt;
			pt.x = it->x() ;
			pt.y = it->y() ;
			pt.z = it->z() ;
			pt.r = it->R;
			pt.g = it->G;
			pt.b = it->B;
			cloud->push_back(pt);
		}
		else if( this->drawOutliers() )
		{
			pcl::PointXYZRGBA pt;
			pt.x = it->x() ;
			pt.y = it->y() ;
			pt.z = it->z() ;
			pt.r = 255;
			pt.g = 0;
			pt.b = 0;
			cloud->push_back(pt);
		}
	}
	pcl::visualization::CloudViewer viewer("Point Viewer");
	viewer.runOnVisualizationThreadOnce(setPointSize);
	viewer.showCloud(cloud->makeShared());
	
}

void setPointSize(pcl::visualization::PCLVisualizer &viewer)
{
	viewer.setPointCloudRenderingProperties(pcl::visualization::PCL_VISUALIZER_POINT_SIZE, 12.0);
}

#endif /* _PCLVIEWER_HXX_ */
