/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file ParallelBundleAdjustment.h
 *  Declaration of the parameter class ParallelBundleAdjustment.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 29.02.2012
 */

#ifndef _PARALLELBUNDLEADJUSTMENT_H_
#define _PARALLELBUNDLEADJUSTMENT_H_

#ifdef _MSC_VER
#ifdef parallelbundleadjustment_EXPORTS
/// Visual C++ specific code
#define parallelbundleadjustment_DECLDIR __declspec(dllexport)
#else
#define parallelbundleadjustment_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define parallelbundleadjustment_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>
#include <pba/pba.h>
#include <pba/util.h>

using namespace hekate;
using namespace std;
using namespace cimg_library;

/// Performs a bundle adjustment for a given set Cameras, 3DPoints, 2DPoints
/** Performs a bundle adjustment for a given set Cameras, 3DPoints, 2DPoints
 *  using several threads
 */
template <typename T>
class parallelbundleadjustment_DECLDIR ParallelBundleAdjustment :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	ParallelBundleAdjustment(const std::string& name = "");
	
	/// Minimum number of views, a point is visible in
	Parameter < int > minPtViews; 	
	/// Focal length is fixed for all cameras (calibrated case).
	Parameter < bool > fixedFocalLength; 		
	/// Use CUDA for GPU calculation
	Parameter < bool > useCUDA; 		
	/// Calculate consistent data by taking the overlap between image and point indices of 2d, 3d and cams
	Parameter < bool > calculateDataOverlap; 		
	/// Flag cameras as constant. Only 3D points will be modified by PBA
	Parameter < bool > constantCameras; 	
	
	/// Projection matrices
	InputSlot< IndexedMatrixList > cameraMatrices;
	/// calibrationMatrix
	InputSlot< CImgList<T> > calibrationMatrix;
	/// 3D point clouds
	InputSlot< CorrespondenceList<T> > pointCloud;
	/// 2D correspondences
	InputSlot< CorrespondenceList<T> > correspondences;

	/// Adjusted camera matrices
	OutputSlot< IndexedMatrixList > cameraMatricesOut;
	/// Adjusted 3D point clouds
	OutputSlot< CorrespondenceList<T> > pointCloudOut;
	/// calibrationMatrixOut
	OutputSlot< CImgList<T> > calibrationMatrixOut;

	/// Update object.
	virtual void execute();

};

#endif // _PARALLELBUNDLEADJUSTMENT_H_
