/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Plot3Dpoints.h
 *  Declaration of the parameter class Plot3Dpoints.
 *  \author Moritz Becker
 *  \date 20.10.2010
 */

#ifndef _PLOT3DPOINTS_H_
#define _PLOT3DPOINTS_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef plot3dpoints_EXPORTS
/// Visual C++ specific code
#define plot3dpoints_DECLDIR __declspec(dllexport)
#else
#define plot3dpoints_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define plot3dpoints_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Plots 3D points to 2D images.
template <typename T>
class plot3dpoints_DECLDIR Plot3Dpoints :
		public TemplatedParameteredObject<T> {
public:
	/// imageSize 
	Parameter < int > imageSize;	

	/// inputPoints 
	InputSlot < std::vector<std::vector<T> > > inputPoints;
	/// outputImages 
	OutputSlot < cimg_library::CImgList<T> > outputImages;
	

	/// create a new Plot3Dpoints object
	/// \param name          Instance name
	Plot3Dpoints(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _PLOT3DPOINTS_H_



