/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Plot3Dpoints.hxx
 *  Implementation of the parameter class Plot3Dpoints.
 *  \author Moritz Becker
 *  \date 20.10.2010
 */

#ifndef _PLOT3DPOINTS_HXX_
#define _PLOT3DPOINTS_HXX_

#include <hekate/Plot3Dpoints.h>

template <typename T>
Plot3Dpoints<T>::Plot3Dpoints(const std::string& name) :
		TemplatedParameteredObject<T>(
				"testPlot3Dpoints", name,
				"Plots 3D points to 2D images."
		)
			,inputPoints(false, false) 
			
			
{
	ParameteredObject::_addParameter (imageSize, "imageSize", 
	"imageSize", 
	1200);	

	ParameteredObject::_addInputSlot(inputPoints, "inputPoints", 
	"inputPoints", 
	"vector<vector<T> >"); 
	ParameteredObject::_addOutputSlot(outputImages, "outputImages", 
	"outputImages", 
	"CImgList<T>"); 
	
}

template <typename T>
void Plot3Dpoints<T>::execute() {
	PARAMETEREDOBJECT_AVOID_REEXECUTION;
	ParameteredObject::execute();

	//create output images
	int imageSize = this->imageSize();
	cimg_library::CImg<T> outputImage1(imageSize,imageSize), outputImage2, outputImage3;
	outputImage1.fill(255);
	outputImage2 = outputImage1;
	outputImage3 = outputImage1;
	const unsigned char black[] = { 0,0,0 } ;

	double maxX=0., maxY=0., maxZ=0.;

	size_t pointSize = this->inputPoints().size();
	
	for(size_t i=0; i<pointSize; i++){

		double x = this->inputPoints()[i][0];
		double y = this->inputPoints()[i][1];
		double z = this->inputPoints()[i][2];

		if(std::abs(x) > std::abs(maxX)) maxX = x;
		if(std::abs(y) > std::abs(maxY)) maxY = y;
		if(std::abs(z) > std::abs(maxZ)) maxZ = z;
	
		/*
		sumX += pow(pow(x,2.),0.25);
		sumY += pow(pow(y,2.),0.25);
		sumZ += pow(pow(y,2.),0.25);
		*/
			
	}

	double maxCoord = maxX;
	if(std::abs(maxY) > std::abs(maxCoord)) maxCoord = maxY;
	if(std::abs(maxZ) > std::abs(maxCoord)) maxCoord = maxZ;

	// normalize coord values
	for(int i=0; i<pointSize; i++){

		char txt[32];
		std::sprintf(txt,"%u",i);

		double x = std::abs(this->inputPoints()[i][0] / maxCoord * imageSize);
		double y = std::abs(this->inputPoints()[i][1] / maxCoord * imageSize);
		double z = std::abs(this->inputPoints()[i][2] / maxCoord * imageSize);

		sout << "(" << x << ", " << y << ", " << z << ")" << std::endl;

		outputImage1.draw_text(x,y,txt,black);
		outputImage2.draw_text(x,z,txt,black);
		outputImage3.draw_text(y,z,txt,black);

	}

	this->outputImages().push_back(outputImage1);
	this->outputImages().push_back(outputImage2);
	this->outputImages().push_back(outputImage3);

}

#endif /* _PLOT3DPOINTS_HXX_ */



