/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file 3Dpoints2orderedsequence.hxx
 *  Implementation of the parameter class 3Dpoints2orderedsequence.
 *  \author Moritz Becker
 *  \date 12.07.2010
 */

#ifndef _POINTS2ORDEREDSEQUENCE_HXX_
#define _POINTS2ORDEREDSEQUENCE_HXX_

#include <hekate/Points2OrderedSequence.h>

template <typename T>
points2orderedsequence<T>::points2orderedsequence(const std::string& name) :
		TemplatedParameteredObject<T>(
				"Points2orderedsequence", name,
				"merges two 3D point clouds to a vector<vector<vector<..>>>"
		)
			,points1(false, false) 
			,points2(false, false) 			
{
	ParameteredObject::_addInputSlot(points1, "3Dpoints1", 
	"", 
	"vector<vector<T> >"); 
	ParameteredObject::_addInputSlot(points2, "3Dpoints2", 
	"", 
	"vector<vector<T> >"); 
	ParameteredObject::_addOutputSlot(points, "3Dpoints", 
	"", 
	"vector<vector<vector<T> > >"); 	
	
	ParameteredObject::_setTags("Hekate") ;
}

template <typename T>
void points2orderedsequence<T>::execute() {
	PARAMETEREDOBJECT_AVOID_REEXECUTION;
	ParameteredObject::execute();

	for(size_t i = 0; i < this->points1().size(); i++){
	
		std::vector<std::vector<T> > container;
		container.push_back(this->points1()[i]);
		container.push_back(this->points2()[i]);
		this->points().push_back(container);

		sout << "Points2Sequence: (" << this->points1()[i][0] << ", " 
				<< this->points1()[i][1] << ", "
				<< this->points1()[i][2] << ") / ("
				<< this->points2()[i][0] << ", " 
				<< this->points2()[i][1] << ", "
				<< this->points2()[i][2] << ")" << std::endl;
	}

}

#endif /* _3DPOINTS2ORDEREDSEQUENCE_HXX_ */



