/*  Copyright (C) 2009 SasiKanth

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file projectivesba.h
 *  Declaration of the parameter class projectivesba.
 *  \author SasiKanth
 *  \date 08.07.2010
 */

#ifndef _PROJECTIVESBA_H_
#define _PROJECTIVESBA_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef projectivesba_EXPORTS
/// Visual C++ specific code
#define projectivesba_DECLDIR __declspec(dllexport)
#else
#define projectivesba_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define projectivesba_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// This plugin performs sparse bundle adjustment on a projective camera using the sba package
template <typename T>
class projectivesba_DECLDIR projectivesba :
		public TemplatedParameteredObject<T> {
public:
	

	

	/// The set of input 3D points 
	InputSlot < std::vector<std::vector<T> > > input_3d_points;
	/// The set of input correspondences 
	InputSlot < std::vector<std::vector<std::pair<T, T> > > > input_correspondences;
	/// The set of camera matrices 
	InputSlot < cimg_library::CImgList<T> > input_p_matrices;
	/// The adjusted 3D points 
	OutputSlot < std::vector<std::vector<T> > > output_3d_points;
	/// The adjusted camera matrices 
	OutputSlot < cimg_library::CImgList<T> > output_p_matrices;
	

	/// create a new projectivesba object
	/// \param name          Instance name
	projectivesba(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _PROJECTIVESBA_H_



