/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file RGB2Grey.h
 *  Declaration of the parameter class RGB2Grey.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 24.10.2013
 */

#ifndef _RGB2GREY_H_
#define _RGB2GREY_H_

#ifdef _MSC_VER
#ifdef rgb2grey_EXPORTS
/// Visual C++ specific code
#define rgb2grey_DECLDIR __declspec(dllexport)
#else
#define rgb2grey_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define rgb2grey_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Converts RGB values to grey values
/** Converts RGB values to grey values
 */
template <typename T>
class rgb2grey_DECLDIR RGB2Grey :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	RGB2Grey(const std::string& name = "");

	/// RGB
	InputSlot< cimg_library::CImgList<T> > in_rgb;

	/// Grey values
	OutputSlot< cimg_library::CImgList<T> > out_grey;

	/// weight
	Parameter< float > r;
	/// weight
	Parameter< float > g;
	/// weight
	Parameter< float > b;


protected:
	/// Update object.
	virtual void execute();
};

#endif // _RGB2GREY_H_
