/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file RGB2Grey.hxx
 *  Implementation of the parameter class RGB2Grey.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 24.10.2013
 */

#ifndef _RGB2GREY_HXX_
#define _RGB2GREY_HXX_

#include <charon-core/ParameteredObject.hxx>
#include "RGB2Grey.h"

template <typename T>
RGB2Grey<T>::RGB2Grey(const std::string& name) :
		TemplatedParameteredObject<T>(
			"RGB2Grey", name,
			"<h2>Converts RGB values to grey values</h2><br>"
			"Converts RGB values to grey values"
		)
{
	ParameteredObject::_addInputSlot(
		in_rgb, "in_rgb",
		"RGB",
		"CImgList<T>");

	ParameteredObject::_addOutputSlot(
		out_grey, "out_grey",
		"Grey values",
		"CImgList<T>");

	ParameteredObject::_addParameter< float >(
		r, "r",
		"weight",
		1, "float");
	ParameteredObject::_addParameter< float >(
		g, "g",
		"weight",
		1, "float");
	ParameteredObject::_addParameter< float >(
		b, "b",
		"weight",
		1, "float");

}

template <typename T>
void RGB2Grey<T>::execute() {

	if( this->in_rgb().size() != 0 ){
		if( this->in_rgb()[0].spectrum() > 0 ){

			sout << "image has " << this->in_rgb()[0].spectrum() << " channels." << std::endl;
			if( this->in_rgb()[0].spectrum() > 3 )
				sout << this->in_rgb()[0].spectrum() - 3 << " channels won't be considered for conversion" << std::endl;
	
			float r = std::max( float(0), this->r() );
			float g = std::max( float(0), this->g() );
			float b = std::max( float(0), this->b() );

			float sum = r + g + b;
			r /= sum ;
			g /= sum ;
			b /= sum ;
			sout << "Weighting channels by R=" << r << ", G=" << g << ", B=" << b << std::endl;

			for( unsigned int i = 0; i < this->in_rgb().size(); i++ ){
				const cimg_library::CImg<float> &img = this->in_rgb()[i];
				cimg_library::CImg<float> grey(img.width(),img.height(),1,1,0);
				cimg_forXYC(grey,x,y,c){
					if( c == 0 )
						grey(x,y,0,c) += r * img(x,y,0,c);
					else if( c == 1 )
						grey(x,y,0,c) += g * img(x,y,0,c);
					else if( c == 2 )
						grey(x,y,0,c) += b * img(x,y,0,c) ;

				}

				this->out_grey().push_back( grey.normalize(0,255) ) ;
	
			}

		}
		else{
			sout << "No RGB image!" << std::endl;
		}
	}

}

#endif /* _RGB2GREY_HXX_ */
