/*  Copyright (C) 2009 Julian Coordts

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file RectifyUncallibratedImages.h
 *  Declaration of the parameter class rectifyUncallibratedImages.
 *  \author Julian Coordts
 *  \date 11.03.2010
 */

#ifndef _rectifyUNCALLIBRATEDIMAGES_H_
#define _rectifyUNCALLIBRATEDIMAGES_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef rectifyuncallibratedimages_EXPORTS
/// Visual C++ specific code
#define rectifyuncallibratedimages_DECLDIR __declspec(dllexport)
#else
#define rectifyuncallibratedimages_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define rectifyuncallibratedimages_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include <levmar/levmar.h>
#include <hekate/LevmarSolverBC.h>

///Rectifies 2 Images using the point correspondences
 template <typename T>
class rectifyuncallibratedimages_DECLDIR rectifyUncallibratedImages :
		public TemplatedParameteredObject<T> {
public:
	///boolean to avoid multiple calls
	bool called;

	//Input Slots:

		///The F matrix(just used for calculating Sampson error) not necessary in general
		InputSlot < cimg_library::CImgList<T> > fmatrix;
		/// the correspondences
		InputSlot < std::vector<std::vector<std::pair<T, T> > > > feature_correspondences;
		/// the input image sequence (first left, second right)
		InputSlot < cimg_library::CImgList<T> > image_input;
		///pointer to the solver to use for the minimization problem
		InputSlot < lsqnonlinsolver<T>* > solver;

	///The output matrices
	OutputSlot < cimg_library::CImgList<T> > warp_matrices;
	/// automatic centering
	Parameter<bool> cntr;

	/// create a new rectifyUncallibratedImages object
	/// \param name          Instance name
	rectifyUncallibratedImages(const std::string& name);

	///calculates the homographies for left and right image
	void calcHlHr(double p[6],
				  T width,
				  T height,
				  cimg_library::CImg<T> &Hl,
				  cimg_library::CImg<T> &Hr);
	///calculates the bounding box of the Image using homographie h
	cimg_library::CImg<T> calcBB(cimg_library::CImg<T>&H,
							 	cimg_library::CImg<T>&Image);
	///computes a rotation matrix given three Euler angles
	cimg_library::CImg<T> eulR(T xa, T ya, T za);
	///Return the skew-symmetric matrix X s.t. cross(x,y) = X*y
	cimg_library::CImg<T> star(T x1, T x2, T x3);
	///cost callback function for levmar
	static void costRectif(double *p, double *x, int m, int n, void *data);
    ///pointer to the initializing Instance of this
	static void* callBackObject;
	///compute Sampson error
	cimg_library::CImg<T> sampson(cimg_library::CImg<T> &F,
								  cimg_library::CImg<T> &m1,
								  cimg_library::CImg<T> &m2);
	///Output a cImg matrix to sout
	void soutCImgMatrix(cimg_library::CImg<T> &m,const char* text);
	///outputs an array vector in human readable format through sout
    void soutVectorArray(const double *ar, const int length, const char* text);

	/// Update object.
	virtual void execute();

	///container for data transfer to the callback function through levmar
	struct costData
	{
		///width
		int w;
		///height
		int h;
		///matches left
		cimg_library::CImg<T>* ml;
		///matches right
		cimg_library::CImg<T>* mr;
	};
};
		template <typename T>
		 void* rectifyUncallibratedImages<T>::callBackObject = NULL;

#endif // _rectifyUNCALLIBRATEDIMAGES_H_



