/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file SelectCImgById.h
 *  Declaration of the parameter class SelectCImgById.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 21.02.2013
 */

#ifndef _SELECTCIMGBYID_H_
#define _SELECTCIMGBYID_H_

#ifdef _MSC_VER
#ifdef selectcimgbyid_EXPORTS
/// Visual C++ specific code
#define selectcimgbyid_DECLDIR __declspec(dllexport)
#else
#define selectcimgbyid_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define selectcimgbyid_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Selects one or more CImg by IDs and push it to CImgList
/** Selects one or more CImg by IDs and push it to CImgList.
 *  The ID(s) are given by a parameter list (id;id2;...;id_n)
 */
template <typename T>
class selectcimgbyid_DECLDIR SelectCImgById :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	SelectCImgById(const std::string& name = "");

	/// Image list to select from
	InputSlot< cimg_library::CImgList<T> > images;

	/// Selected images
	OutputSlot< cimg_library::CImgList<T> > imagesOut;

	/// List of indices to select images
	ParameterList< int > indices;


protected:
	/// Update object.
	virtual void execute();
};

#endif // _SELECTCIMGBYID_H_
