/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file SelectCImgById.hxx
 *  Implementation of the parameter class SelectCImgById.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 21.02.2013
 */

#ifndef _SELECTCIMGBYID_HXX_
#define _SELECTCIMGBYID_HXX_

#include <charon-core/ParameteredObject.hxx>
#include <hekate/SelectCImgById.h>

template <typename T>
SelectCImgById<T>::SelectCImgById(const std::string& name) :
		TemplatedParameteredObject<T>(
			"SelectCImgById", name,
			"<h2>Selects one or more CImg by IDs and push it to "
			"CImgList</h2><br>"
			"Selects one or more CImg by IDs and push it to CImgList.<br>"
			"The ID(s) are given by a parameter list (id;id2;...;id_n)"
		)
{
	ParameteredObject::_addInputSlot(
		images, "images",
		"Image list to select from",
		"CImgList<T>");

	ParameteredObject::_addOutputSlot(
		imagesOut, "imagesOut",
		"Selected images",
		"CImgList<T>");

	ParameteredObject::_addParameter(
		indices, "indices",
		"List of indices to select images",
		"int");

}

template <typename T>
void SelectCImgById<T>::execute() {
	
	const unsigned int nimg = this->images().size();
	const unsigned int nid = this->indices().size();
	int nimgOut = 0;

	// Check parameter list
	if( nid == 0 )
		throw( std::runtime_error("No indices given!") );
	if( nid > nimg )
		throw( std::range_error("More indices than images given!") );

	// Select images by index
	for( size_t i = 0; i < nid; i++)
	{

		const int id = this->indices()[i] ;

		// If index out of range, continue with next index
		if( id > nimg )
		{
			sout << "Attention: No image with index " << id << " available" << std::endl;
			continue;
		}

		// Save image with current index
		this->imagesOut().push_back( this->images()[ this->indices()[i] ] );
		nimgOut++;

	}

	sout << nimgOut << " images selected from CImgList." << std::endl;

}

#endif /* _SELECTCIMGBYID_HXX_ */
