/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file SortOutPoints.h
 *  Declaration of the parameter class SortOutPoints.
 *  \author Moritz Becker
 *  \date 17.01.2011
 */

#ifndef _SORTOUTPOINTS_H_
#define _SORTOUTPOINTS_H_

#ifdef _MSC_VER
#ifdef sortoutpoints_EXPORTS
/// Visual C++ specific code
#define sortoutpoints_DECLDIR __declspec(dllexport)
#else
#define sortoutpoints_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define sortoutpoints_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>

using namespace hekate;

/// Synchronizes 2d and 3d input outliers (which are marked as 0-vectors) and gives out cleaned up vectors.
/** The plugin compares according 2D and 3D points and removes those pairs of 2D and 3D vectors, where
	at least one point is a 0-vector (Other plugins can mark outliers by setting all values of a vector to 0). */
template <typename T>
class sortoutpoints_DECLDIR SortOutPoints :
		public TemplatedParameteredObject<T> {
public:
	
	/// Only reject manual outliers given by index list
	Parameter < bool > rejectManualOutliers;
	/// Index list of manual outliers
	ParameterList < double > manualOutliers;


	/// First point list
	InputSlot < typename CorrespondenceList<T> > pointList1;
	/// Second point list
	InputSlot < typename CorrespondenceList<T> > pointList2;
	/// First cleaned point list
	OutputSlot < typename CorrespondenceList<T> > pointList1out;
	/// Second cleaned point list
	OutputSlot < typename CorrespondenceList<T> > pointList2out;
	

	/// create a new SortOutPoints object
	/// \param name          Instance name
	SortOutPoints(const std::string& name ="");

	/// Update object.
	virtual void execute();
};

#endif // _SORTOUTPOINTS_H_



