/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file SortOutPointsByMask.hxx
 *  Implementation of the parameter class SortOutPointsByMask.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 24.01.2013
 */

#ifndef _SORTOUTPOINTSBYMASK_HXX_
#define _SORTOUTPOINTSBYMASK_HXX_

#include <charon-core/ParameteredObject.hxx>
#include <hekate/SortOutPointsByMask.h>

template <typename T>
SortOutPointsByMask<T>::SortOutPointsByMask(const std::string& name) :
		TemplatedParameteredObject<T>(
			"SortOutPointsByMask", name,
			"<h2>Masks correspondences</h2><br>Masks correspondences"
		)
{
	ParameteredObject::_addInputSlot(
		correspondences, "correspondences",
		"correspondences",
		"CorrespondenceList");
	ParameteredObject::_addInputSlot(
		mask, "mask",
		"mask",
		"vector<string>");

	ParameteredObject::_addOutputSlot(
		correspondencesOut, "correspondencesOut",
		"correspondencesOut",
		"CorrespondenceList");

}

template <typename T>
void SortOutPointsByMask<T>::execute() {
	
	if( this->correspondences().sizeImages() != this->mask().size() )
		throw( std::range_error("number of images in correspondence list != mask list size") ) ;

	for( size_t i = 0; i < this->mask().size(); i++)
	{
		sout << "Mask correspondences with image index " << i << std::endl;
		CImg<float> mask( this->mask()[i].c_str() );
		mask.normalize(0,255);
		auto it = correspondences().get<imgIndex>().lower_bound(i) ;
		auto itEnd = correspondences().get<imgIndex>().upper_bound(i) ;
		for(; it != itEnd ; it++)
		{
			if( mask( int(it->x()), int(it->y()) ) > 1 )
				this->correspondencesOut().push_back( (*it) );
		}
	}
}

#endif /* _SORTOUTPOINTSBYMASK_HXX_ */
