/*  Copyright (C) 2014 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file TrackAnnotationBoxesKLT.h
 *  \author <a href="mailto:moritz.becker@pallas-ludens.com">
 *      Moritz Becker</a>
 *  \date 03.06.2014
 *  Declaraion of parameter class TrackAnnotationBoxesKLT.
 */
#ifndef _TRACKANNOTATIONBOXESKLT_H_
#define _TRACKANNOTATIONBOXESKLT_H_

#ifdef _MSC_VER
#ifdef trackannotationboxesklt_EXPORTS
/// Visual C++ specific code
#define trackannotationboxesklt_DECLDIR __declspec(dllexport)
#else
#define trackannotationboxesklt_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define trackannotationboxesklt_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <charon-utils/Annotation.hxx>
#include <opencv/cv.h>
#include <opencv/highgui.h>
#include <hekate/openCvUtil.hxx>



#include <opencv/cv.h>
#include <opencv/highgui.h>
#include <opencv/cxcore.h>
#include <opencv2/core/core.hpp>
#include <opencv2/features2d/features2d.hpp>
#include <opencv2/highgui/highgui.hpp>
#include "opencv2/gpu/gpu.hpp"
#include "opencv2/nonfree/nonfree.hpp"

/// Track annotation boxes using KLT from OpenCV
/** Track annotation boxes using KLT from OpenCV
 */
class trackannotationboxesklt_DECLDIR TrackAnnotationBoxesKLT : public ParameteredObject {
public:
	/// default constructor
	/// \param name             instance name
	TrackAnnotationBoxesKLT(const std::string& name = "");

	/// Box objects
	InputSlot< annotation::AnnotationList > annotations;

	/// vector of image file names 
	InputSlot< std::vector< std::string > > filepaths;

	/// Tracked Boxes
	OutputSlot< annotation::AnnotationList > boxesTracked;

	/// Tracked Boxes
	OutputSlot< cimg_library::CImgList<double> > debug;

	/// Number of frames to track boxes
	Parameter< int > trackLength;

	/// Minimum width of height of annotation box
	Parameter< int > minBoxSize;

	/// Maximum width of height of annotation box
	Parameter< int > maxBoxSize;


protected:
	/// Update object.
	virtual void execute();
};

#endif /* _TRACKANNOTATIONBOXESKLT_H_ */
