/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file TransformationCImg.h
 *  Declaration of the parameter class TransformationCImg.
 *  \author Moritz Becker
 *  \date 05.11.2010
 */

#ifndef _TRANSFORMATIONCIMG_H_
#define _TRANSFORMATIONCIMG_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef transformationcimg_EXPORTS
/// Visual C++ specific code
#define transformationcimg_DECLDIR __declspec(dllexport)
#else
#define transformationcimg_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define transformationcimg_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Transforms a matrix by another matrix: M3 = M2 * M1.
template <typename T>
class transformationcimg_DECLDIR TransformationCImg :
		public TemplatedParameteredObject<T> {
public:	

	/// Matrix which will be transformed M1. 
	InputSlot < cimg_library::CImgList<T> > inputMatrix1;
	/// Transformation matrix M2. 
	InputSlot < cimg_library::CImgList<T> > inputMatrix2;
	/// Transformed matrix M3. 
	OutputSlot < cimg_library::CImgList<T> > outputmatrix;
	

	/// create a new TransformationCImg object
	/// \param name          Instance name
	TransformationCImg(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _TRANSFORMATIONCIMG_H_



