/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file TransformationCImg.hxx
 *  Implementation of the parameter class TransformationCImg.
 *  \author Moritz Becker
 *  \date 05.11.2010
 */

#ifndef _TRANSFORMATIONCIMG_HXX_
#define _TRANSFORMATIONCIMG_HXX_

#include <hekate/TransformationCImg.h>
#include <charon-utils/ImgTool.hxx>

template <typename T>
TransformationCImg<T>::TransformationCImg(const std::string& name) :
		TemplatedParameteredObject<T>(
				"TransformationCImg", name,
				"Transforms a matrix by another matrix: M3 = M2 * M1."
		)
			,inputMatrix1(false, false) 
			,inputMatrix2(false, false) 
			
			
{
	

	

	ParameteredObject::_addInputSlot(inputMatrix1, "M1", 
	"Matrix which will be transformed.", 
	"CImgList<T>"); 
	ParameteredObject::_addInputSlot(inputMatrix2, "M2", 
	"Transformation matrix.", 
	"CImgList<T>"); 
	ParameteredObject::_addOutputSlot(outputmatrix, "M3", 
	"Transformed matrix.", 
	"CImgList<T>"); 
	
	ParameteredObject::_setTags("Hekate") ;
	
}

template <typename T>
void TransformationCImg<T>::execute() {
	PARAMETEREDOBJECT_AVOID_REEXECUTION;
	ParameteredObject::execute();

	for(size_t i=0; i<this->inputMatrix1().size(); i++){
		for(size_t j=0; j<this->inputMatrix2().size(); j++){

			cimg_library::CImg<T> output;
			output = this->inputMatrix2()[j] * this->inputMatrix1()[i];

			this->outputmatrix().push_back(output);
			ImgTool::printInfo(sout,output,"TransformationCImg");

		}
	}



}

#endif /* _TRANSFORMATIONCIMG_HXX_ */



