/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file TransformationX3D.h
 *  Declaration of the parameter class TransformationX3D.
 *  \author Moritz Becker
 *  \date 14.02.2011
 */

#ifndef _TRANSFORMATIONX3D_H_
#define _TRANSFORMATIONX3D_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef transformationx3d_EXPORTS
/// Visual C++ specific code
#define transformationx3d_DECLDIR __declspec(dllexport)
#else
#define transformationx3d_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define transformationx3d_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Transforms 3D points by a matrix. (3D->3D Homography)
template <typename T>
class transformationx3d_DECLDIR TransformationX3D :
		public TemplatedParameteredObject<T> {
public:
	

	

	/// Input points.
	InputSlot < std::vector<std::vector<T> > > input3D;
	/// Transformation matrix.
	InputSlot < cimg_library::CImgList<T> > transformationMatrix;
	/// Transformed 3D points.
	OutputSlot < std::vector<std::vector<T> > > output3D;
	

	/// create a new TransformationX3D object
	/// \param name          Instance name
	TransformationX3D(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _TRANSFORMATIONX3D_H_



