/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file TransformationX3D.hxx
 *  Implementation of the parameter class TransformationX3D.
 *  \author Moritz Becker
 *  \date 14.02.2011
 */

#ifndef _TRANSFORMATIONX3D_HXX_
#define _TRANSFORMATIONX3D_HXX_

#include <hekate/TransformationX3D.h>
#include <charon-utils/ImgTool.hxx>

template <typename T>
TransformationX3D<T>::TransformationX3D(const std::string& name) :
		TemplatedParameteredObject<T>(
				"TransformationX3D", name,
				"Transforms 3D points by a matrix. (3D->3D Homography)"
		)
			,input3D(false, false) 
			,transformationMatrix(false, false) 
			
			
{	

	ParameteredObject::_addInputSlot(input3D, "input3D", 
	"Input points.", 
	"vector<vector<T> >"); 
	ParameteredObject::_addInputSlot(transformationMatrix, "transformationMatrix", 
	"Transformation matrix.", 
	"CImgList<T>"); 
	ParameteredObject::_addOutputSlot(output3D, "output3D", 
	"Transformed 3D points.", 
	"vector<vector<T> >"); 
	
	ParameteredObject::_setTags("Hekate") ;
	
}

template <typename T>
void TransformationX3D<T>::execute() {
	PARAMETEREDOBJECT_AVOID_REEXECUTION;
	ParameteredObject::execute();

	typename std::vector< std::vector<T> >::const_iterator pointsIt =
		this->input3D().begin();
	typename std::vector< std::vector<T> >::const_iterator pointsEnd =
		this->input3D().end();
	
	for(;pointsIt != pointsEnd ; pointsIt++){
		cimg_library::CImg<T> x1(1,4), x2(1,4);
		x1[0] = (*pointsIt)[0];
		x1[1] = (*pointsIt)[1];
		x1[2] = (*pointsIt)[2];
		x1[3] = 1.;
		x2 = this->transformationMatrix()[0] * x1;
		if(x2[3] != 0) x2 /= x2[3];
		std::vector<T> point;
		point.push_back(x2[0]);
		point.push_back(x2[1]);
		point.push_back(x2[2]);
		point.push_back(x2[3]);
		this->output3D().push_back(point);
		
		//ImgTool::printInfo(sout,x1,"x1");
		//ImgTool::printInfo(sout,x2,"x2");

	}

}

#endif /* _TRANSFORMATIONX3D_HXX_ */



