/*  Copyright (C) 2012 Michael Mller

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file TriangulationGPU.h
 *  Declaration of the parameter class TriangulationGPU.
 *  \author <a href="mailto:michael@fds-team.de">
 *      Michael Mller</a>
 *  \date 12.04.2012
 */

#ifndef _TriangulationGPU_H_
#define _TriangulationGPU_H_

#ifdef _MSC_VER
#ifdef triangulationgpu_EXPORTS
/// Visual C++ specific code
#define triangulationgpu_DECLDIR __declspec(dllexport)
#else
#define triangulationgpu_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define triangulationgpu_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>

/// Triangulates 2D points and camera matrices to 3D points
/** Triangulates 2D points and camera matrices to 3D points
 */
template <typename T>
class triangulationgpu_DECLDIR TriangulationGPU :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	TriangulationGPU(const std::string& name = "");

	/// correspondences
	InputSlot< hekate::CorrespondenceList<T> > correspondences;
	/// cameras
	InputSlot< IndexedMatrixList > cameras;

	/// pointcloud
	OutputSlot< hekate::CorrespondenceList<T> > pointcloud;

	/// Update object.
	virtual void execute();
};

#endif // _TriangulationGPU_H_
