/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file UpgradeMetric.h
 *  Declaration of the parameter class UpgradeMetric.
 *  \author Moritz Becker
 *  \date 05.08.2010
 */

#ifndef _UPGRADEMETRIC_H_
#define _UPGRADEMETRIC_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef upgrademetric_EXPORTS
/// Visual C++ specific code
#define upgrademetric_DECLDIR __declspec(dllexport)
#else
#define upgrademetric_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define upgrademetric_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Calculates H_euclidean. Needs camera matrix and plane at infinity of affinity.
template <typename T>
class upgrademetric_DECLDIR UpgradeMetric :
		public TemplatedParameteredObject<T> {
public:
	


	/// zero skew
	Parameter < bool > zeroSkew; 	
	/// principal point at origin
	Parameter < bool > principalPoint; 	
	/// principal point at origin
	Parameter < double > constraintWeight; 	
	/// aspect ratio constraint
	Parameter < bool > ratioConstraint; 	
	/// aspect ratio
	Parameter < double > ratio; 	

	/// camera matrix 1 
	InputSlot < cimg_library::CImgList<T> > cameraMatrix1;
	/// plane at infinity 
	InputSlot < std::vector<T> > planeAtInfinity;
	/// matrix that upgrades 3D points to metric scale 
	OutputSlot < cimg_library::CImgList<T> > transformationMatrix;

	/// The input 3D coordinates accurate up to a projective transformation 
	InputSlot < std::vector<std::vector<T> > > input_3d_point_cloud;
	/// The output of the affine 3d point coordinates 
	OutputSlot < std::vector<std::vector<T> > > output_3d_point_cloud;

	/// create a new UpgradeMetric object
	/// \param name          Instance name
	UpgradeMetric(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _UPGRADEMETRIC_H_



