/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file UpgradeMetric.h
 *  Declaration of the parameter class UpgradeMetric.
 *  \author Moritz Becker
 *  \date 05.08.2010
 */

#ifndef _UPGRADEMETRICADQ_H_
#define _UPGRADEMETRICADQ_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef upgrademetricadq_EXPORTS
/// Visual C++ specific code
#define upgrademetricadq_DECLDIR __declspec(dllexport)
#else
#define upgrademetricadq_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define upgrademetricadq_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include <hekate/IndexedMatrix.h>
#include <hekate/Correspondence.hxx>
#include <vigra/windows.h>

using namespace hekate;
using namespace cimg_library;

/// Calculates a metric transformation based on projective camera matrices.
/** This plugin is based on Thormhlen et al., Robust Linear Auto-Calibration of a
	Moving Camera from Image Sequences.*/
template <typename T>
class upgrademetricadq_DECLDIR UpgradeMetricADQ :
		public TemplatedParameteredObject<T> {
public:
	
	
	/// Aspect ratio of image sensor.
	Parameter < double > ratio; 	
	/// Image width
	Parameter < float > imageWidth; 	
	/// Image height
	Parameter < float > imageHeight; 	
	
	/// Camera matrices
	InputSlot < IndexedMatrixList > cameraMatrices;
	/// Calibratrion matrix 
	InputSlot < cimg_library::CImgList<T> > calibrationMatrix;
	/// Matrix that upgrades 3D points to metric scale.
	OutputSlot < CImgList<T> > transformationMatrix;

	/// The projective input 3D points.
	InputSlot < typename CorrespondenceList<T> > input3dPointCloud;
	/// Transformed 3D points.
	OutputSlot < typename CorrespondenceList<T> > output3dPointCloud;
	/// Camera matrices
	OutputSlot < IndexedMatrixList > cameraMatricesOut;

	/// create a new UpgradeMetric object
	/// \param name          Instance name
	UpgradeMetricADQ(const std::string& name="");

	/// Creates linear system based on Thormhlen et al. equation (14)-(19) to calculate the ADQ.
	/** For every camera matrix, additional six rows are added to the matrix A.
		At least three camera matrices are needed to overdetermine the linear system.
	*/
	CImg<double> createLinearSystem(IndexedMatrixList &cameras,
				double c1, double c2, double c3, double c4, double c5, double c6);

	/// Calculates the RQ transformation for matrix A = R*Q.
	void getRQ(CImg<double> A, CImg<double> &R_out, CImg<double> &Q_out);

	/// Update object.
	virtual void execute();
};

#endif // _UPGRADEMETRICADQ_H_



