/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file upgradeMetricTrackingMatrix.h
 *  Declaration of the parameter class upgradeMetricTrackingMatrix.
 *  \author Moritz Becker
 *  \date 19.08.2011
 */

#ifndef _UPGRADEMETRICTRACKINGMATRIX_H_
#define _UPGRADEMETRICTRACKINGMATRIX_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef upgrademetrictrackingmatrix_EXPORTS
/// Visual C++ specific code
#define upgrademetrictrackingmatrix_DECLDIR __declspec(dllexport)
#else
#define upgrademetrictrackingmatrix_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define upgrademetrictrackingmatrix_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// This plugin upgrades a tracking matrix with a given calibration.
/// Then the metric points in space and projective cameras can be retrieved.
template <typename T>
class upgrademetrictrackingmatrix_DECLDIR upgradeMetricTrackingMatrix :
		public TemplatedParameteredObject<T> {
public:
	

	

	/// trackingMatrix 
	InputSlot < cimg_library::CImgList<T> > trackingMatrix;
	/// calibrationMatrix 
	InputSlot < cimg_library::CImgList<T> > calibrationMatrix;
	/// metricTrackingMatrix 
	OutputSlot < cimg_library::CImg<T> > metricTrackingMatrix;
	/// cameraMatrices 
	OutputSlot < cimg_library::CImgList<T> > cameraMatrices;
	/// points3D 
	OutputSlot < std::vector<std::vector<T> > > points3D;
	

	/// create a new upgradeMetricTrackingMatrix object
	/// \param name          Instance name
	upgradeMetricTrackingMatrix(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _UPGRADEMETRICTRACKINGMATRIX_H_



