/*  Copyright (C) 2010 Julian Coordts

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file WarpBB.h
 *  Declaration of the parameter class WarpBB.
 *  \author Julian Coordts
 *  \date 08.03.2010
 */

#ifndef _WarpBB_H_
#define _WarpBB_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef warpbb_EXPORTS
/// Visual C++ specific code
#define WarpBB_DECLDIR __declspec(dllexport)
#else
#define WarpBB_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define WarpBB_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include <charon-utils/Interpolator.h>

/// Warps an image
template <typename T>
class WarpBB_DECLDIR WarpBB :
		public TemplatedParameteredObject<T> {
public:
	/// Input image sequence (first left - second right)
	InputSlot < cimg_library::CImgList<T> > input_images;
	/// Interpolator 
	InputSlot < Interpolator<T> * > interpolator;
	/// Warped image sequence (first left - second right)
	OutputSlot < cimg_library::CImgList<T> > warped_images;
	/// Warp matrices (first left - second right)
	InputSlot <cimg_library::CImgList<T> > warp_matrices;
	///Input Feature Correspondences
	InputSlot < std::vector<std::vector<std::pair<T, T> > > > input_feature_corres;
	///Warped Feature Correspondences
	OutputSlot < std::vector<std::vector<std::pair<T, T> > > > warped_feature_corres;

	///Set for fast nearest neighbor interpolation
	Parameter<bool> quick_warp;


	/// create a new WarpBB object
	/// \param name          Instance name
	WarpBB(const std::string& name);

	///Output a cImg matrix to sout
	void soutCImgMatrix(cimg_library::CImg<T> &m,const char* text);
	///calculates the bounding box of the image after transformation H
	cimg_library::CImg<T> calcBB(cimg_library::CImg<T>&H,
								 cimg_library::CImg<T>&Image);
	///main warp procedure for one Image
	void warp(cimg_library::CImg<T>& inputImage,
						 cimg_library::CImg<T>& outputImage,
						 cimg_library::CImg<T>& warpMatrix,
						 cimg_library::CImg<T>& boundingBox);
	///warps the input correspondences using the warp_matrices
	void warpCorrespondences(cimg_library::CImg<T>& boundingBox);
	/// Update object.
	virtual void execute();

	///boolean to avoid multiple calls
	bool called;
};

#endif // _WarpBB_H_



