/*  Copyright (C) 2010 Mirko Schmidt

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.

	This File is based on WarpBB.h by Julian Coordts.

*/
/** \file WarpBBinv.h
 *  Declaration of the parameter class WarpBB.
 *  \author Mirko Schmidt
 *  \date 08.03.2010
 */

#ifndef _WarpBBinv_H_
#define _WarpBBinv_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef warpbbinv_EXPORTS
/// Visual C++ specific code
#define WarpBBinv_DECLDIR __declspec(dllexport)
#else
#define WarpBBinv_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define WarpBBinv_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include <charon-utils/Interpolator.h>

/// Warps an image
template <typename T>
class WarpBBinv_DECLDIR WarpBBinv :
		public TemplatedParameteredObject<T> {
public:
	/// Input image sequence (first left - second right) (warped images)
	InputSlot < cimg_library::CImgList<T> > input_images;
	/// Interpolator 
	InputSlot < Interpolator<T> * > interpolator;
	/// Inversely warped image sequence (first left - second right) 
	OutputSlot < cimg_library::CImgList<T> > warpedinv_images;
	/// Warp matrices (first left - second right)
	InputSlot <cimg_library::CImgList<T> > warp_matrices;

	
	Parameter<bool> quick_warp;


	/// create a new WarpBBinv object
	/// \param name          Instance name
	WarpBBinv(const std::string& name);
	void soutCImgMatrix(cimg_library::CImg<T> &m,const char* text);
	cimg_library::CImg<T> calcBBinv(cimg_library::CImg<T>&H,
								 cimg_library::CImg<T>&Image);
	void warp(cimg_library::CImg<T>& inputImage,
						 cimg_library::CImg<T>& outputImage,
						 cimg_library::CImg<T>& warpMatrix,
						 cimg_library::CImg<T>& boundingBox);
	//void warpCorrespondences(cimg_library::CImg<T>& boundingBox);
	/// Update object.
	virtual void execute();


	bool called;
};

#endif // _WarpBBinv_H_



