/*  Copyright (C) 2009 Julian Coordts

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file correspondencepainter.h
 *  Declaration of the parameter class correspondencepainter.
 *  \author Julian Coordts
 *  \date 30.03.2010
 */

#ifndef _CORRESPONDENCEPAINTER_H_
#define _CORRESPONDENCEPAINTER_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef correspondencepainter_EXPORTS
/// Visual C++ specific code
#define correspondencepainter_DECLDIR __declspec(dllexport)
#else
#define correspondencepainter_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define correspondencepainter_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>

using namespace hekate;

/// Combines two images into one and draws lines between their correspondence points
template <typename T>
class correspondencepainter_DECLDIR correspondencepainter :
		public TemplatedParameteredObject<T> {
public:
	
	/// Draw index to every feature correspondence
	Parameter < bool > withIndices;
	
	/// Also draw correspondences known as outliers
	Parameter < bool > withOutliers;
	
	/// Draw all correspondences into one image
	Parameter < bool > drawAll;
	
	/// Number of images to draw in
	Parameter < int > numberOfImages;
	
	/// step size
	Parameter < int > stepSize;
	
	/// image width
	Parameter < int > width;
	
	/// image height
	Parameter < int > height;
	
	/// Draw colors of correspondences into one image
	Parameter < bool > drawPointColors;

	/// the input images (first left, second right)
	InputSlot < std::vector<std::string> > imagePath;

	/// imageSequence
	InputSlot < cimg_library::CImgList<T> > imageSequence;

	/// correspondences 
	InputSlot < typename CorrespondenceList<T> > correspondences;
	/// the drawn image 
	OutputSlot < cimg_library::CImgList<T> > image_result;

	/// create a new correspondencepainter object
	/// \param name          Instance name
	correspondencepainter(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _CORRESPONDENCEPAINTER_H_



