/*  Copyright (C) 2010 Moritz Becker

    This source code is distributed under the GNU GPL.
	
	See <http://www.gnu.org/licenses/>.
*/
/** \file FeatureDetectionSURF.h
 *  Declaration of the parameter class FeatureDetectionSURF.
 *  \author Moritz Becker
 *  \date 01.04.2010
 */

#ifndef _FEATUREDETECTIONSURF_H_
#define _FEATUREDETECTIONSURF_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef featuredetectionsurf_EXPORTS
/// Visual C++ specific code
#define featuredetectionsurf_DECLDIR __declspec(dllexport)
#else
#define featuredetectionsurf_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define featuredetectionsurf_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Feature detection and matching based on openSURF
template <typename T>
class featuredetectionsurf_DECLDIR FeatureDetectionSURF :
		public TemplatedParameteredObject<T> {
public:
	/// blob response threshold
	Parameter < float > thres; 
	/// number of intervals per octave
	Parameter < int > interval;
	/// number of octaves to calculate
	Parameter < int > octaves;
	/// initial sampling step
	Parameter < int > init_sample;

	/// image sequence 
	InputSlot < cimg_library::CImgList<T> > image_sequence;
	/// correspondece points 

	/// matches between second image pair 
	OutputSlot < std::vector<std::vector<std::vector<T> > > > correspondence;	
	/// features1
	OutputSlot<std::vector<std::vector<T> > > features1;
	/// features2
	OutputSlot<std::vector<std::vector<T> > > features2;

	/// create a new FeatureDetectionSURF object
	/// \param name          Instance name
	FeatureDetectionSURF(const std::string& name);

	/// Update object.
	virtual void execute();

};

#endif // _FEATUREDETECTIONSURF_H_



