/*  Copyright (C) 2011 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file FeatureKLTSequence.h
 *  Declaration of the parameter class FeatureKLTSequence.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 05.09.2011
 */

#ifndef _FEATUREKLTSEQUENCE_H_
#define _FEATUREKLTSEQUENCE_H_

#ifdef _MSC_VER
#ifdef featurekltsequence_EXPORTS
/// Visual C++ specific code
#define featurekltsequence_DECLDIR __declspec(dllexport)
#else
#define featurekltsequence_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define featurekltsequence_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Calculates KLT features and correspondences from an image sequence
/** Calculates KLT features and correspondences from an image sequence. The
 *  Sequence is given by the path of the first image file.
 *  The correspondences are outputed as a list of tracking matrices.
 */
template <typename T>
class featurekltsequence_DECLDIR FeatureKLTSequence :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	FeatureKLTSequence(const std::string& name = "");

	/// trackingMatrices
	OutputSlot< cimg_library::CImgList<T> > trackingMatrices;
	/// Correspondences
	OutputSlot< std::vector<std::vector<std::vector<T> > > > correspondences;
	/// Images
	OutputSlot< cimg_library::CImgList<T> > images;

	/// Path to the first image file of the sequence.
	Parameter< std::string > imageSequence;

	/// Maximal number of images per sequence.
	Parameter < int > maxImages;

	/// Path to the output tracking matrices.
	Parameter< std::string > trackingMatricesFile;
	
	/// Takes the first image in folder as base image which will be taken 
	/// as one of two images for correspondence estimation
	Parameter <bool> useBaseImage ;
	/// Maximal numbers of features that should be found in every image.
	Parameter <int > _minNumberOfCorrs ;
	Parameter <int > _maxFeatures ;
	Parameter <double> _featureQualityLevel ;
	/// Minimal distance between features.
	Parameter <double> _featureMinDistance ;
	Parameter <int> _featureBlockSize ;
	Parameter <bool> _useHarrisDetector ;
	Parameter <double> _harrisFreeParameter ;
	Parameter <bool> _useSubPixRefine ;
	
	Parameter <int> _lkWindowSize ;
	Parameter <int> _lkLevels ;

	/// Update object.
	virtual void execute();
};

#endif // _FEATUREKLTSEQUENCE_H_
