/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file featureMatcher3dSURF.h
 *  Declaration of the parameter class featureMatcher3dSURF.
 *  \author Moritz Becker
 *  \date 12.07.2010
 */

#ifndef _FEATUREMATCHER3DSURF_H_
#define _FEATUREMATCHER3DSURF_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef featurematcher3dsurf_EXPORTS
/// Visual C++ specific code
#define featurematcher3dsurf_DECLDIR __declspec(dllexport)
#else
#define featurematcher3dsurf_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define featurematcher3dsurf_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Matches features between two image pairs and between the images of each pair. Output order is matched.
template <typename T>
class featurematcher3dsurf_DECLDIR featureMatcher3dSURF :
		public TemplatedParameteredObject<T> {
public:
	
	/// blob response threshold
	Parameter < float > thres; 
	/// number of intervals per octave
	Parameter < int > interval;
	/// number of octaves to calculate
	Parameter < int > octaves;
	/// initial sampling step
	Parameter < int > init_sample;	
	
	/// First image pair 
	InputSlot < cimg_library::CImgList<T> > imageSequence1;
	/// Second image pair 
	InputSlot < cimg_library::CImgList<T> > imageSequence2;
	/// Matches between first image pair
	OutputSlot < std::vector<std::vector<std::vector<T> > > > imageCorrespondence1;
	/// Matches between second image pair
	OutputSlot < std::vector<std::vector<std::vector<T> > > > imageCorrespondence2;	
	/// Features 1
	OutputSlot < std::vector<std::vector<T> > > imageFeatures11;
	/// Features 2
	OutputSlot < std::vector<std::vector<T> > > imageFeatures12;

	/// create a new featureMatcher3dSURF object
	/// \param name          Instance name
	featureMatcher3dSURF(const std::string& name);

	/// Update object.
	virtual void execute();

private:

	void getFeatureCorrespondences();

};

#endif // _FEATUREMATCHER3DSURF_H_



