/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file featurepainter.h
 *  Declaration of the parameter class featurepainter.
 *  \author Moritz Becker
 *  \date 08.04.2010
 */

#ifndef _FEATUREPAINTER_H_
#define _FEATUREPAINTER_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef featurepainter_EXPORTS
/// Visual C++ specific code
#define featurepainter_DECLDIR __declspec(dllexport)
#else
#define featurepainter_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define featurepainter_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Draws feature points in source images
template <typename T>
class featurepainter_DECLDIR featurepainter :
		public TemplatedParameteredObject<T> {
public:

	/// Images
	InputSlot < cimg_library::CImgList<T> > image_sequence;
	/// Correspondences
	InputSlot <std::vector<std::vector<std::vector<T> > > > correspondence_vector;
	/// Output image list with painted features on it
	OutputSlot< cimg_library::CImgList<T> > output_sequence;

	/// create a new featurepainter object
	/// \param name          Instance name
	featurepainter(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _FEATUREPAINTER_H_



