/*  Copyright (C) 2009 Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file featurepainter.hxx
 *  Implementation of the parameter class featurepainter.
 *  \author Moritz Becker
 *  \date 08.04.2010
 */

#ifndef _FEATUREPAINTER_HXX_
#define _FEATUREPAINTER_HXX_

#include <hekate/features/FeaturePainter.h>

template <typename T>
featurepainter<T>::featurepainter(const std::string& name) :
		TemplatedParameteredObject<T>("testFeaturePainter", name,
			"Draws detected feature points in source images.")
			,image_sequence(true, false) 
			,correspondence_vector(false, false) 

{

	ParameteredObject::_addInputSlot(image_sequence, "image_sequence", "Image sequence", "CImgList<T>");
	ParameteredObject::_addInputSlot(correspondence_vector, "correspondence_vector", "Correspondences", "vector<vector<vector<T> > >");
	
	ParameteredObject::_addOutputSlot(output_sequence, "output_image1", "Output image list with painted features on it.", "CImgList<T>");
	
	ParameteredObject::_setTags("Hekate") ;

}

template <typename T>
void featurepainter<T>::execute() {
	PARAMETEREDOBJECT_AVOID_REEXECUTION;
	ParameteredObject::execute();

	const unsigned char red[] = { 255,0,0 } ;

	// check whether image sequence is connected
	if(this->image_sequence.connected()){
		sout << "FeaturePainter drawing image...." << std::endl ;
		cimg_library::CImg<T> res1;
		res1.assign(this->image_sequence()[0]);	
		if(res1.max() > 255.) res1 = res1 >> 4;
		
		// draw points to image

		typename std::vector<std::vector<T> >::const_iterator pointIt =
			correspondence_vector()[0].begin();
		const typename std::vector<std::vector<T> >::const_iterator pointEnd =
			correspondence_vector()[0].end();
			
		for(; pointIt != pointEnd; pointIt++){
			double x = (*pointIt)[0];
			double y = (*pointIt)[1];
			if(x>1 && x<res1.width() && y>3 && y<res1.height()){
				res1.draw_triangle(
					int(x),int(y),int(x+1.),
					int(y+3.),int(x-1.),int(y+3.),red);
			}
		}

		output_sequence().push_back(res1) ;
	}
	// otherwise create image with dimensions max(x), max(y) of feature values
	else{
		sout << "No Image connected, creating empty image...." << std::endl ;
		sout << "FeaturePainter drawing image...." << std::endl ;
		double maxX=0., maxY=0., minX=0, minY=0;

		typename std::vector<std::vector<T> >::const_iterator pointIt =
			correspondence_vector()[0].begin();
		const typename std::vector<std::vector<T> >::const_iterator pointEnd =
			correspondence_vector()[0].end();

		for(; pointIt != pointEnd; pointIt++){
			if((*pointIt)[0] > maxX) maxX = (*pointIt)[0];
			if((*pointIt)[1] > maxY) maxY = (*pointIt)[1];
			if((*pointIt)[0] < minX) minX = (*pointIt)[0];
			if((*pointIt)[1] < minY) minY = (*pointIt)[1];
		}
		cimg_library::CImg<T> res1((int)(maxX-minX)+3,(int)(maxY-minY)+3);
		
		sout << "Maximal X: " << maxX << std::endl;
		sout << "Maximal Y: " << maxY << std::endl;
		sout << "Minimal X: " << minX << std::endl;
		sout << "Minimal Y: " << minY << std::endl;

		// draw points to image
		pointIt = correspondence_vector()[0].begin();
		for(; pointIt != pointEnd; pointIt++){
			double x = (*pointIt)[0]-minX;
			double y = (*pointIt)[1]-minY;
			if(x>1 && x<res1.width() && y>3 && y<res1.height()){
				res1.draw_triangle(int(x),int(y),int(x+1.),int(y+3.),int(x-1.),int(y+3.),red) ;
			}
		}

		output_sequence().push_back(res1) ;
	}




	sout << std::endl ;



}

#endif /* _FEATUREPAINTER_HXX_ */



