/*  Copyright (C) 2011 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file FeatureSequence.h
 *  Declaration of the parameter class FeatureSequence.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 20.10.2011
 */

#ifndef _FEATURESEQUENCE_H_
#define _FEATURESEQUENCE_H_

#ifdef _MSC_VER
#ifdef featuresequence_EXPORTS
/// Visual C++ specific code
#define featuresequence_DECLDIR __declspec(dllexport)
#else
#define featuresequence_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define featuresequence_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>

using namespace hekate;

/// Calculates correspondences between images of an image sequence
/** An image sequence is needed as input. All images of a given folder are
 *  read in.
 *  One can choose between several feature descriptors from OpenCV.
 */
template <typename T>
class featuresequence_DECLDIR FeatureSequence :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	FeatureSequence(const std::string & pluginName, const std::string& name,
			const std::string & doc);
	
	/// Input path
	InputSlot< std::vector<std::string> > imgPath;
	/// Estimated correspondences
	OutputSlot< typename hekate::CorrespondenceList<T> > correspondences;
	/// imgMatches
	OutputSlot< std::vector<std::string> > imgPaths;
	
	/// Directory of image sequence
	Parameter< std::string > imageDir;
	/// Maximal number of images from which correspondences are calculated
	Parameter< size_t > maximalFrames;
	/// Minimal number of correspondences between images. If the number of
	/// correspondences between two images is less than this threshold, the
	/// second image won't be applied to the image/correspondence list
	Parameter< size_t > minimalNumberOfCorrs;
	/// Maximal number of subsequent frames, which do not exhibit
	/// the minimal number of correspondences (minimalNumberOfCorrs)
	///	between those frames
	Parameter< size_t > maxFWC;
	/// How many sub sequent frames before and after, a single frame gets matched with
	Parameter< size_t > matchRange;

	/// Update object.
	virtual void execute();

	int calcMax, calcCurrent;

	int _numberOfCorrespondences;

protected:
	/// Performs the calculation.
	virtual void _calculate() = 0;
	std::vector<std::string> filenames;
	std::string filepath;
	size_t numberOfFiles;
	void removeOutliers(std::vector<std::vector<std::vector<T> > > & corr);

};

#endif // _FEATURESEQUENCE_H_
