/*  Copyright (C) 2012 Moritz Emanuel Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file FeatureSequenceASIFT.h
 *  Declaration of the parameter class FeatureSequenceASIFT.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Emanuel Becker</a>
 *  \date 24.01.2012
 */

#ifndef _FEATURESEQUENCEASIFT_H_
#define _FEATURESEQUENCEASIFT_H_

#ifdef _MSC_VER
#ifdef featuresequenceasift_EXPORTS
/// Visual C++ specific code
#define featuresequenceasift_DECLDIR __declspec(dllexport)
#else
#define featuresequenceasift_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define featuresequenceasift_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/features/FeatureSequence.h>
#include <hekate/Correspondence.hxx>

#include "demo_lib_sift.h"
//#include "io_png/io_png.h"
//#include "io_png/io_png.c"
#include "library.h"
#include "frot.h"
#include "fproj.h"
#include "compute_asift_keypoints.h"
#include "compute_asift_matches.h"

/// Affine SIFT version of the sequence feature extractor
/** Calculates feature correspondences of an image sequence. For this, the
 *  affine SIFT version is used.
 */
template <typename T>
class featuresequenceasift_DECLDIR FeatureSequenceASIFT :
		public FeatureSequence<T> {
public:
	/// default constructor
	/// \param name          Instance name
	FeatureSequenceASIFT(const std::string& name = "");
	
	/// Scale frames
	Parameter < bool > scaleFrames;
	/// Width of scaled image
	Parameter < int > scaleX;
	/// Height of scaled image
	Parameter < int > scaleY;
	/// Number of tilts
	Parameter < int > nTilts;
	/// number of decimal places which have to be equal between
	/// correspondences through the time
	Parameter < int > nDecPlaces;
	/// nMinCorr", "minimal number of correspondences which are 
	/// used to define a match between two frames
	Parameter < int > nMinCorr;
	
protected:
	virtual void _calculate();
	void zoomImage(std::vector<float> &, std::vector<float> &, int, int, vector<int>&, vector<int>&, vector<float>&);
	matchingslist matchFrames(vector< vector< vector< keypointslist > > > &keypoints, int frame1, int frame2);
	cimg_library::CImg<float> getImg(size_t i);

private:
	vector<int> widthList, heightList;
	siftPar siftparameters;	
	int num_of_tilts;
	int verb;	

};

#endif // _FEATURESEQUENCEASIFT_H_
