/*  Copyright (C) 2012 Martin Treiber

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file FeatureSequenceGPUSift.h
 *  Declaration of the parameter class FeatureSequenceGPUSift.
 *  \author <a href="mailto:martin.treiber.hd@gmx.de">
 *      Martin Treiber</a>
 *  \date 08.03.2012
 */

#ifndef _FEATURESEQUENCEGPUSIFT_H_
#define _FEATURESEQUENCEGPUSIFT_H_

#ifdef _MSC_VER
#ifdef featuresequencegpusift_EXPORTS
/// Visual C++ specific code
#define featuresequencegpusift_DECLDIR __declspec(dllexport)
#else
#define featuresequencegpusift_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define featuresequencegpusift_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/features/FeatureSequence.h>
#include <hekate/Correspondence.hxx>
#include <SiftGPU.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <vector>
#include <time.h>

using namespace std;

/// GPUSIFT version of the sequence feature extractor
/** Calculates feature correspondences of an image sequence. For this, the
 *  SiftGPU version is used.
 */
template <typename T>
class __declspec(dllexport) FeatureSequenceGPUSift :
		public FeatureSequence<T> {
public:
	/// default constructor
	/// \param name          Instance name
	FeatureSequenceGPUSift(const std::string& name = "");
	
	/// Use CUDA (or GLSL)
	Parameter < bool > useCUDA;
	/// Use Sub-Pixel Localization
	Parameter < bool > useSubPixel;
	/// Use tight storage control (not recommended)
	Parameter < bool > setTight;
	///Use dynamic Array Indexing
	Parameter < bool > useDI;
	/// Octave to start with
	Parameter < int > startingOctave;
	/// DOG Levels in an Octave
	Parameter < int > DOGOctave;
	/// DOG Treshold
	Parameter < float > DOGTreshold;
	/// Edge Treshold
	Parameter < float > EdgeTreshold;
	///Maximal Nmber of Matches to search for
	Parameter < int > MaxMatches;


protected:
	virtual void _calculate();
	cimg_library::CImg<float> getImg(size_t i);

private:
	vector<int> widthList, heightList;
	int num_of_tilts;
	int verb;	

};

#endif // _FEATURESEQUENCEGPUSIFT_H_
