/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file FeatureSequenceGPUSiftFM.h
 *  Declaration of the parameter class FeatureSequenceGPUSiftFM.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 18.04.2013
 */

#ifndef _FEATURESEQUENCEGPUSIFTFM_H_
#define _FEATURESEQUENCEGPUSIFTFM_H_

#ifdef _MSC_VER
#ifdef featuresequencegpusiftfm_EXPORTS
/// Visual C++ specific code
#define featuresequencegpusiftfm_DECLDIR __declspec(dllexport)
#else
#define featuresequencegpusiftfm_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define featuresequencegpusiftfm_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <opencv/cv.h>
#include <hekate/openCvUtil.hxx>
#include <charon-utils/ImgTool.hxx>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>
#include <SiftGPU.h>

using namespace cimg_library;
using namespace hekate;
using namespace cv ;

/// Determines SIFT features and matches with epipolar constraint
/** Determines SIFT features using SIFTGPU
 *  (http://www.cs.unc.edu/~ccwu/siftgpu/). The matching process uses a
 *  calculated fundamental matrix (OpenCV) to prevent outliers.
 */
template <typename T>
class featuresequencegpusiftfm_DECLDIR FeatureSequenceGPUSiftFM :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	FeatureSequenceGPUSiftFM(const std::string& name = "");

	/// Path to images files
	InputSlot< std::vector< std::string > > imagePaths;

	/// Calculated correspondences
	OutputSlot< CorrespondenceList<T> > correspondences;
	/// Calculated fundamental matrices
	OutputSlot< IndexedMatrixList > fundamentalMatrices;

	/// Step width of chosen image sequence to process
	Parameter< int > stepWidth;
	/// Octave to start with
	Parameter< float > startingOctave;
	/// DOG Levels in an Octave
	Parameter< float > dogOctave;
	/// DOG Treshold
	Parameter< float > dogThreshold;
	/// Edge Treshold
	Parameter< float > edgeThreshold;
	/// Maximal Nmber of Matches to search for
	Parameter< int > maxMatches;
	/// Number of subsequent images to match
	Parameter< int > matchRange;


protected:
	/// Update object.
	virtual void execute();
};

#endif // _FEATURESEQUENCEGPUSIFTFM_H_
