/*  Copyright (C) 2011 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file FeatureSequenceSIFT.h
 *  Declaration of the parameter class FeatureSequenceSIFT.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 20.10.2011
 */

#ifndef _FEATURESEQUENCESIFT_H_
#define _FEATURESEQUENCESIFT_H_

#ifdef _MSC_VER
#ifdef featuresequencesift_EXPORTS
/// Visual C++ specific code
#define featuresequencesift_DECLDIR __declspec(dllexport)
#else
#define featuresequencesift_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define featuresequencesift_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/features/FeatureSequence.h>
#include <hekate/Correspondence.hxx>

/// Calculates correspondences between images of an image list by using the
/// OpenCV implementation of SIFT
/** Calculates correspondences between images of an image list by using the
 *  OpenCV implementation of SIFT
 */
template <typename T>
class featuresequencesift_DECLDIR FeatureSequenceSIFT :
		public FeatureSequence<T> {
public:
	/// default constructor
	/// \param name          Instance name
	FeatureSequenceSIFT(const std::string& name = "");
	
	/// Threshold for maximal distance between descriptors. (Matching parameter)
	Parameter< int > maxDistance;

	/// The peak threshold filters peaks of the DoG scale space that are
	///	too small (in absolute value)
	Parameter< double > peakThreshold;

	/// The edge threshold eliminates peaks of the DoG scale space whose
	///	curvature is too small (such peaks yield badly localized frames)
	Parameter< double > edgeThreshold;

	/// The number of a gaussian pyramid octaves that the detector uses.
	///	It is set to 4 by default; If you want to get very large features,
	///	use the larger value; If you want just small features, decrease it
	Parameter< int > octaves;

	/// The number of images within each octave of a gaussian pyramid, 
	/// It is set to 2 by default
	Parameter< int > octaveLayers;

	/// firstOctave
	Parameter< int > firstOctave;

	/// angleMode
	Parameter< int > angleMode;
	
protected:
	virtual void _calculate();

};

#endif // _FEATURESEQUENCESIFT_H_
