/*  Copyright (C) 2014 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file FeatureSequenceSURF.h
 *  Declaration of the parameter class FeatureSequenceSURF.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 16.01.2014
 */

#ifndef _FEATURESEQUENCESURF_H_
#define _FEATURESEQUENCESURF_H_

#ifdef _MSC_VER
#ifdef featuresequencesurf_EXPORTS
/// Visual C++ specific code
#define featuresequencesurf_DECLDIR __declspec(dllexport)
#else
#define featuresequencesurf_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define featuresequencesurf_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/IndexedMatrix.h>
#include <hekate/Correspondence.hxx>
#include <hekate/openCvUtil.hxx>

#include <opencv/cv.h>
#include <opencv/highgui.h>
#include <opencv/cxcore.h>
#include <opencv2/core/core.hpp>
#include <opencv2/features2d/features2d.hpp>
#include <opencv2/highgui/highgui.hpp>
#include "opencv2/gpu/gpu.hpp"
#include "opencv2/nonfree/nonfree.hpp"


#include <stdio.h>
#include <iostream>

using namespace hekate;
using namespace std;
using namespace cv ;
using namespace openCvUtil ;
using namespace cv::gpu;

/// Calculates SURF features and matches them
/** Calculates SURF features and matches them
 */
template <typename T>
class featuresequencesurf_DECLDIR FeatureSequenceSURF :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	FeatureSequenceSURF(const std::string& name = "");
	
	/// Path to images files
	InputSlot< std::vector< std::string > > imagePaths;

	/// Correspondence list
	OutputSlot< CorrespondenceList<T> > correspondence;
	/// Fundamental matrices
	OutputSlot< IndexedMatrixList > fundamentalmatrices;

	/// The time window for with a point is matched with subsequent
	/// correspondences
	Parameter< int > matchRange;

	/// Step width of chosen image sequence to process
	Parameter< int > stepWidth;

	/// How many points shall be compared to find the right correspondence
	Parameter< int > knn;

	/// Parameter 1 used for RANSAC
	/**
	It is the maximum distance from a point to an epipolar line in pixels, beyond which
	the point is considered an outlier and is not used for computing the final fundamental 
	matrix. It can be set to something like 1-3, depending on the accuracy of the point 
	localization, image resolution, and the image noise.
	*/
	Parameter< float > ransacDistance;
	
	/// It specifies a desirable level of confidence (probability) that the estimated matrix is correct.
	Parameter< float > ransacConfidence;

	/// Threshold for inliers (0-1]
	Parameter< float > threshold;

	/// Maximal allowed spacial distance between correspondences.
	/**
		The value should be near the maximal flow value of this sequence.
	*/
	Parameter< float > maxFlow;

	/// Additionally use equalized images. This will double the computation time.
	Parameter< bool > useEqualizedImg;


protected:
	/// Update object.
	virtual void execute();

	/// Extract good matches by comparing distances
	void getGoodMatches( std::vector< DMatch > &good_matches, 
		const std::vector< DMatch > &matches,
		const std::vector<KeyPoint> &keypoints1,
		const std::vector<KeyPoint> &keypoints2
		);
};

#endif // _FEATURESEQUENCESURF_H_
