/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file FeaturesSIFT.h
 *  Declaration of the parameter class FeaturesSIFT.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 23.01.2012
 */

#ifndef _FEATURESSIFT_H_
#define _FEATURESSIFT_H_

#ifdef _MSC_VER
#ifdef featuressift_EXPORTS
/// Visual C++ specific code
#define featuressift_DECLDIR __declspec(dllexport)
#else
#define featuressift_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define featuressift_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>

using namespace hekate;

/// Extracts SIFT features from an image or image sequence
/** Extracts the OpenCV version of SIFT features from an image or image
 *  sequence.
 */
template <typename T>
class featuressift_DECLDIR FeaturesSIFT :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	FeaturesSIFT(const std::string& name = "");


	/// extracted features put into an CorrespondenceList<T> struct
	OutputSlot< typename CorrespondenceList<T> > features;	
	
	/// imgMatches
	OutputSlot< std::vector<std::string> > imgPaths;

	/// path to image
	Parameter< std::string > imageDir;
	/// Choose all images of the selected type inside the according folder.
	Parameter< bool > useSequence;

	/// Threshold for maximal distance between descriptors. (Matching parameter)
	Parameter< int > maxDistance;

	/// The peak threshold filters peaks of the DoG scale space that are
	///	too small (in absolute value)
	Parameter< double > peakThreshold;

	/// The edge threshold eliminates peaks of the DoG scale space whose
	///	curvature is too small (such peaks yield badly localized frames)
	Parameter< double > edgeThreshold;

	/// The number of a gaussian pyramid octaves that the detector uses.
	///	It is set to 4 by default; If you want to get very large features,
	///	use the larger value; If you want just small features, decrease it
	Parameter< int > octaves;

	/// The number of images within each octave of a gaussian pyramid, 
	/// It is set to 2 by default
	Parameter< int > octaveLayers;

	/// firstOctave
	Parameter< int > firstOctave;

	/// angleMode
	Parameter< int > angleMode;

	/// Update object.
	virtual void execute();
};

#endif // _FEATURESSIFT_H_
