#ifndef _CONVTOPIXMAP_HXX_
#define _CONVTOPIXMAP_HXX_

#include <QPixmap>
#include <charon-utils/CImg.h>

template <typename T>
void convToPixmap(
		const cimg_library::CImg<T>& img, QPixmap& pix){
	const int imgWidth = img.width() ;
	const int imgHeight = img.height() ;
	const int imgChannels = img.spectrum() ;
	Q_ASSERT_X(img.depth() == 1, "convToPixmap", "2D images supported only");

	QImage tempImage(imgWidth, imgHeight, QImage::Format_RGB32);
	tempImage.fill(0xffffffff) ;
	const long pixCount = imgHeight * imgWidth ; //number of pixels
	const long pixCount2 = pixCount * 2 ;
	const T* cImgArray = img.data() ; //data buffer of the CImg object
	Q_ASSERT(cImgArray) ;
	if (imgChannels < 3) {
		// grayscale image
		for (int ii = 0 ; ii < imgHeight ; ii++) {
			// starting pixel offset of current scanline,
			ulong lineCount = ii * imgWidth ;
			for (int jj = 0 ; jj < imgWidth ; jj++) {
				Q_ASSERT(tempImage.scanLine(ii)) ;
				tempImage.scanLine(ii)[jj * 4] =
					tempImage.scanLine(ii)[jj * 4 + 1] =
						tempImage.scanLine(ii)[jj * 4 + 2] =
							cImgArray[lineCount + jj] ;
			}
		}
	}
	else {
		// color image
		for (int ii = 0 ; ii < imgHeight ; ii++) {
			//starting pixel offset of current scanline,
			ulong lineCount = ii * imgWidth ;
			for(int jj = 0 ; jj < imgWidth ; jj++) {
				// QImage pixel order is: RGBARGBARGBA...,
				// even if alpha channel is deactivated
				// CImg pixel order is: BBB...GGG...RRR...
				Q_ASSERT(tempImage.scanLine(ii)) ;

				// red-channel
				tempImage.scanLine(ii)[jj * 4] =
						cImgArray[pixCount2 + lineCount + jj] ;
				// green-channel
				tempImage.scanLine(ii)[jj * 4 + 1] =
						cImgArray[pixCount + lineCount + jj] ;
				// blue-channel
				tempImage.scanLine(ii)[jj * 4 + 2] =
						cImgArray[lineCount + jj] ;
				// alpha-channel
				//image->scanLine(ii)[jj * 4 + 3] =
				//		test_img->data()[3*dimx*dimy + ii * dimy + jj] ;
			}
		}
	}
	pix = QPixmap::fromImage(tempImage) ;
}

#endif
