/*  Copyright (C) 2009 Julian Coordts

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file densification.h
 *  Declaration of the parameter class densification.
 *  \author Julian Coordts
 *  \date 09.04.2010
 */

#ifndef _DENSIFICATION_H_
#define _DENSIFICATION_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef densification_EXPORTS
/// Visual C++ specific code
#define densification_DECLDIR __declspec(dllexport)
#else
#define densification_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define densification_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Fills black pixel in an image.
/// This plugin is necessary to get a dense disparity map.
template <typename T>
class densification_DECLDIR densification :
		public TemplatedParameteredObject<T> {
public:
	

	/// sparse image
	InputSlot < cimg_library::CImgList<T> > input_image;
	/// input confidence map
	InputSlot < cimg_library::CImgList<T> > confidence_map;
	/// dense image 
	OutputSlot < cimg_library::CImgList<T> > output_image;
	/// out confidence map
	OutputSlot < cimg_library::CImgList<T> > new_confidence_map;
	
	///set if interpolation is done by y or by x and y axis
	Parameter<bool> double_interpolation;

	/// create a new densification object
	/// \param name          Instance name
	densification(const std::string& name);

	/// Update object.
	virtual void execute();

};

#endif // _DENSIFICATION_H_



