/*  Copyright (C) 2009 Cornelius Ratsch

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Stereo.h
 *  Declaration of the parameter class Stereo.
 *  \author Cornelius Ratsch
 *  \date 02.03.2010
 */

#ifndef _STEREO_H_
#define _STEREO_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef stereo_EXPORTS
/// Visual C++ specific code
#define stereo_DECLDIR __declspec(dllexport)
#else
#define stereo_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define stereo_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Takes two stereo images and calculates the depth information
template <typename T>
class stereo_DECLDIR Stereo :
		public TemplatedParameteredObject<T> {
public:
	

	/// The left image 
	InputSlot < cimg_library::CImgList<T> > image_sequence;
	/// The depth information 
	OutputSlot < cimg_library::CImgList<T> > result;
	
	OutputSlot < cimg_library::CImgList<T> > disparity_map;
	
	OutputSlot < cimg_library::CImgList<T> > confidence_map;
	

	/// create a new Stereo object
	/// \param pluginName    Plugin name
	/// \param name          Instance name
	/// \param doc           Description
	Stereo(const std::string & pluginName, const std::string& name, 
		const std::string & doc);

	/// Update object.
	virtual void execute();
	
protected:
	/// Performs calculation.
	virtual void _calculate() = 0;
};

#endif // _STEREO_H_



