/*  Copyright (C) 2009 Cornelius Ratsch

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Stereo.hxx
 *  Implementation of the parameter class Stereo.
 *  \author Cornelius Ratsch
 *  \date 02.03.2010
 */

#ifndef _STEREO_HXX_
#define _STEREO_HXX_

#include <hekate/stereo/Stereo.h>

template <typename T>
Stereo<T>::Stereo(const std::string & pluginName, const std::string& name,
	const std::string & doc) :
		TemplatedParameteredObject<T>(pluginName, name, doc), image_sequence(false, true)
{
	

	ParameteredObject::_addInputSlot(image_sequence, "image_sequence", "Image sequence", "CImgList<T>");
	ParameteredObject::_addOutputSlot(result, "result", "The (raw) depth information", "CImgList<T>");
	ParameteredObject::_addOutputSlot(disparity_map, "disparity_map", "Disparity map (for left image, in pixels)", "CImgList<T>");
	ParameteredObject::_addOutputSlot(confidence_map, "confidence_map", "Confidence map (reliability in percent, 0 = no info, 100 = perfectly sure)", "CImgList<T>");
	
}

template <typename T>
void Stereo<T>::execute() {
	ParameteredObject::execute();

	_calculate();
}

#endif /* _STEREO_HXX_ */



