/*  Copyright (C) 2009 Cornelius Ratsch

 This file is part of Charon.

 Charon is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Charon is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Charon.  If not, see <http://www.gnu.org/licenses/>.
 */
/** \file StereoEarlyVision.h
 *  Declaration of the parameter class StereoEarlyVision.
 *  \author Cornelius Ratsch
 *  \date 02.03.2010
 */

#ifndef _STEREOEARLYVISION_H_
#define _STEREOEARLYVISION_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef stereoearlyvision_EXPORTS
/// Visual C++ specific code
#define stereoearlyvision_DECLDIR __declspec(dllexport)
#else
#define stereoearlyvision_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define stereoearlyvision_DECLDIR
#endif

#include <hekate/stereo/Stereo.h>
#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Takes two stereo images and calculates the depth information using the Early Vision method
template<typename T>
class stereoearlyvision_DECLDIR StereoEarlyVision: public Stereo<T> {
public:
	/// number of BP iterations at each scale
	Parameter<int> iterations;
	/// number of scales
	Parameter<int> scales;

	/// truncation of discontinuity cost
	Parameter<float> trunk_disc_cost;
	/// truncation of data cost
	Parameter<float> trunk_data_cost;
	/// weighting of data cost
	Parameter<float> weight_data_cost;

	/// large cost
	Parameter<float> large_cost;
	/// number of possible disparities
	Parameter<int> poss_disparities;
	/// scaling from disparity to graylevel in output
	Parameter<int> scaling;

	/// amount to smooth the input images
	Parameter<double> smoothing;

	/// create a new StereoEarlyVision object
	/// \param name          Instance name
	StereoEarlyVision(const std::string& name);

	/// Performs the calculation using the EarlyVision method.
	virtual void _calculate();
};

#endif // _STEREOEARLYVISION_H_

