/*  Copyright (C) 2010 Stephan Meister, template by Cornelius Ratsch

 This file is part of Charon.

 Charon is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Charon is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Charon.  If not, see <http://www.gnu.org/licenses/>.
 */
/** \file StereoOCV.h
 *  Declaration of the parameter class StereoOCV.
 *  \author Stephan Meister
 *  \date 11.03.2010
 */

#ifndef _STEREOOCV_H_
#define _STEREOOCV_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef stereoocv_EXPORTS
/// Visual C++ specific code
#define StereoOCV_DECLDIR __declspec(dllexport)
#else
#define StereoOCV_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define StereoOCV_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include <hekate/stereo/Stereo.h>

/// Takes two stereo images and calculates the depth information using the method of OpenCV 2.0
template<typename T>
class StereoOCV_DECLDIR StereoOCV: public Stereo<T> {
public:

	/// create a new StereoOCV object
	/// \param name          Instance name
	StereoOCV(const std::string& name);

	/// Performs the calculation using the EarlyVision method.
	virtual void _calculate();

public:

	/// optimal disparity range in pixel
	Parameter<int> _nDisparities;
	/// size of BlockMatching Windows in pixel
	Parameter<int> _bmWindowSize;

};

#endif // _stereoOCV_H_

