/*  Copyright (C) 2010 Stephan Meister, 
	based on work by Moritz Becker

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file TriangulationHartleySturm.h
 *  Declaration of the parameter class TriangulationHartleySturm.
 *  \author Stephan Meister
 *  \date 06.06.2010
 */

#ifndef _TRIANGULATIONFROMDISPARITY_H_
#define _TRIANGULATIONFROMDISPARITY_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef triangulationfromdisparity_EXPORTS
/// Visual C++ specific code
#define triangulationfromdisparity_DECLDIR __declspec(dllexport)
#else
#define triangulationfromdisparity_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define triangulationfromdisparity_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>

/// Triangulation of 3D points from disparity map and camera parameters
template <typename T>
class triangulationfromdisparity_DECLDIR TriangulationFromDisparity :
		public TemplatedParameteredObject<T> {
public:
	

	//Distance between the focal points of the stereo camera system [meter]
	Parameter<double> _stereoBaseline ;

	//Focal length of the lenses [meter]
	Parameter<double> _focalLength ;

	//Width of a single sensor pixel [meter]
	Parameter<double> _pixelWidth ;

	//Height of a single sensor pixel [meter]
	Parameter<double> _pixelHeight ;

	/// Camera matrix (currently not used) 
	InputSlot < cimg_library::CImgList<T> > _pmatrix;
	/// disparities 
	InputSlot < cimg_library::CImgList<T> > _disparities;
	/// correctedCorrespondences 
	//OutputSlot < cimg_library::CImgList<T> > triangulatedPoints;
	OutputSlot < std::vector<std::vector<T > > > triangulatedPoints;

	/// create a new TriangulationHartleySturm object
	/// \param name          Instance name
	TriangulationFromDisparity(const std::string& name);

	/// Update object.
	virtual void execute();
};

#endif // _TRIANGULATIONFROMDISPARITY_H_



