/*  Copyright (C) 2010 Moritz Becker

    This source code is distributed under the GNU GPL.
*/
/** \file TriangulationHartleySturm.h
 *  Declaration of the parameter class TriangulationHartleySturm.
 *  \author Moritz Becker
 *  \date 26.05.2010
 */

#ifndef _TRIANGULATIONHARTLEYSTURM_H_
#define _TRIANGULATIONHARTLEYSTURM_H_

#if defined(MSVC) && defined(HANDLE_DLL)
#ifdef triangulationhartleysturm_EXPORTS
/// Visual C++ specific code
#define triangulationhartleysturm_DECLDIR __declspec(dllexport)
#else
#define triangulationhartleysturm_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else /* No DLL handling or GCC */
/// Not needed with GCC
#define triangulationhartleysturm_DECLDIR
#endif

#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>

/// Triangulation based on Hartley and Sturm
/** This plugin triangulates two corresponding 2D points from two different cameras
	to receive the according 3D point. Therefore the camera matrices have to be known.
*/
template <typename T>
class triangulationhartleysturm_DECLDIR TriangulationHartleySturm :
		public TemplatedParameteredObject<T> {
public:
		
	/// If this flag is set, all reconstructed 3D points that ly behind the image planes are marked as outliers (0-vector).
	Parameter < bool > rejectNegativeDepths;
		
	/// Calculates all correspondences instead of only the first correspondence pair
	Parameter < bool > calculateAll;
		
	/// Triangulate between P_i and reference cam K(I|0)
	Parameter < bool > useReferenceCam;

	/// Camera matrices.
	InputSlot < cimg_library::CImgList<T> > pmatrix;
	/// 2D correspondences.
	InputSlot < typename hekate::CorrespondenceList<T> > correspondences;
	/// Reconstructed 3D points.
	OutputSlot < typename hekate::CorrespondenceList<T> > triangulatedPoints;
	

	/// create a new TriangulationHartleySturm object
	/// \param name          Instance name
	TriangulationHartleySturm(const std::string& name);

	/// Update object.
	virtual void execute();

};

#endif // _TRIANGULATIONHARTLEYSTURM_H_



