/*  Copyright (C) 2010 Julian Coordts

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "AdjustBaseDisplayPluginGui.h"
#include <stdio.h>
#include <iostream>
#include <string>
#include <charon-core/ParameteredObject.hxx>
#include <iomanip>
#include <QCheckBox>

#include "ui_AdjustBaseDisplay.h"

AdjustBaseDisplayPluginGui::AdjustBaseDisplayPluginGui(AdjustBaseDisplayInterface* parent) :
		_parent(parent) {

	_sliderFactor = 10. ;

	_window = new QWidget();
	_gui.setupUi(_window);
	_window->showMaximized() ;

	//set up scroll bar policy of rightview/lefview
	_gui.leftView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	_gui.leftView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	_gui.rightView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	_gui.rightView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	_gui.leftView->setDragMode(QGraphicsView::ScrollHandDrag) ;
	_gui.rightView->setDragMode(QGraphicsView::ScrollHandDrag) ;
	//set up scroll bar position
	_gui.leftView->setHorizontalScrollBar(_gui.rightView->horizontalScrollBar()) ;
	_gui.leftView->setVerticalScrollBar(_gui.rightView->verticalScrollBar()) ;
	_gui.displayLayout->addWidget(_gui.rightView->horizontalScrollBar(),2,0,1,2);
	_gui.displayLayout->addWidget(_gui.rightView->verticalScrollBar(),1,2);

	_gui.leftView->setScene(new QGraphicsScene(this));
	_gui.rightView->setScene(new QGraphicsScene(this));

	//set up slider and spinbox values
	_gui.leftSlider->setRange(-1*int(_sliderFactor), int(_sliderFactor)*this->_parent->getMaxAlphaLeft());
	_gui.rightSlider->setRange(-1*int(_sliderFactor), int(_sliderFactor)*this->_parent->getMaxAlphaRight());
	_gui.leftSpinBox->setRange(-1., this->_parent->getMaxAlphaLeft());
	_gui.rightSpinBox->setRange(-1., this->_parent->getMaxAlphaRight());

	connect(_gui.runButton, SIGNAL(clicked()),
			this,SLOT(runWorkflow()));

	//slider and spinbox functions
	connect(_gui.leftSlider, SIGNAL(valueChanged(int)), this, SLOT(setLeftValueDouble(int)) );
	connect(_gui.leftSpinBox, SIGNAL(valueChanged(double)), this, SLOT(setLeftValueInt(double)) );
	connect(_gui.rightSlider, SIGNAL(valueChanged(int)), this, SLOT(setRightValueDouble(int)) );
	connect(_gui.rightSpinBox, SIGNAL(valueChanged(double)), this, SLOT(setRightValueInt(double)) );

	//checkbox functions
	connect(_gui.checkBoxAutoExecute, SIGNAL(stateChanged(int)), this, SLOT(setAutoExecute(int)) );
	connect(_gui.checkBoxSwap, SIGNAL(stateChanged(int)), this, SLOT(runWorkflow()) );
	connect(_gui.checkBoxZoom, SIGNAL(stateChanged(int)), this, SLOT(runWorkflow()) );

	_window->show();
}

void AdjustBaseDisplayPluginGui::setAutoExecute(int state) {
	if(state){
		connect(_gui.leftSlider, SIGNAL(sliderReleased()), this, SLOT(runWorkflow()) );
		connect(_gui.rightSlider, SIGNAL(sliderReleased()), this, SLOT(runWorkflow()) );
		connect(_gui.leftSpinBox, SIGNAL(valueChanged(double)), this, SLOT(runWorkflow()) );
		connect(_gui.rightSpinBox, SIGNAL(valueChanged(double)), this, SLOT(runWorkflow()) );
	}
	else {
		disconnect(_gui.leftSlider, SIGNAL(sliderReleased()), this, SLOT(runWorkflow()) );
		disconnect(_gui.rightSlider, SIGNAL(sliderReleased()), this, SLOT(runWorkflow()) );
		disconnect(_gui.leftSpinBox, SIGNAL(valueChanged(double)), this, SLOT(runWorkflow()) );
		disconnect(_gui.rightSpinBox, SIGNAL(valueChanged(double)), this, SLOT(runWorkflow()) );
	}
}

void AdjustBaseDisplayPluginGui::runWorkflow() {
	this->_parent->processNew(this->_gui.leftSlider->value()/_sliderFactor,
							  this->_gui.rightSlider->value()/_sliderFactor);
}

void AdjustBaseDisplayPluginGui::setLeftValueDouble(int x) {
	this->_gui.leftSpinBox->setValue(x / _sliderFactor) ;
}
void AdjustBaseDisplayPluginGui::setRightValueDouble(int x) {
	this->_gui.rightSpinBox->setValue(x / _sliderFactor) ;
}
void AdjustBaseDisplayPluginGui::setLeftValueInt(double x) {
	this->_gui.leftSlider->setValue(int(x*_sliderFactor)) ;
}
void AdjustBaseDisplayPluginGui::setRightValueInt(double x) {
	this->_gui.rightSlider->setValue(int(x*_sliderFactor)) ;
}


void AdjustBaseDisplayPluginGui::drawPictures(QPixmap leftImg,
				 QPixmap rightImg)
{

	QList<QGraphicsItem*> items = _gui.leftView->scene()->items();
	for(int i=0; i < items.size(); i++) {
		_gui.leftView->scene()->removeItem(items[i]);
		delete items[i];
	}
	items = _gui.rightView->scene()->items();
	for(int i=0; i < items.size(); i++) {
		_gui.rightView->scene()->removeItem(items[i]);
		delete items[i];
	}

	if(!_gui.checkBoxSwap->isChecked()){
		_gui.labelLeftView->setText("Left image") ;
		_gui.labelRightView->setText("Right image") ;
	}
	else{
		_gui.labelLeftView->setText("Right image") ;
		_gui.labelRightView->setText("Left image") ;
	}

	// show new items
	QGraphicsItem* item;

	item = _gui.leftView->scene()->addPixmap(_gui.checkBoxSwap->isChecked() ? rightImg : leftImg);
	if(!_gui.checkBoxZoom->isChecked()) _gui.leftView->fitInView(item,Qt::KeepAspectRatio);
	else _gui.leftView->setTransform(QTransform());

	item = _gui.rightView->scene()->addPixmap(_gui.checkBoxSwap->isChecked() ? leftImg : rightImg);
	if(!_gui.checkBoxZoom->isChecked()) _gui.rightView->fitInView(item,Qt::KeepAspectRatio);
	else _gui.rightView->setTransform(QTransform());
}

AdjustBaseDisplayPluginGui::~AdjustBaseDisplayPluginGui() {
	_window->close();
	delete _window;
}

#include "AdjustBaseDisplayPluginGui.moc"
