/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file CreateId.cpp
 *  Implementation of parameter class CreateId.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 23.10.2013
 */

#include <charon-core/ParameteredObject.hxx>
#include <hekate/CreateId.h>

CreateId::CreateId(const std::string& name) :
		ParameteredObject(
			"CreateId", name,
			"<h2>Creates vector of ids</h2><br>Creates vector of ids"
		),
		id("0")
{

	ParameteredObject::_addOutputSlot(
		ids, "ids",
		"Ids",
		"vector<int>");

	ParameteredObject::_addParameter(
		id, "id",
		"Ids");

}

void CreateId::execute() {

	for(unsigned int i = 0; i < this->id().size(); i++){
		this->ids().push_back(this->id()[i]);
	}


}

// the following functions are needed
// for class CreateId to work as a charon plugin.
extern "C" createid_DECLDIR ParameteredObject*
		create(const std::string& name, ParameteredObject::template_type) {
	return new CreateId(name);
}

extern "C" createid_DECLDIR void destroy(ParameteredObject* b) {
	delete b;
}

/// Report build configuration to prevent linking of incompatibel runtime libs
extern "C" createid_DECLDIR ParameteredObject::build_type getBuildType() {
#ifdef _DEBUG
	return ParameteredObject::DEBUG_BUILD;
#else
	return ParameteredObject::RELEASE_BUILD;
#endif
}
