/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file DepthEdgesRFWriter.h
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 08.07.2013
 *  Declaraion of parameter class DepthEdgesRFWriter.
 */
#ifndef _DEPTHEDGESRFWRITER_H_
#define _DEPTHEDGESRFWRITER_H_

#ifdef _MSC_VER
#ifdef depthedgesrfwriter_EXPORTS
/// Visual C++ specific code
#define depthedgesrfwriter_DECLDIR __declspec(dllexport)
#else
#define depthedgesrfwriter_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define depthedgesrfwriter_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <vigra/multi_array.hxx>
#include <charon-utils/CImg.h>

/// Writes random forests learned by DepthEdgesLearn to a local file
/** Saves random forests and corresponding image ids to a seperate file
 */
class depthedgesrfwriter_DECLDIR DepthEdgesRFWriter : public ParameteredObject {
public:
	/// default constructor
	/// \param name             instance name
	DepthEdgesRFWriter(const std::string& name = "");

	/// Random forests
	InputSlot< std::vector< std::vector< RandomForest<int> > > > rf;
	/// Image ids for random forests
	InputSlot< std::vector<int> > ids;
	/// Coordinates for random forest patches (if there are more random
	/// forests per image)
	InputSlot< std::vector<std::vector<std::pair<int,int>>> > coordinates;

	/// Path to write files to.
	Parameter< std::string > filepath;


protected:
	/// Update object.
	virtual void execute();
};

#endif /* _DEPTHEDGESRFWRITER_H_ */
