/*  Copyright (C) 2010 Julian Coordts

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef PLUGINGUI_H_
#define PLUGINGUI_H_

#include <QtGui>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <string>
#include <stdio.h>
#include <charon-core/ParameteredObject.hxx>
#include <charon-utils/CImg.h>
#include "ui_FileSequenceReader.h"
#include "FileSequenceReaderInterface.h"

class FileSequenceReaderInterface;

namespace Ui {
	class FileSequenceReader;
}

class PluginGui : public QObject
{
	Q_OBJECT
private:
	///the main Window
	QWidget* _window;
	///the according qt UI
	Ui::FileSequenceReader _gui;

	/// pointer to parent FileSequenceReader
	/** May be used e.g. to reexecute the workflow by
	 *  calling the corresponding functions from the
	 *  FileSequenceReader interface.
	 */
	FileSequenceReaderInterface* _parent;

public:
	///Gui for selecting an image pair out and reexecuting the workflow
	PluginGui(FileSequenceReaderInterface* parent,
			  std::string rightPath,
			  std::string leftPath,
			  std::string rightName,
			  std::string leftName,
			  int numbersCount);
	///Destructor
	~PluginGui();

	///check if the names at the textfields are correct
	bool inputValid() const;
	///check if the files really exit
	bool valid(int i) const;
	///check if the current selected file really exit
	bool currentExists() const;
	///returns the name of the left file with index i
	QString leftName(int i) const;
	///returns the name of the right file with index i
	QString rightName(int i) const;
	///returns the name of the selected left file
	QString curLeftName() const;
	///returns the name of the selected right file
	QString curRightName() const;

public slots:
	///Slot sliderChanged
	void sliderChanged();
	///Slot loadPaths(loading the images in the path to the list of possible
	///images)
	void loadPaths(bool message=true);
	///Slot runWorkflow
	void runWorkflow();
};

#endif /* PLUGINGUI_H_ */
