/*  Copyright (C) 2011 Julian Wintermayr

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "drawingarea.h"


// Konstruktor
DrawingArea::DrawingArea(QWidget* parent) : QLabel(parent) {

    cursor = new QCursor(Qt::CrossCursor);
    setCursor(*cursor);

    tree = new QTreeWidget;
    tree->setColumnCount(1);
    tree->setHeaderLabel(tr("Start- and Endpoint (X/Y in pixel)"));

    treePoints = new QTreeWidget;
    treePoints->setColumnCount(1);
    treePoints->setHeaderLabel(tr("Start- and Endpoint (X/Y in pixel)"));

    treeParallelLines = new QTreeWidget;
    treeParallelLines->setColumnCount(1);
    treeParallelLines->setHeaderLabel(tr("Start- and Endpoint (X/Y in pixel)"));

    treeOrthographicLines = new QTreeWidget;
    treeOrthographicLines->setColumnCount(1);
    treeOrthographicLines->setHeaderLabel(tr("Start- and Endpoint (X/Y in pixel)"));


    // Rechtsklick Menü
    editLineGroupAct = new QAction(tr("Change line group"), this);
    connect(editLineGroupAct, SIGNAL(triggered()), this, SLOT(editLineGroup()));

    deleteLineAct = new QAction(tr("Delete"), this);
    connect(deleteLineAct, SIGNAL(triggered()), this, SLOT(deleteLine()));


    modified = false;
    drawing = false;
    lineSelected = false;
    editLine = false;
    pointsChanged = false;
    selectionMode = false;
    moveMode = false;
    startPointSet = false;
    endPointSet = false;
    thirdPointSet = false;
    hideCircles = false;
    drawSecondLine = false;
    thirdPointClicked = false;

    lock1 = false;
    lock2 = false;
    lock3 = false;

    lastLineGroup = 1;
    mode = 0;
    scaleFactor = 1.0;
    deletedListElement = 0;
}


void DrawingArea::createCursor(Qt::CursorShape shape) {

    delete cursor;

    cursor = new QCursor(shape);
    setCursor(*cursor);
}


//-------TODO--------
// Ausgewählter Listeneintrag muss IMMER mit ausgewähltem Vektor übereinstimmen
int DrawingArea::markLineFromList(QTreeWidgetItem* clickedListElement, QTreeWidgetItem* prevListElement) {

    if(prevListElement != deletedListElement) {

        if(lineSelected) {

            deleteLineMode = true;
            startPoint = selectedStartPoint * scaleFactor;
            lastPoint = selectedEndPoint * scaleFactor;

            penColor = Qt::green;
            drawLineTo(lastPoint);
            penColor = Qt::white;
            drawLineTo(lastPoint);

            if(mode == 3) {

                lastPoint = selectedThirdPoint * scaleFactor;
                penColor = Qt::green;
                drawLineTo(lastPoint);
                penColor = Qt::white;
                drawLineTo(lastPoint);
                lastPoint = selectedEndPoint * scaleFactor;
            }

            lineSelected = false;
            markListElement(false);
        }

        for(std::vector<drawnLines>::iterator itAllDrawnLines = allDrawnLinesInCurrentMode.begin();
            itAllDrawnLines != allDrawnLinesInCurrentMode.end(); ++itAllDrawnLines) {

              QString coordinatesStringQ;

              if(mode == 3) {

                std::stringstream coordinatesStream;
                coordinatesStream << itAllDrawnLines->start.x() << "/" << itAllDrawnLines->start.y()
                       << " --> " << itAllDrawnLines->end.x() << "/" << itAllDrawnLines->end.y()
                       << " --> " << itAllDrawnLines->third.x() << "/" << itAllDrawnLines->third.y();
                std::string coordinatesString = coordinatesStream.str();
                coordinatesStringQ = QString::fromStdString(coordinatesString);

              } else {

                std::stringstream coordinatesStream;
                coordinatesStream << itAllDrawnLines->start.x() << "/" << itAllDrawnLines->start.y()
                       << " --> " << itAllDrawnLines->end.x() << "/" << itAllDrawnLines->end.y();
                std::string coordinatesString = coordinatesStream.str();
                coordinatesStringQ = QString::fromStdString(coordinatesString);
              }

              if(coordinatesStringQ == clickedListElement->text(0)) {

                selectedVectorEntry = itAllDrawnLines;
                deleteLineMode = true;
                startPoint = itAllDrawnLines->start * scaleFactor;
                selectedStartPoint = itAllDrawnLines->start;
                lastPoint = itAllDrawnLines->end * scaleFactor;
                selectedEndPoint = itAllDrawnLines->end;
                penColor = Qt::white;
                drawLineTo(lastPoint);
                penColor = Qt::green;
                drawLineTo(lastPoint);

                if(mode == 3) {

                    lastPoint = itAllDrawnLines->third * scaleFactor;
                    selectedThirdPoint = itAllDrawnLines->third;
                    penColor = Qt::white;
                    drawLineTo(lastPoint);
                    penColor = Qt::green;
                    drawLineTo(lastPoint);
                    lastPoint = itAllDrawnLines->end * scaleFactor;
                }

                lineSelected = true;
                markListElement(true); // Könnte man noch ändern, indem man der Funktion das clickedListElement
                                       // übergibt. Dann müsste die Funktion nicht auch noch nach dem Eintrag suchen.

                // Damit die Funktion endet und nicht mehr weitersucht:
                return 0;
            }
        }

        // Wenn die Linie nicht gefunden wurde (weil es zB die Liniengruppennummmer war die angeklickt wurde),
        // wird der Eintrag wieder deselektiert

        clickedListElement->setSelected(false);
        return 0;
    }

    return 1;
}


void DrawingArea::addListElement(int group, QPointF start, QPointF end, QPointF third) {

    // Punkte und Liniengruppe in String und QString umwandeln

    std::stringstream groupStream;
    groupStream << group;
    std::string groupString = groupStream.str();
    QString groupStringQ = QString::fromStdString(groupString);

    QString coordinatesStringQ;

    if(mode == 3) {

        std::stringstream coordinatesStream;
        coordinatesStream << start.x() << "/" << start.y() << " --> " << end.x() << "/" << end.y()
                          << " --> " << third.x() << "/" << third.y();
        std::string coordinatesString = coordinatesStream.str();
        coordinatesStringQ = QString::fromStdString(coordinatesString);

    } else {

        std::stringstream coordinatesStream;
        coordinatesStream << start.x() << "/" << start.y() << " --> " << end.x() << "/" << end.y();
        std::string coordinatesString = coordinatesStream.str();
        coordinatesStringQ = QString::fromStdString(coordinatesString);
    }

    if(mode == 2) {

        QList<QTreeWidgetItem*> foundItem = tree->findItems(groupStringQ, Qt::MatchFixedString, 0);

        if(foundItem.size() == 1) {

            QTreeWidgetItem* treeItem = new QTreeWidgetItem(foundItem.at(0));
            treeItem->setText(0, coordinatesStringQ);
            foundItem.at(0)->setExpanded(true);

        } else {

            QTreeWidgetItem* treeItemGroup = new QTreeWidgetItem(tree);
            treeItemGroup->setText(0, groupStringQ);

            QTreeWidgetItem* treeItem = new QTreeWidgetItem(treeItemGroup);
            treeItem->setText(0, coordinatesStringQ);

            treeItemGroup->setExpanded(true);
        }

    } else {

        QTreeWidgetItem* treeItem = new QTreeWidgetItem(tree);
        treeItem->setText(0, coordinatesStringQ);
    }

    tree->sortItems(0, Qt::AscendingOrder);
}


void DrawingArea::deleteListElement() {

    QString coordinatesStringQ;

    if(mode == 3) {

       std::stringstream coordinatesStream;
       coordinatesStream << selectedVectorEntry->start.x() << "/" << selectedVectorEntry->start.y()
                         << " --> " << selectedVectorEntry->end.x() << "/" << selectedVectorEntry->end.y()
                         << " --> " << selectedVectorEntry->third.x() << "/" << selectedVectorEntry->third.y();
       std::string coordinatesString = coordinatesStream.str();
       coordinatesStringQ = QString::fromStdString(coordinatesString);

    } else {

        std::stringstream coordinatesStream;
        coordinatesStream << selectedVectorEntry->start.x() << "/" << selectedVectorEntry->start.y()
                          << " --> " << selectedVectorEntry->end.x() << "/" << selectedVectorEntry->end.y();
        std::string coordinatesString = coordinatesStream.str();
        coordinatesStringQ = QString::fromStdString(coordinatesString);
    }

    QList<QTreeWidgetItem*> foundItem = tree->findItems(coordinatesStringQ, Qt::MatchFixedString |
                                                        Qt::MatchRecursive | Qt::MatchWrap, 0);

    QTreeWidgetItem* parent = foundItem.at(0)->parent();
    deletedListElement = foundItem.at(0);
    delete foundItem.at(0);

    if(mode == 2 && parent->childCount() == 0) {

        deletedListElement = parent;
        delete parent;
    }

    tree->sortItems(0, Qt::AscendingOrder);

//    QList<QTreeWidgetItem*> selectedItem = tree->selectedItems();
//
//    for (int i = 0; i < selectedItem.size(); ++i) {
//
//        selectedItem.at(i)->setSelected(false);
//    }
}


void DrawingArea::markListElement(bool select) {

    QString coordinatesStringQ;

    if(mode == 3) {

        std::stringstream coordinatesStream;
        coordinatesStream << selectedVectorEntry->start.x() << "/" << selectedVectorEntry->start.y()
                          << " --> " << selectedVectorEntry->end.x() << "/" << selectedVectorEntry->end.y()
                          << " --> " << selectedVectorEntry->third.x() << "/" << selectedVectorEntry->third.y();
        std::string coordinatesString = coordinatesStream.str();
        coordinatesStringQ = QString::fromStdString(coordinatesString);

    } else {

        std::stringstream coordinatesStream;
        coordinatesStream << selectedVectorEntry->start.x() << "/" << selectedVectorEntry->start.y()
                          << " --> " << selectedVectorEntry->end.x() << "/" << selectedVectorEntry->end.y();
        std::string coordinatesString = coordinatesStream.str();
        coordinatesStringQ = QString::fromStdString(coordinatesString);
    }

    QList<QTreeWidgetItem*> foundItem = tree->findItems(coordinatesStringQ, Qt::MatchFixedString |
                                                        Qt::MatchRecursive | Qt::MatchWrap, 0);

    foundItem.at(0)->setSelected(select);
}


void DrawingArea::draw(std::vector<drawnLines> &drawVector, int drawMode) {

    if(drawMode == 1 && lineSelected) {

        startPoint = selectedStartPoint * scaleFactor;
        lastPoint = selectedEndPoint * scaleFactor;
        penColor = Qt::green;
        drawLineTo(lastPoint);
        lastPoint = selectedEndPoint * scaleFactor;
        penColor = Qt::white;
        drawLineTo(lastPoint);

        if(mode == 3) {

            lastPoint = selectedThirdPoint * scaleFactor;
            penColor = Qt::green;
            drawLineTo(lastPoint);
            lastPoint = selectedThirdPoint * scaleFactor;
            penColor = Qt::white;
            drawLineTo(lastPoint);
            lastPoint = selectedEndPoint * scaleFactor;
        }

        lineSelected = false;
    }

    penColor = Qt::white;

    for(std::vector<drawnLines>::iterator itDrawVector = drawVector.begin();
        itDrawVector != drawVector.end(); ++itDrawVector) {

            startPoint = itDrawVector->start * scaleFactor;
            lastPoint = itDrawVector->end * scaleFactor;
            drawLineTo(lastPoint);

            if(mode == 3) {

                lastPoint = itDrawVector->third * scaleFactor;
                drawLineTo(lastPoint);
                lastPoint = itDrawVector->end * scaleFactor;
            }
    }

    if(drawMode == 2 && lineSelected) {

        startPoint = selectedStartPoint * scaleFactor;
        lastPoint = selectedEndPoint * scaleFactor;
        penColor = Qt::white;
        drawLineTo(lastPoint);
        lastPoint = selectedEndPoint * scaleFactor;
        penColor = Qt::green;
        drawLineTo(lastPoint);

        if(mode == 3) {

            lastPoint = selectedThirdPoint * scaleFactor;
            penColor = Qt::white;
            drawLineTo(lastPoint);
            lastPoint = selectedThirdPoint * scaleFactor;
            penColor = Qt::green;
            drawLineTo(lastPoint);
            lastPoint = selectedEndPoint * scaleFactor;
        }
    }
}


void DrawingArea::backup(std::vector<drawnLines>& targetVector) {

    // Vektor vorher löschen, weil sonst die Einträge doppelt drin wären!

    targetVector.clear();

    for(std::vector<drawnLines>::iterator itAllDrawnLines = allDrawnLinesInCurrentMode.begin();
        itAllDrawnLines != allDrawnLinesInCurrentMode.end(); ++itAllDrawnLines) {

            startPoint = itAllDrawnLines->start;
            lastPoint = itAllDrawnLines->end;
            int lineGroup = itAllDrawnLines->group;

            if(mode == 3) {

                thirdPoint = itAllDrawnLines->third;
                targetVector.push_back(drawnLines(startPoint, lastPoint, lineGroup, thirdPoint));

            } else {

                targetVector.push_back(drawnLines(startPoint, lastPoint, lineGroup));
            }
    }
}


void DrawingArea::load(std::vector<drawnLines>& sourceVector) {

    for(std::vector<drawnLines>::iterator itSourceVector = sourceVector.begin();
        itSourceVector != sourceVector.end(); ++itSourceVector) {

            startPoint = itSourceVector->start;
            lastPoint = itSourceVector->end;
            int lineGroup = itSourceVector->group;

            if(mode == 3) {

                thirdPoint = itSourceVector->third;
                allDrawnLinesInCurrentMode.push_back(drawnLines(startPoint, lastPoint, lineGroup, thirdPoint));

            } else {

                allDrawnLinesInCurrentMode.push_back(drawnLines(startPoint, lastPoint, lineGroup));
            }
      }
}


void DrawingArea::points() {

    lock1 = true;

    if(lineSelected) {

        markListElement(false);
    }

    if(mode == 0) {

        mode = 1;
        tree = treePoints;

    } else if(mode == 2) {

        mode = 1;

        deleteLineMode = true;

        // Alle Linien löschen durch überzeichnen
        draw(allDrawnLinesInCurrentMode, 1);

        // Alle Linien sichern bzw. den vorhandenen erweitern
        backup(allDrawnParallelLines);

        // Vektor löschen
        allDrawnLinesInCurrentMode.clear();

        // Alle schon gezeichneten Punkte in den Vektor laden
        load(allDrawnPoints);

        // Alle schon gezeichneten Punkte wieder auf den Bildschirm zeichnen
        draw(allDrawnLinesInCurrentMode, 1);
        deleteLineMode = false;

        treeParallelLines = tree;
        tree = treePoints;

    } else if(mode == 3) {

        deleteLineMode = true;

        // Alle Linien löschen durch überzeichnen
        draw(allDrawnLinesInCurrentMode, 1);

        // Alle Linien sichern bzw. den vorhandenen erweitern
        backup(allDrawnOrthographicLines);

        // Vektor löschen
        allDrawnLinesInCurrentMode.clear();

        // Muss genau hier stehen
        mode = 1;

        // Alle schon gezeichneten Punkte in den Vektor laden
        load(allDrawnPoints);

        // Alle schon gezeichneten Punkte wieder auf den Bildschirm zeichnen
        draw(allDrawnLinesInCurrentMode, 1);
        deleteLineMode = false;

        treeOrthographicLines = tree;
        tree = treePoints;
    }

    printVector();
    lock1 = false;
}


void DrawingArea::parallelLines() {

    lock2 = true;

    if(lineSelected) {

        markListElement(false);
    }

    if(mode == 0) {

        mode = 2;
        tree = treeParallelLines;

    } else if(mode == 1) {

        mode = 2;

        deleteLineMode = true;

        // Alle Linien löschen durch überzeichnen
        draw(allDrawnLinesInCurrentMode, 1);

        // Alle Linien sichern bzw. den vorhandenen erweitern
        backup(allDrawnPoints);

        // Vektor löschen
        allDrawnLinesInCurrentMode.clear();

        // Alle schon gezeichneten Punkte in den Vektor laden
        load(allDrawnParallelLines);

        // Alle schon gezeichneten Punkte wieder auf den Bildschirm zeichnen
        draw(allDrawnLinesInCurrentMode, 1);
        deleteLineMode = false;

        treePoints = tree;
        tree = treeParallelLines;

    } else if(mode == 3) {

        deleteLineMode = true;

        // Alle Linien löschen durch überzeichnen
        draw(allDrawnLinesInCurrentMode, 1);

        // Alle Linien sichern bzw. den vorhandenen erweitern
        backup(allDrawnOrthographicLines);

        // Vektor löschen
        allDrawnLinesInCurrentMode.clear();

        // Muss genau hier stehen
        mode = 2;

        // Alle schon gezeichneten Punkte in den Vektor laden
        load(allDrawnParallelLines);

        // Alle schon gezeichneten Punkte wieder auf den Bildschirm zeichnen
        draw(allDrawnLinesInCurrentMode, 1);
        deleteLineMode = false;

        treeOrthographicLines = tree;
        tree = treeParallelLines;
    }

    printVector();
    lock2 = false;
}


void DrawingArea::orthographicLines() {

    lock3 = true;

    if(lineSelected) {

        markListElement(false);
    }

    if(mode == 0) {

        mode = 3;
        tree = treeOrthographicLines;

    } else if(mode == 1) {

        deleteLineMode = true;

        // Alle Linien löschen durch überzeichnen
        draw(allDrawnLinesInCurrentMode, 1);

        // Alle Linien sichern bzw. den vorhandenen erweitern
        backup(allDrawnPoints);

        // Vektor löschen
        allDrawnLinesInCurrentMode.clear();

        // Muss genau hier stehen
        mode = 3;

        // Alle schon gezeichneten Punkte in den Vektor laden
        load(allDrawnOrthographicLines);

        // Alle schon gezeichneten Punkte wieder auf den Bildschirm zeichnen
        draw(allDrawnLinesInCurrentMode, 1);
        deleteLineMode = false;

        treePoints = tree;
        tree = treeOrthographicLines;

    } else if(mode == 2) {

        deleteLineMode = true;

        // Alle Linien löschen durch überzeichnen
        draw(allDrawnLinesInCurrentMode, 1);

        // Alle Linien sichern bzw. den vorhandenen erweitern
        backup(allDrawnParallelLines);

        // Vektor löschen
        allDrawnLinesInCurrentMode.clear();

        // Muss genau hier stehen
        mode = 3;

        // Alle schon gezeichneten Punkte in den Vektor laden
        load(allDrawnOrthographicLines);

        // Alle schon gezeichneten Punkte wieder auf den Bildschirm zeichnen
        draw(allDrawnLinesInCurrentMode, 1);
        deleteLineMode = false;

        treeParallelLines = tree;
        tree = treeOrthographicLines;
    }

    printVector();
    lock3 = false;
}


// Bild öffnen und 1:1 anzeigen
bool DrawingArea::openImage(QImage img) {

    image = img;
    originalImage = img;

    resize(image.size());

    modified = false;
    update();

    return true;
}


// Startpunkt zum Linien Zeichnen setzen
void DrawingArea::mousePressEvent(QMouseEvent* event) {

    if(mode != 0 && event->button() == Qt::LeftButton && !editLine && !drawSecondLine) {

        if(lineSelected && !moveMode && !selectionMode) {

            selectLine(selectedStartPoint * scaleFactor);
        }

        startPoint = event->posF();
        lastPoint = startPoint;
        drawing = true;
        deleteLineMode = false;
        penColor = Qt::white;
    }

    if(mode != 0 && event->button() == Qt::RightButton) {

        if(!lineSelected) {

            selectLine(event->posF());
            drawing = false;
        }

        QMenu menu(this);

        if(mode == 2) {

            menu.addAction(editLineGroupAct);
        }

        menu.addAction(deleteLineAct);
        menu.exec(event->globalPos());
    }
}


// Linie zeichnen oder verschieben
void DrawingArea::mouseMoveEvent(QMouseEvent* event) {

    if((!lineSelected && drawing && !editLine && mode != 0 && (event->buttons() & Qt::LeftButton))
       || (editLine && mode != 0) || drawSecondLine) {

        QPointF eventPoint = event->posF();
        drawLineTo(eventPoint);
    }
}


// Endpunkt setzen und Linie zeichnen
void DrawingArea::mouseReleaseEvent(QMouseEvent* event) {

    if(!moveMode && selectionMode && !drawSecondLine && mode != 0 && !editLine && event->button() == Qt::LeftButton && startPoint == event->posF()) {

        drawing = false;

        if(lineSelected) {

            selectLine(selectedStartPoint * scaleFactor);

            if(clickInPoint(event->posF(), selectedStartPoint) ||
               clickInPoint(event->posF(), selectedEndPoint)) {

                return;

            } else {

                selectLine(event->posF());
            }

        } else {

            selectLine(event->posF());
        }

    } else if(!moveMode && !selectionMode && !drawSecondLine && mode != 0 && !editLine && event->button() == Qt::LeftButton && startPoint == event->posF()) {

        if(!startPointSet && !endPointSet) {

            twoClickStartPoint = startPoint;
            startPointSet = true;

        } else if(!endPointSet) {

            twoClickEndPoint = startPoint;
            endPointSet = true;

        } else if(mode == 3 && startPointSet && endPointSet && !thirdPointSet) {

            threeClickThirdPoint = startPoint;
            thirdPointSet = true;
        }

        if(mode == 3 && startPointSet && endPointSet && thirdPointSet) {

            startPoint = twoClickStartPoint;
            lastPoint = twoClickEndPoint;
            thirdPoint = threeClickThirdPoint;

            deleteLineMode = true;
            drawLineTo(lastPoint);
            lastPoint = thirdPoint;
            drawLineTo(lastPoint);
            lastPoint = twoClickEndPoint;
            deleteLineMode = false;
            drawing = false;
            setVector();

            startPointSet = false;
            endPointSet = false;
            thirdPointSet = false;

        } else if(mode != 3 && startPointSet && endPointSet) {

            startPoint = twoClickStartPoint;
            lastPoint = twoClickEndPoint;

            deleteLineMode = true;
            drawLineTo(lastPoint);
            deleteLineMode = false;
            drawing = false;
            setVector();

            startPointSet = false;
            endPointSet = false;
        }

    } else if(!moveMode && !lineSelected && drawing && mode != 0 && !editLine && event->button() == Qt::LeftButton) {

        QPointF eventPoint = event->posF();
        drawLineTo(eventPoint);

        if(mode == 3 && !drawSecondLine) {

            thirdPoint = eventPoint;
            drawSecondLine = true;
            deleteLineMode = true;
            drawLineTo(eventPoint);
            deleteLineMode = false;
            setMouseTracking(true);

        } else if(mode != 3 || drawSecondLine) {

            drawSecondLine = false;
            setMouseTracking(false);
            drawing = false;
            setVector();
       }

    } else if(moveMode && !selectionMode && !drawSecondLine && mode != 0 && !editLine && event->button() == Qt::LeftButton && startPoint == event->posF()) {

        drawing = false;

        if(lineSelected) {

            if(mode != 3 && (clickInPoint(event->posF(), selectedStartPoint) ||
                             clickInPoint(event->posF(), selectedEndPoint))) {

                editLine = true;
                drawing = true;

            } else if(mode == 3 && (clickInPoint(event->posF(), selectedStartPoint) ||
                                    clickInPoint(event->posF(), selectedEndPoint) ||
                                    clickInPoint(event->posF(), selectedThirdPoint))) {

                editLine = true;
                drawing = true;

            } else {

                selectLine(selectedStartPoint * scaleFactor);
                selectLine(event->posF());
            }

        } else {

            selectLine(event->posF());
        }

        if(lineSelected) {

            editLine = true;
            drawing = true;

            if(mode == 3) {

                if(clickInPoint(event->posF(), selectedStartPoint)) {

                    editLine = false;
                    drawing = false;
                    selectLine(event->posF());
                    return;

                } else if(clickInPoint(event->posF(), selectedEndPoint)) {

                    startPoint = selectedStartPoint * scaleFactor;
                    lastPoint = selectedEndPoint * scaleFactor;
                    pointsChanged = false;

                } else {

                    thirdPointClicked = true;
                    startPoint = selectedStartPoint * scaleFactor;
                    lastPoint = selectedThirdPoint * scaleFactor;
                    pointsChanged = false;
                }

            } else {

                if(clickInPoint(event->posF(), selectedStartPoint)) {

                    startPoint = selectedEndPoint * scaleFactor;
                    lastPoint = selectedStartPoint * scaleFactor;
                    pointsChanged = true;

                } else {

                    startPoint = selectedStartPoint * scaleFactor;
                    lastPoint = selectedEndPoint * scaleFactor;
                    pointsChanged = false;
                }
            }

            penColor = Qt::green;
            deleteLineMode = false;
            circle(lastPoint);
            setMouseTracking(true);
            hideCircles = true;

            cursor->setShape(Qt::BlankCursor);
            setCursor(*cursor);
        }

    } else if(editLine && event->button() == Qt::LeftButton) {

        QPointF eventPoint = event->posF();
        penColor = Qt::green;
        drawLineTo(eventPoint);
        circle(lastPoint);
        drawing = false;
        editLine = false;
        deleteListElement();

        if(!pointsChanged) {

            if(thirdPointClicked) {

                selectedVectorEntry->third = lastPoint / scaleFactor;
                thirdPointClicked = false;

            } else {

                selectedVectorEntry->start = startPoint / scaleFactor;
                selectedVectorEntry->end = lastPoint / scaleFactor;
            }

        } else {

            selectedVectorEntry->start = lastPoint / scaleFactor;
            selectedVectorEntry->end = startPoint / scaleFactor;
        }

        selectedStartPoint = selectedVectorEntry->start;
        selectedEndPoint = selectedVectorEntry->end;

        if(mode == 3) {

            selectedThirdPoint = selectedVectorEntry->third;
            addListElement(selectedVectorEntry->group, selectedVectorEntry->start, selectedVectorEntry->end, selectedVectorEntry->third);

        } else {

            addListElement(selectedVectorEntry->group, selectedVectorEntry->start, selectedVectorEntry->end, QPointF(0,0));
        }

        pointsChanged = false;
        printVector();

        penColor = Qt::white;
        setMouseTracking(false);
        hideCircles = false;

        cursor->setShape(Qt::CrossCursor);
        setCursor(*cursor);

        lastPoint = selectedEndPoint * scaleFactor;
    }
}


// Vektor mit Startpunkt, Endpunkt und Liniengruppe der gerade gezeichneten Linie füllen
void DrawingArea::setVector() {

    // Für den Fall, dass doch nach jeder gezeichneten Linie die Liniengruppenabfrage erscheinen soll
    //bool ok = false;
    //int lineGroup = QInputDialog::getInteger(this, tr("Line group"), tr("Enter line group:"),
    //                                         lastLineGroup, 1, 2147483647, 1, &ok);

    int lineGroup = lastLineGroup;

    //if(ok) {

    if(mode == 3) {

        allDrawnLinesInCurrentMode.push_back(drawnLines(startPoint / scaleFactor, lastPoint / scaleFactor,
                                                        lineGroup, thirdPoint / scaleFactor));
        addListElement(lineGroup, startPoint / scaleFactor, lastPoint / scaleFactor, thirdPoint / scaleFactor);

    } else {

        allDrawnLinesInCurrentMode.push_back(drawnLines(startPoint / scaleFactor,
                                                        lastPoint / scaleFactor, lineGroup));
        addListElement(lineGroup, startPoint / scaleFactor, lastPoint / scaleFactor, QPointF(0,0));
    }

    lastLineGroup = lineGroup;
    printVector();
    selectLine(lastPoint);

    //} else {

        //deleteLineMode = true;
        //drawLineTo(lastPoint);
    //}
}


// Überprüft, ob sich der angeklickte Punkt in dem Start- oder Endkreis
// einer Linie befindet
bool DrawingArea::clickInPoint(QPointF clickedPoint, QPointF fixedPoint) {

    if((clickedPoint.x() / scaleFactor - fixedPoint.x()) * (clickedPoint.x() / scaleFactor - fixedPoint.x()) +
       (clickedPoint.y() / scaleFactor - fixedPoint.y()) * (clickedPoint.y() / scaleFactor - fixedPoint.y()) <=
       lineEndRadius * lineEndRadius) {

        return true;

    } else {

        return false;
    }
}


// Linie mit einem Linksklick auswählen
// Es kann immer nur eine Linie ausgewählt werden
void DrawingArea::selectLine(QPointF clickedPoint) {

    if(!lineSelected) {

    // Überprüfen ob sich der angeklickte Punkt in einem Kreis am Ende oder Anfang
    // einer Linie befindet, indem alle Start- und Endpunkte des Vektors überprüft werden

        if(mode == 3) {

            for(std::vector<drawnLines>::iterator itAllDrawnLines = allDrawnLinesInCurrentMode.begin();
                itAllDrawnLines != allDrawnLinesInCurrentMode.end(); ++itAllDrawnLines) {

                 if(clickInPoint(clickedPoint, itAllDrawnLines->start) ||
                    clickInPoint(clickedPoint, itAllDrawnLines->end) ||
                    clickInPoint(clickedPoint, itAllDrawnLines->third)) {

                        selectedVectorEntry = itAllDrawnLines;
                        deleteLineMode = true;
                        startPoint = itAllDrawnLines->start * scaleFactor;
                        selectedStartPoint = itAllDrawnLines->start;
                        lastPoint = itAllDrawnLines->end * scaleFactor;
                        selectedEndPoint = itAllDrawnLines->end;
                        penColor = Qt::white;
                        drawLineTo(lastPoint);
                        penColor = Qt::green;
                        drawLineTo(lastPoint);

                        lastPoint = itAllDrawnLines->third * scaleFactor;
                        selectedThirdPoint = itAllDrawnLines->third;
                        penColor = Qt::white;
                        drawLineTo(lastPoint);
                        penColor = Qt::green;
                        drawLineTo(lastPoint);
                        lastPoint = itAllDrawnLines->end * scaleFactor;

                        lineSelected = true;
                        markListElement(true);

                        return;
                 }
            }

        } else {

            for(std::vector<drawnLines>::iterator itAllDrawnLines = allDrawnLinesInCurrentMode.begin();
                itAllDrawnLines != allDrawnLinesInCurrentMode.end(); ++itAllDrawnLines) {

                 if(clickInPoint(clickedPoint, itAllDrawnLines->start) ||
                    clickInPoint(clickedPoint, itAllDrawnLines->end)) {

                        selectedVectorEntry = itAllDrawnLines;
                        deleteLineMode = true;
                        startPoint = itAllDrawnLines->start * scaleFactor;
                        selectedStartPoint = itAllDrawnLines->start;
                        lastPoint = itAllDrawnLines->end * scaleFactor;
                        selectedEndPoint = itAllDrawnLines->end;
                        penColor = Qt::white;
                        drawLineTo(lastPoint);
                        penColor = Qt::green;
                        drawLineTo(lastPoint);
                        lineSelected = true;
                        markListElement(true);

                        return;
                 }
            }
        }
    }

    // Wenn eine Linie ausgewählt wurde, soll sie wieder deselektiert werden, man man auf den
    // Anfangs- oder Endpunkt klickt

    if(lineSelected) {

        if(mode == 3) {

            if(clickInPoint(clickedPoint, selectedStartPoint) ||
               clickInPoint(clickedPoint, selectedEndPoint) ||
               clickInPoint(clickedPoint, selectedThirdPoint)) {

                   deleteLineMode = true;
                   startPoint = selectedStartPoint * scaleFactor;
                   lastPoint = selectedEndPoint * scaleFactor;
                   penColor = Qt::green;
                   drawLineTo(lastPoint);
                   penColor = Qt::white;
                   drawLineTo(lastPoint);

                   lastPoint = selectedThirdPoint * scaleFactor;
                   penColor = Qt::green;
                   drawLineTo(lastPoint);
                   penColor = Qt::white;
                   drawLineTo(lastPoint);
                   lastPoint = selectedEndPoint * scaleFactor;

                   lineSelected = false;
                   markListElement(false);

                   return;
            }

        } else {

            if(clickInPoint(clickedPoint, selectedStartPoint) ||
               clickInPoint(clickedPoint, selectedEndPoint)) {

                   deleteLineMode = true;
                   startPoint = selectedStartPoint * scaleFactor;
                   lastPoint = selectedEndPoint * scaleFactor;
                   penColor = Qt::green;
                   drawLineTo(lastPoint);
                   penColor = Qt::white;
                   drawLineTo(lastPoint);
                   lineSelected = false;
                   markListElement(false);

                   return;
            }
        }
    }
}


// Rechtsklick Menu
// Liniengruppe der ausgewählten Linie ändern und Linie deselektieren
void DrawingArea::editLineGroup() {

    if(lineSelected) {

        bool ok = false;
        int lineGroup = QInputDialog::getInteger(this, tr("Line group"), tr("Enter new line group:"),
                                                 selectedVectorEntry->group, 1, 2147483647, 1, &ok);

        if(ok) {

            selectedVectorEntry->group = lineGroup;
            selectLine(selectedVectorEntry->start * scaleFactor);
            deleteListElement();
            addListElement(lineGroup, selectedVectorEntry->start, selectedVectorEntry->end, QPointF(0,0));
            markListElement(false);
            lastLineGroup = lineGroup;
            printVector();
        }
    }
}


// Rechtsklick Menu
// Linie, Listeneintrag und Vektoreintrag löschen, wenn eine Linie ausgewählt wurde
void DrawingArea::deleteLine() {

    if(lineSelected) {

        deleteLineMode = true;
        penColor = Qt::green;
        startPoint = selectedStartPoint * scaleFactor;
        lastPoint = selectedEndPoint * scaleFactor;
        drawLineTo(lastPoint);

        if(mode == 3) {

            lastPoint = selectedThirdPoint * scaleFactor;
            drawLineTo(lastPoint);
            lastPoint = selectedEndPoint * scaleFactor;
        }

        deleteListElement();
        allDrawnLinesInCurrentMode.erase(selectedVectorEntry);
        lineSelected = false;
        printVector();
    }
}


void DrawingArea::paintEvent(QPaintEvent* event) {

    QPainter painter(this);
    QRect dirtyRect = event->rect();

    painter.drawImage(dirtyRect, image, dirtyRect);
}


// Linie zum aktuellen Endpunkt im XOR-Modus zeichnen
void DrawingArea::drawLineTo(QPointF &endPoint) {

    QPainter painter(&image);
    painter.setPen(QPen(penColor, lineWidth, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
    painter.setBrush(penColor);

    // XOR-Zeichenmodus auswählen. WICHTIG!!
    painter.setCompositionMode(QPainter::RasterOp_SourceXorDestination);

    // Alte Linie und Kreise überzeichnen
    painter.drawLine(startPoint, lastPoint);

    if(!hideCircles) {

        painter.drawEllipse(startPoint, lineEndRadius, lineEndRadius);
        painter.drawEllipse(lastPoint, lineEndRadius, lineEndRadius);
    }

    // Wenn eine Linie gelöscht werden soll, wird die Linie natürlich nicht neu gezeichnet,
    // sondern nur die alte überzeichnet.
    if(!deleteLineMode) {

       // Neue Linie und Kreise zeichnen
       painter.drawLine(startPoint, endPoint);

       if(!hideCircles) {

           painter.drawEllipse(startPoint, lineEndRadius, lineEndRadius);
           painter.drawEllipse(endPoint, lineEndRadius, lineEndRadius);
       }
    }

    painter.setBrush(Qt::NoBrush);

    // Damit die Kreise sofort schön angezeigt werden
    update();

    modified = true;

    lastPoint = endPoint;
}


void DrawingArea::circle(QPointF &endPoint) {

    QPainter painter(&image);
    painter.setPen(QPen(penColor, lineWidth, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
    painter.setBrush(penColor);

    // XOR-Zeichenmodus auswählen. WICHTIG!!
    painter.setCompositionMode(QPainter::RasterOp_SourceXorDestination);

    painter.drawEllipse(endPoint, lineEndRadius, lineEndRadius);

    painter.setBrush(Qt::NoBrush);

    // Damit die Kreise sofort schön angezeigt werden
    update();

    modified = true;
}


void DrawingArea::scale(float factor) {

    image = originalImage.scaledToHeight(factor * originalImage.height(), Qt::SmoothTransformation);
    resize(image.size());
    setPixmap(QPixmap::fromImage(image));

    // Damit beim Zoomen gleichzeitig auch noch gezeichnet werden kann
    QPointF tempStart;
    QPointF tempLast;
    QPointF tempThird;
    QPointF tempTwoClickPoint;
    QPointF tempTwoClickPointEnd;

    if(drawSecondLine) {

        tempStart = startPoint / scaleFactor;
        tempThird = thirdPoint / scaleFactor;

    } else if(startPointSet && !endPointSet) {

        tempTwoClickPoint = twoClickStartPoint / scaleFactor;

    } else if(startPointSet && endPointSet) {

        tempTwoClickPoint = twoClickStartPoint / scaleFactor;
        tempTwoClickPointEnd = twoClickEndPoint / scaleFactor;

    } else if(drawing) {

        tempStart = startPoint / scaleFactor;
        tempLast = lastPoint / scaleFactor;
    }

    scaleFactor = factor;

    deleteLineMode = true;

    if(editLine) {

        hideCircles = false;
    }

    draw(allDrawnLinesInCurrentMode, 2);

    if(editLine) {

        if(thirdPointClicked) {

            lastPoint = selectedThirdPoint * scaleFactor;

        } else {

            lastPoint = selectedEndPoint * scaleFactor;
        }

        drawLineTo(lastPoint);

        hideCircles = true;
    }

    deleteLineMode = false;

    if(drawSecondLine) {

        startPoint = tempStart * scaleFactor;
        thirdPoint = tempThird * scaleFactor;

        QPointF tempLast = lastPoint;

        deleteLineMode = true;
        drawLineTo(lastPoint);
        lastPoint = thirdPoint;
        drawLineTo(lastPoint);
        deleteLineMode = false;
        lastPoint = tempLast;

    } else if(startPointSet && !endPointSet) {

        twoClickStartPoint = tempTwoClickPoint * scaleFactor;

    } else if(startPointSet && endPointSet) {

        twoClickStartPoint = tempTwoClickPoint * scaleFactor;
        twoClickEndPoint = tempTwoClickPointEnd * scaleFactor;

    } else if(drawing) {

        startPoint = tempStart * scaleFactor;
        lastPoint = tempLast * scaleFactor;

        deleteLineMode = true;
        drawLineTo(lastPoint);
        deleteLineMode = false;
    }
}


void DrawingArea::saveVector() {

    if(mode == 1) {

        backup(allDrawnPoints);

    } else if(mode == 2) {

        backup(allDrawnParallelLines);

    } else if(mode == 3) {

        backup(allDrawnOrthographicLines);
    }
}


// Vektor in der Konsole ausgeben
void DrawingArea::printVector() {

    for(std::vector<drawnLines>::iterator itAllDrawnLines = allDrawnLinesInCurrentMode.begin();
        itAllDrawnLines != allDrawnLinesInCurrentMode.end(); ++itAllDrawnLines) {

            std::cout << "Start (X/Y): " << itAllDrawnLines->start.x() << "/"
                                         << itAllDrawnLines->start.y() << std::endl;
            std::cout << "End (X/Y): "   << itAllDrawnLines->end.x() << "/"
                                         << itAllDrawnLines->end.y() << std::endl;
            std::cout << "Third (X/Y): " << itAllDrawnLines->third.x() << "/"
                                         << itAllDrawnLines->third.y() << std::endl;
            std::cout << "Group: "       << itAllDrawnLines->group << std::endl << std::endl;
    }

    std::cout << "---------------------------------------" << std::endl;
}
