/*  Copyright (C) 2011 Julian Wintermayr

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef DRAWINGAREA_H
#define DRAWINGAREA_H


#include <QtGui>
#include <QColor>
#include <QImage>
#include <QPoint>
#include <QWidget>
#include <QLabel>
#include <QTreeWidget>

#include <vector>
#include <iostream>
#include <string>
#include <sstream>

#undef CursorShape


class DrawingArea : public QLabel {
    Q_OBJECT

public:
    DrawingArea(QWidget* parent = 0);

    bool openImage(QImage img);

    void markListElement(bool select);
    void scale(float factor);
    void points();
    void parallelLines();
    void orthographicLines();

    QImage* getImage() {return &image;}
    QTreeWidget* getTree() {return tree;}
    QCursor* getCursor() {return cursor;}

    bool getLock1() {return lock1;}
    bool getLock2() {return lock2;}
    bool getLock3() {return lock3;}

    int getMode() {return mode;}
    bool getLineSelected() {return lineSelected;}

    void createCursor(Qt::CursorShape shape);

    void setSelectionMode(bool select) {selectionMode = select;}
    void setMoveMode(bool select) {moveMode = select;}

    void saveVector();

    struct drawnLines {
        QPointF start;
        QPointF end;
        int group;

        // Für den Orthogonalmodus
        QPointF third;

        drawnLines(QPointF s, QPointF e, int g): start(s), end(e), group(g) {third = QPointF(0,0);}
        drawnLines(QPointF s, QPointF e, int g, QPointF t): start(s), end(e), group(g), third(t) {}
    };

    std::vector<drawnLines>& getPoints() {return allDrawnPoints;}
    std::vector<drawnLines>& getParallelLines() {return allDrawnParallelLines;}
    std::vector<drawnLines>& getOrthographicLines() {return allDrawnOrthographicLines;}

public slots:
    void editLineGroup();
    void deleteLine();

    int markLineFromList(QTreeWidgetItem* clickedListElement, QTreeWidgetItem* prevListElement);

private:
    static const int lineEndRadius = 5;
    static const int lineWidth = 1;

    void mousePressEvent(QMouseEvent* event);
    void mouseMoveEvent(QMouseEvent* event);
    void mouseReleaseEvent(QMouseEvent* event);
    void paintEvent(QPaintEvent* event);

    void drawLineTo(QPointF &endPoint);
    void circle(QPointF &endPoint);
    void setVector();
    void printVector();
    void selectLine(QPointF clickedPoint);
    bool clickInPoint(QPointF clickedPoint, QPointF fixedPoint);
    void addListElement(int group, QPointF start, QPointF end, QPointF third);
    void deleteListElement();

    void backup(std::vector<drawnLines>& targetVector);
    void draw(std::vector<drawnLines>& drawVector, int drawMode);
    void load(std::vector<drawnLines>& sourceVector);
    void scaleVector();

    QCursor* cursor;

    bool deleteLineMode;
    bool lineSelected;
    bool modified;
    bool drawing;
    bool editLine;
    bool pointsChanged;
    bool selectionMode;
    bool moveMode;
    bool startPointSet;
    bool endPointSet;
    bool thirdPointSet;
    bool hideCircles;
    bool drawSecondLine;
    bool thirdPointClicked;

    bool lock1;
    bool lock2;
    bool lock3;

    int lastLineGroup;
    int mode;

    float scaleFactor;

    QColor penColor;
    QImage image;
    QImage originalImage;
    
    QPointF startPoint;
    QPointF lastPoint;
    QPointF thirdPoint;
    QPointF selectedStartPoint;
    QPointF selectedEndPoint;
    QPointF selectedThirdPoint;
    QPointF twoClickStartPoint;
    QPointF twoClickEndPoint;
    QPointF threeClickThirdPoint;

    QTreeWidget* tree;
    QTreeWidget* treePoints;
    QTreeWidget* treeParallelLines;
    QTreeWidget* treeOrthographicLines;
    QTreeWidgetItem* deletedListElement;

    QAction* editLineGroupAct;
    QAction* deleteLineAct;

    std::vector<drawnLines> allDrawnPoints;
    std::vector<drawnLines> allDrawnParallelLines;
    std::vector<drawnLines> allDrawnOrthographicLines;
    std::vector<drawnLines> allDrawnLinesInCurrentMode;
    std::vector<drawnLines> allDrawnLinesInCurrentModeZoomed;
    std::vector<drawnLines>::iterator selectedVectorEntry;
};

#endif // DRAWINGAREA_H
