/*  Copyright (C) 2011 Julian Wintermayr

    This file is part of Charon.

    Charon is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Charon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef MAINWINDOW_H
#define MAINWINDOW_H


#include <QMainWindow>
#include <QtGui>
#include <QScrollArea>
#include <QLabel>
#include <QDockWidget>

#include "drawingarea.h"


class MainWindow : public QDialog {
    Q_OBJECT

public:
    MainWindow(QImage img, std::vector<DrawingArea::drawnLines>* outputPoints, 
                           std::vector<DrawingArea::drawnLines>* outputParallelLines,
                           std::vector<DrawingArea::drawnLines>* outputOrthographicLines,
                           QWidget* parent = 0);

private slots:
    void points();
    void parallelLines();
    void orthographicLines();
    void calculate();

private:
    void contextMenuEvent(QContextMenuEvent* event);
    void keyPressEvent(QKeyEvent* event);
    void keyReleaseEvent(QKeyEvent* event);
    void wheelEvent(QWheelEvent* event);
    bool eventFilter(QObject* obj, QEvent* evt);

    void createActions();
    void createToolBar();

    void scaleImage(float factor);

    bool imageLoaded;

    float scaleFactor;

    DrawingArea* drawingArea;
    QScrollArea* scrollArea;
    QDockWidget* dock;

    QAction* pointsAct;
    QAction* parallelLinesAct;
    QAction* orthographicLinesAct;
    QAction* calculateAct;
    QAction* editLineGroupAct;
    QAction* deleteLineAct;

    QVBoxLayout* layoutV;
    QToolBar* mainToolBar;

    std::vector<DrawingArea::drawnLines>* pOutputPoints;
    std::vector<DrawingArea::drawnLines>* pOutputParallelLines;
    std::vector<DrawingArea::drawnLines>* pOutputOrthographicLines;
};


#endif // MAINWINDOW_H
