/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file SelectIndexedMatrixById.cpp
 *  Implementation of parameter class SelectIndexedMatrixById.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 27.02.2013
 */

#include <charon-core/ParameteredObject.hxx>
#include <hekate/SelectIndexedMatrixById.h>

SelectIndexedMatrixById::SelectIndexedMatrixById(const std::string& name) :
		ParameteredObject(
			"SelectIndexedMatrixById", name,
			"<h2>Selects IndexedMatrix by parameter list</h2><br>"
			"Selects IndexedMatrix by parameter list"
		)
		,indices()
{

	ParameteredObject::_addInputSlot(
		indexedMatrix, "indexedMatrix",
		"IndexedMatrix",
		"IndexedMatrixList");

	ParameteredObject::_addOutputSlot(
		indexedMatrixSelected, "indexedMatrixSelected",
		"Selected indexed matrices",
		"IndexedMatrixList");

	ParameteredObject::_addParameter(
		indices, "indices",
		"Indices of matrices to select");
	
}

void SelectIndexedMatrixById::execute() {

	// iterate over index parameter list and forward matrices with matching index
	for( size_t i = 0; i < this->indices().size(); i++ )
	{
		for( size_t j = 0; j < this->indexedMatrix().size(); j++ )
		{
			if( this->indexedMatrix()[j].getIndex1() == this->indices()[i] )
			{
				sout << "\tselect matrix with index " << this->indices()[i] << std::endl;
				this->indexedMatrixSelected().push_back( this->indexedMatrix()[j] );
			}
		}
	}

	sout << "Selected " << this->indexedMatrixSelected().size() << " matrices." << std::endl;

}

// the following functions are needed
// for class SelectIndexedMatrixById to work as a charon plugin.
extern "C" selectindexedmatrixbyid_DECLDIR ParameteredObject*
		create(const std::string& name, ParameteredObject::template_type) {
	return new SelectIndexedMatrixById(name);
}

extern "C" selectindexedmatrixbyid_DECLDIR void destroy(ParameteredObject* b) {
	delete b;
}

/// Report build configuration to prevent linking of incompatibel runtime libs
extern "C" selectindexedmatrixbyid_DECLDIR ParameteredObject::build_type getBuildType() {
#ifdef _DEBUG
	return ParameteredObject::DEBUG_BUILD;
#else
	return ParameteredObject::RELEASE_BUILD;
#endif
}
