/*  Copyright (C) 2010 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/

//#include <charon-core/ParameteredObject.hxx>

#include "SfmGui.h"

#include "ui_sfmgui.h"
#include "qtogpcv/QtOGPCV.h"
#include <charon-utils/Roi.h>

SfmGui::SfmGui(QWidget* p) : QMainWindow(p), _ui(new Ui::SfmGui)//, viewer2(0)
{

	this->setWindowFlags(Qt::Window | Qt::WindowMinimizeButtonHint | Qt::WindowMaximizeButtonHint | Qt::WindowCloseButtonHint);

	// Set up designer UI
	_ui->setupUi(this);

	currentImg = new QImage();

	loadedImages = false;
	calculatedCorrs = false;
	calculated3D = false;
	calculatedFM = false;
	
	_ui->tableWidget->horizontalHeader()->setResizeMode(1,QHeaderView::Stretch);
	prog = new QProgressDialog(this);
	prog->setLabelText("Calculate Correspondences");
	prog->setMinimum(0);
	prog->setMaximum(0);
	prog->setValue(-1);
	prog->setCancelButton(0);
	prog->setWindowModality(Qt::WindowModal);
	prog->show();
	prog->hide();

	timer.setInterval(1000);
	connect(&timer,SIGNAL(timeout()),this,SLOT(updateProgressBarCorr()));
		
	// Disable groups
	_ui->dockConversion->setDisabled(1);
	_ui->groupCorrs->setDisabled(1);
	_ui->toolButton_4->setDisabled(1);
	_ui->toolButton_3->setDisabled(1);
	_ui->toolButton_7->setDisabled(1);
	_ui->groupBox_2->setDisabled(1);
	_ui->groupBox_6->setDisabled(1);
	_ui->groupBox_7->setDisabled(1);
	_ui->checkBox_10->setDisabled(1);
	
	_ui->checkBox->setDisabled(1);
	_ui->spinBox->setDisabled(1);
	
	_ui->checkBox_4->setDisabled(1);
	_ui->lineEdit_3->setDisabled(1);
	_ui->lineEdit_4->setDisabled(1);

	_ui->checkBox_5->setEnabled(1);
	_ui->spinBox_4->setEnabled(1);

	// Set style sheets
	cssSuccess = "background:#584; color:#8e3;";
	cssDefault = "background:#999; color:#fff;";
	cssError = "background:#900; color:#fff;";	
	_ui->label->setStyleSheet( cssDefault );
	_ui->label_2->setStyleSheet( cssDefault );
	_ui->label_12->setStyleSheet( cssDefault );
	_ui->label_13->setStyleSheet( cssDefault );
	
	_ui->graphicsView->setDragMode( QGraphicsView::ScrollHandDrag );
	_ui->graphicsView->setScene(new QGraphicsScene());

	// Connect slots and signals
	connect(_ui->toolButton,SIGNAL(clicked()),this,SLOT(loadImagesFromFiles()));
	connect(_ui->toolButton_2,SIGNAL(clicked()),this,SLOT(loadCorrespondences()));
	connect( _ui->groupCorrs2, SIGNAL(clicked(bool)), this, SLOT(enableGroup2(bool)) );
	connect( _ui->groupCorrs1, SIGNAL(clicked(bool)), this, SLOT(enableGroup1(bool)) );
	connect( _ui->pushButton, SIGNAL(clicked()), this, SLOT(calculateCorrespondences()) );
	connect( _ui->pushButton_2, SIGNAL(clicked()), this, SLOT(calculateFundamentalMatrices()) );
	connect( _ui->pushButton_4, SIGNAL(clicked()), this, SLOT(calculate3DPoints()) );
	//bool test1 = connect( _ui->tableWidget, SIGNAL(clicked(QModelIndex)), this, SLOT(loadSelectedImage(QModelIndex))); 
	//sout << (test1 ? "ja" : "nein" ) << std::endl;
	connect( _ui->pushButton_6, SIGNAL(clicked()), this, SLOT(loadPrevious()) );
	connect( _ui->pushButton_5, SIGNAL(clicked()), this, SLOT(loadNext()) );
	
	connect( _ui->checkBox_8, SIGNAL(clicked(bool)), this, SLOT(viewFeatures(bool)) );
	connect( _ui->checkBox_9, SIGNAL(clicked(bool)), this, SLOT(viewCorrs(bool)) );
	connect( _ui->checkBox_10, SIGNAL(clicked(bool)), this, SLOT(viewBackP(bool)) );
		
	QCoreApplication::setOrganizationName("StructureFromMotion GUI");
	QCoreApplication::setOrganizationDomain("http://sourceforge.net/apps/trac/charon-suite");
	QCoreApplication::setApplicationName("Charon Suite");
		
	readSettings();

	reconstruct = new Create3DPointCloud<T>("3dCreator");

	//if(!viewer2)
	//	viewer2 = new pcl::visualization::CloudViewer("Auto-Calibration");


}


void SfmGui::updateBar1(){
	sout << fmatrix.calcCurrent << std::endl;
}


// Toggle between load/calculate correspondence groups
void SfmGui::enableGroup1(bool checked){

	_ui->groupCorrs2->setChecked(!checked);

}


// Toggle between load/calculate correspondence groups
void SfmGui::enableGroup2(bool checked){

	_ui->groupCorrs1->setChecked(!checked);

}


// Write number of correspondences per frame to tableView
void SfmGui::updateTableView(const std::vector< int > &nPointsPerImage, const int nImages){
	sout << "Update Table View: nImage = " << nImages << std::endl;
	for(size_t i = 0; i < nImages; i++)
	{		
		sout << " i = " << i << std::endl;
		// Get frame ID of row i
		QString id = _ui->tableWidget->item(i,0)->text();
		sout << "ui_id = " << id.toStdString() << std::endl;
		QString nCorrsPF = QString().setNum( nPointsPerImage[id.toInt()] );
		_ui->tableWidget->item(i,2)->setText( nCorrsPF );	
	}
}


void SfmGui::resetImages(){
	loadedImages = false;
	resetCorrs();
	_ui->groupCorrs->setDisabled(1);	
	imgList.clear();
	fileList.clear();
	size_t rowCount = _ui->tableWidget->rowCount();
	for(size_t i = 0; i < rowCount; i++)
		_ui->tableWidget->removeRow(0);
	_ui->graphicsView->setScene(new QGraphicsScene());
}


void SfmGui::resetCorrs(){
	sout << "Reset correspondences" << std::endl;
	calculatedCorrs = false;
	resetFM();
	corrList.clear();
	imgCorrs.clear();
	imgFeatures.clear();
	_ui->toolButton_3->setDisabled(1);
	_ui->groupBox_2->setDisabled(1);
	_ui->groupBox_6->setDisabled(1);
	_ui->checkBox_4->setDisabled(1);
	_ui->checkBox_8->setDisabled(1);
	_ui->checkBox_9->setDisabled(1);
	_ui->checkBox_8->setChecked(0);
	_ui->checkBox_9->setChecked(0);
	_ui->label->setStyleSheet( cssDefault );
	_ui->label->setText(tr("No correspondences loaded..."));
	_ui->lineEdit_2->setText(tr(""));
	loadImage( imgList[currentID], currentID );
	sout << "Correspondences reseted" << std::endl;
}


void SfmGui::resetFM(){
	calculatedFM = false;
	resetPointCloud();
	_ui->dockConversion->setDisabled(1);
	_ui->label_2->setStyleSheet( cssDefault );
	_ui->pushButton_2->setText(tr("Calculate Fundamental Matrices"));
	_ui->label_2->setText(tr("0 fundamental matrices calculated"));
}


void SfmGui::resetPointCloud(){
	calculated3D = false;
	imgBackP.clear();
	_ui->toolButton_4->setDisabled(1);
	_ui->groupBox_7->setDisabled(1);
	_ui->checkBox_10->setDisabled(1);
	_ui->checkBox_10->setChecked(0);
	_ui->label_12->setStyleSheet( cssDefault );
	_ui->label_12->setText(tr("0 3D points / 0 cameras reconstructed"));
	_ui->pushButton_4->setText(tr("Calculate 3D Points"));
	reconstruct->correspondencesOut().clear();
	reconstruct->pointCloud().clear();
	reconstruct->calibrationMatrixOut().clear();
	reconstruct->cameraMatrices().clear();
}


// Open dialog box and read in correspondences
void SfmGui::setCorrs(const CorrespondenceList<T> &corrs, const int size){

	calculatedCorrs = true;
	_ui->toolButton_3->setEnabled(1);
	this->corrList = corrs;

	_ui->label->setStyleSheet( cssSuccess );
	_ui->label->setText( QString().setNum(size) + " correspondences found");

	// enable options for fundamental matrices
	_ui->groupBox_2->setEnabled(1);	

	// enable viewing options for features
	_ui->groupBox_6->setEnabled(1);

	_ui->checkBox_8->setEnabled(1);
	_ui->checkBox_9->setEnabled(1);

}


void SfmGui::setFM(const int size){
	calculatedFM = true;
	_ui->dockConversion->setEnabled(1);

	_ui->pushButton_2->setText(tr("Recalculate Fundamental Matrices"));
	_ui->label_2->setText(QString().setNum(size) + tr(" fundamental matrices calculated"));
	_ui->label_2->setStyleSheet( cssSuccess );
}


void SfmGui::setPointCloud(const int sizePoints, const int sizeCams){
	calculated3D = true;
	_ui->toolButton_4->setEnabled(1);
	_ui->groupBox_7->setEnabled(1);
	_ui->checkBox_10->setEnabled(1);
	_ui->label_12->setText(QString().setNum(sizePoints) + tr(" 3D points / ")
		+ QString().setNum(sizeCams) + tr(" cameras reconstructed"));
	_ui->pushButton_4->setText(tr("Recalculate 3D Points"));
	_ui->label_12->setStyleSheet( cssSuccess );
}


// Calculate 3D structure and cameras
void SfmGui::calculate3DPoints(){

	if( calculated3D ){
		int ret = QMessageBox::warning(this, tr("Attention"),
                            tr("Do you really want to discard the"
							" 3D points and matrcies to reconstruct new ones?"),
                            QMessageBox::Discard
                            | QMessageBox::Cancel);
		switch (ret) {
			case QMessageBox::Discard:
				resetPointCloud();
				break;
			case QMessageBox::Cancel:
				return;
				break;
			default:
				return;
				break;
		}
	}

	QProgressDialog progress("Calculating 3D Points...",0,0,100,this);
	progress.setValue(90);
	progress.show();


	// Prepare correspondences
	Roi<int> test;
	OutputSlot< CorrespondenceList<T> > slotCorrs;
	OutputSlot< cimg_library::CImgList<double> > slotCalibration;
	slotCorrs.init(&test, "pseudoCorrs", "CorrespondenceList");	
	slotCalibration.init(&test, "pseudoCalirbation", "CImgList<double>");		
	slotCorrs = corrList;
	

	// Prepare calibration
	cimg_library::CImg<double> calibration(3,3);
	calibration.fill(0.);
	if( !_ui->groupBox_3->isChecked() ){
		QImage img(fileList[0]);
		double w = (double) img.width();
		double h = (double) img.height();
		double f = (w+h) / 2.;
		double ppx = w / 2.;
		double ppy = h / 2.;
		calibration(0,0) = f;
		calibration(1,1) = f;
		calibration(2,0) = ppx;
		calibration(2,1) = ppy;
		calibration(2,2) = 1.;
	}
	else{
		calibration(0,0) = _ui->lineEdit_7->text().toDouble();
		calibration(1,1) = _ui->lineEdit_7->text().toDouble() 
							/ _ui->lineEdit_8->text().toDouble();
		calibration(2,0) = _ui->doubleSpinBox_2->value();
		calibration(2,1) = _ui->doubleSpinBox_3->value();
		calibration(2,2) = 1.;
	}
	cimg_library::CImgList<double> calibrationList(calibration);
	slotCalibration = calibrationList;
	
	// Connect slots
	reconstruct->resetExecuted();
	reconstruct->calibrationMatrix.connect( slotCalibration );
	reconstruct->correspondences.connect( slotCorrs );
	reconstruct->fundamentalMatrices.connect( fmatrix.fundamentalMatrix );

	// Set parameters
	if( _ui->checkBox_7->isChecked() )
		reconstruct->maxFrames = _ui->spinBox_6->value();
	else
		reconstruct->maxFrames = atoi(reconstruct->maxFrames.getDefaultString().c_str() );

	reconstruct->initialFrame1 = _ui->spinBox_8->value() - 1 ;
	reconstruct->initialFrame2 = _ui->spinBox_9->value() - 1 ;
	reconstruct->minPtViews = _ui->spinBox_7->value();
	sout << "min value = " << _ui->spinBox_7->value() << std::endl;
	reconstruct->min3d2dCorrs = _ui->spinBox_10->value();
	reconstruct->maxBackProjectionError = _ui->doubleSpinBox->value();
	reconstruct->fixedFocalLength = _ui->checkBox_12->isChecked();
	
	
	QCoreApplication::processEvents();
	try{
		reconstruct->execute();
	}
	catch(...){
		sout << "There were errors..." << std::endl;
	}	
	
	progress.setValue(95);

	setPointCloud(reconstruct->pointCloud().size(), reconstruct->cameraMatrices().size() );

	cloud1 = reconstruct->pointCloud();
	
	//drawFeatures(reconstruct->correspondencesOut(), imgFeatures, QColor(254,254,0));
	//drawCorrs();
	drawBackP(corrList,reconstruct->backprojections);
	//_ui->checkBox_10->click();

	//if(_ui->checkBox_6->isChecked())
	performADQ();
	
	QtOGPCV *window = new QtOGPCV;
 	window->setPoints(cloud1);
	window->setCameras(reconstruct->cameraMatrices(), imgList);
	window->show();	
}


// Calculate fundamental matrices
void SfmGui::calculateFundamentalMatrices(){

	if( calculatedFM ){
		int ret = QMessageBox::warning(this, tr("Attention"),
                            tr("Do you really want to discard the"
							" fundamental matrices to calculate new ones?"),
                            QMessageBox::Discard
                            | QMessageBox::Cancel);
		switch (ret) {
			case QMessageBox::Discard:
				resetFM();
				break;
			case QMessageBox::Cancel:
				return;
				break;
			default:
				return;
				break;
		}
	}

	/*
	// Determine number of fmatrices to calculate
	int calcMax = 0;
	for(int i=1; i<=fileList.size()-1; i++)
		calcMax += i;
	calcMax += fileList.size()-1;
	*/

	QProgressDialog progress("Calculating Fundamental Matrices...",0,0,100,this);
	progress.setValue(90);
	progress.show();
	
	// Set up fundamental matrices class
	
	Roi<int> test;

	OutputSlot< CorrespondenceList<T> > slotCorrs;

	slotCorrs.init(&test, "pseudoCorrs", "CorrespondenceList");
		
	slotCorrs = corrList;
	
	fmatrix.correspondences.connect( slotCorrs );
	/*
	QTimer timer(this);
	bool test2 = connect(&timer, SIGNAL(timeout()), this, SLOT(updateBar1()));
	if(test2) sout << "!!!!!!!!!!!!!!" << std::endl;
    timer.start(100);
	*/


	// Calculate fundamental matrices
	QCoreApplication::processEvents();
	fmatrix.resetExecuted();
	fmatrix.execute();
		
	// Save matrices and correspondences
	//fmatrices = fmatrix.fundamentalMatrix();

	corrList = fmatrix.correspondenceInlier();
	
	progress.setValue(95);

	setFM( fmatrix.fundamentalMatrix().size() );
	
	

}


// Calculate Correspondences for opened frames
void SfmGui::calculateCorrespondences(){

	if( calculatedCorrs ){
		int ret = QMessageBox::warning(this, tr("Attention"),
                            tr("Do you really want to discard"
							" correspondences to calculate new ones?"),
                            QMessageBox::Discard
                            | QMessageBox::Cancel);
		switch (ret) {
			case QMessageBox::Discard:
				resetCorrs();
				break;
			case QMessageBox::Cancel:
				return;
				break;
			default:
				return;
				break;
		}
	}

	
	//timer.start(1000);
	//prog->show();
	QProgressDialog progress("Calculating Correspondences...",0,0,100,this);
	progress.setValue(99);
	progress.show();


	// Reset asift
	asift.resetExecuted();
	asift.correspondences().clear();
	
	// Define default parameters
	int defaultMinNumCorr = atoi( asift.minimalNumberOfCorrs.getDefaultString().c_str() ) ;
	int defaultMatchRange = atoi( asift.matchRange.getDefaultString().c_str() ) ;
	int defaultScaleX = atoi( asift.scaleX.getDefaultString().c_str() ) ;
	int defaultScaleY = atoi( asift.scaleY.getDefaultString().c_str() ) ;
	int defaultNumTilts = atoi( asift.nTilts.getDefaultString().c_str() ) ;
	
	// Read file list 
	std::vector<std::string> fileListVector;
	for(size_t i = 0; i < fileList.size(); i++)
		fileListVector.push_back( fileList[i].toStdString() );
	Roi<int> test;
	OutputSlot< std::vector<std::string> > slotPath;
	slotPath.init(&test, "pseudoPath", "vector<string>");
	slotPath = fileListVector;
	asift.imgPath.connect( slotPath );
	
	// Assign parameters
	asift.maximalFrames = _ui->spinBox->value();
	asift.minimalNumberOfCorrs = ( _ui->checkBox_2->isEnabled() ? _ui->spinBox_2->value() : defaultMinNumCorr );
	asift.matchRange = ( _ui->checkBox_3->isEnabled() ? _ui->spinBox_3->value() : defaultMatchRange );
	asift.scaleFrames = true;
	asift.scaleX = ( _ui->checkBox_3->isEnabled() ? _ui->lineEdit_3->text().toInt() : defaultScaleX );
	asift.scaleY = ( _ui->checkBox_3->isEnabled() ? _ui->lineEdit_4->text().toInt() : defaultScaleY );
	asift.nTilts = ( _ui->checkBox_5->isEnabled() ? _ui->spinBox_4->value() : defaultNumTilts );

	// Execuate asift
	QCoreApplication::processEvents();
	asift.execute();
	
	// Save correspondences
	setCorrs( asift.correspondences(), asift._numberOfCorrespondences );

	std::vector< int > nPointsPerImage;
	for(size_t i = 0; i < fileList.size(); i++){
		try{
			size_t size = corrList.sizePoints(i);
			nPointsPerImage.push_back(size);
		}
		catch(...){
			nPointsPerImage.push_back(0);
		}
		sout << "point for image  " << i << ": " << nPointsPerImage[i] << std::endl;
	}

	updateTableView(nPointsPerImage, fileList.size());
	
	drawFeatures(corrList, imgFeatures, QColor(254,254,0));
	drawCorrs();
	_ui->checkBox_8->click();
	
	//timer.stop();
	//prog->hide();

}


// Open dialog box and read in correspondences
void SfmGui::loadCorrespondences()
{

	if( calculatedCorrs ){
		int ret = QMessageBox::warning(this, tr("Attention"),
                            tr("Do you really want to discard"
							" correspondences to load new ones?"),
                            QMessageBox::Discard
                            | QMessageBox::Cancel);
		switch (ret) {
			case QMessageBox::Discard:
				resetCorrs();
				break;
			case QMessageBox::Cancel:
				return;
				break;
			default:
				return;
				break;
		}
	}

	QString dir = settings.value( "dirCorrs", settings.value("dirFrames") ).toString();

	// Open dialog window
	QString filename = QFileDialog::getOpenFileName(this,
		tr("Open Correspondences"), dir,
		tr("Correspondences (*.txt)"));

	// Check if selected file can be opened
	if (filename.isEmpty())
		return;
	QFile file(filename);
	if (!file.open(QIODevice::ReadOnly)) {
		QMessageBox::information(this, tr("Unable to open file"),
			file.errorString());
		return;
	}


	QProgressDialog dia(tr("Loading Correspondences"),0,0,100,this);
	dia.setWindowModality(Qt::WindowModal);
	dia.show();
	dia.setValue(5);
		

	// Update correspondence path
	settings.setValue("dirCorrs", QFileInfo(filename).absolutePath() );


	// generate reader to read in txt file
	QCoreApplication::processEvents();
	reader.resetExecuted();
	reader._out().clear();
	try{
		reader._filename = filename.toStdString() ;
		reader.execute();
	}
	catch(...){
		QMessageBox *box = new QMessageBox();
		box->setText("Could not read in correspondence file.");
		box->show();
		return;
	}

	dia.setValue(50);

	// Get correspondence dimensions
	size_t nCorrs = reader._sizeAll;
	size_t nImages = reader._out().sizeImages();
	std::vector< int > nPointsPerImage;
	for(size_t i = 0; i < nImages; i++){
		try{
			size_t size = reader._out().sizePoints(i);
			nPointsPerImage.push_back(size);
		}
		catch(...){
			nPointsPerImage.push_back(0);
		}
		int val = 70 + float(i) * 20. / float(nImages);
		dia.setValue(val);
	}
	sout << "corrList.size() = " << nCorrs << std::endl;
	sout << "corrList.sizeImages() = " << nImages << std::endl;
	sout << "_ui->tableWidget->rowCount() = " << _ui->tableWidget->rowCount() << std::endl;


	// If number of frames inside the correspondence list does
	// not match the number of read in frames in table view -> return
	if( nImages != _ui->tableWidget->rowCount()){
		QString str1, str2;
		str1.setNum( nImages );
		str2.setNum( _ui->tableWidget->rowCount() );
		QMessageBox::information(this, tr("Invalid correspondence file"),
			"Number of frames in correspondence file (" + str1 +
			") does not match with number of opened frames (" + str2 + ").");
		return;
	}


	setCorrs( reader._out(), nCorrs );

	updateTableView(nPointsPerImage, nImages);

	_ui->lineEdit_2->setText(filename);

	drawFeatures(corrList, imgFeatures, QColor(254,254,0));
	viewFeatures(true);
	_ui->checkBox_8->setChecked(1);

}
		

/// Load images from files using a QDialog
void SfmGui::loadImagesFromFiles()
{

	if( loadedImages ){
		int ret = QMessageBox::warning(this, tr("Attention"),
                            tr("Do you really want to load new images?"
							" All image dependend calculations will be reset!"),
                            QMessageBox::Discard
                            | QMessageBox::Cancel);
		switch (ret) {
			case QMessageBox::Discard:
				resetImages();
				break;
			case QMessageBox::Cancel:
				return;
				break;
			default:
				return;
				break;
		}
	}

	QString dir = settings.value( "dirFrames" , QDir().homePath() ).toString() ;

	// Get supported file formats
	QList<QByteArray> fileFormatList = QImageReader::supportedImageFormats();
	QString fileformat;
	for(int i=0; i<fileFormatList.size(); i++)
		fileformat += "*." + QString(fileFormatList.at(i)) + " ";

	// Open dialog window
	QStringList fileNameList = QFileDialog::getOpenFileNames(this,
		tr("Open Initial Frame"), dir, "Images ("+fileformat+");;All Files (*)");

	// Check if files can be opened
	QStringList fileListTemp;
	QStringList list = fileNameList;
	QStringList::Iterator it = list.begin();
	int nfiles = 0;
	while(it != list.end()) {
		if (!(it->isEmpty()))	{
			QFile file(*it);
			if (!file.open(QIODevice::ReadOnly)) {
				QMessageBox::information(this, tr("Unable to open file"),
					file.errorString());
			}
			else{
				nfiles++;
				fileListTemp.push_back(*it);
			}
		}
		++it;
	}


	// Return if read in failed
	if(nfiles == 0)
		return;

	
	// Set dir
	settings.setValue("dirFrames", QFileInfo(fileListTemp[0]).absolutePath() );


	// Case if only one files was selected
	if(nfiles == 1){
		QMessageBox *box = new QMessageBox();
		box->setText("You need to select at least two frames.");
		box->show();
		return;
	}
	
		
	QProgressDialog dia(tr("Loading Frames..."),tr("Cancel"),0,100,this);
	dia.setWindowModality(Qt::WindowModal);
	dia.show();
	dia.setValue(5);


	// Remove last selection from table view	
	size_t rowCount = _ui->tableWidget->rowCount();
	for(size_t i = 0; i < rowCount; i++)
	{
		_ui->tableWidget->removeRow(0);
	}
	
	sout << _ui->tableWidget->rowCount() << std::endl;
	sout << _ui->tableWidget->columnCount() << std::endl;
	sout << "imgList.size() = " << imgList.size() << std::endl;
	sout << "corrList.size() = " << corrList.size() << std::endl;
	
	dia.setValue(25);
	sout << " fileListTemp.size() " <<  fileListTemp.size() << std::endl;
	// Write images to table view
	for(size_t i = 0; i < fileListTemp.size(); i++)
	{
		_ui->tableWidget->insertRow( 0 );
		
		_ui->tableWidget->setItem( 0, 2, 
			new QTableWidgetItem("0"));
		_ui->tableWidget->setItem( 0, 0, 
			new QTableWidgetItem( QString().setNum(int(i)) ) );
		_ui->tableWidget->setItem( 0, 1, 
			new QTableWidgetItem( QFileInfo(fileListTemp[i]).fileName() ) );

		QImage image( fileListTemp[i]);
		image.convertToFormat(QImage::Format_ARGB32);
		// Scale if too big
		int thx = 2000, thy = 2000;
		int iw = image.width(), ih = image.height();
		if(iw > thx || ih > thy){
			if(iw > ih)
				imgList.push_back(  image.scaledToWidth(thx) );
			else
				imgList.push_back(  image.scaledToHeight(thy) );
		}
		else
			imgList.push_back( image );

		sout << image.width() << "/" << image.width() << std::endl;
		
		sout << "Add image " << fileListTemp[i].toStdString() << " to workflow" << std::endl;

		float c = float(i) / float(fileListTemp.size());
		float c2 = dia.value() + c * float(50) ;
		dia.setValue(c2);

	}
	
	dia.setValue(75);

	// Sort table by file name
	_ui->tableWidget->sortByColumn(1,Qt::AscendingOrder);


	// Write new file list to global list
	fileList = fileListTemp;

	// Enable 2D Geometry dock widget
	_ui->groupCorrs->setEnabled(1);
	_ui->spinBox->setValue(fileList.size());
	_ui->spinBox_6->setValue(fileList.size());
	_ui->pushButton_6->setDisabled(1);
	_ui->pushButton_5->setEnabled(1);
	_ui->lineEdit->setText(QFileInfo(fileListTemp[0]).absolutePath());

	loadedImages = true;

	currentID = 0;
	loadImage( imgList[currentID], currentID );
		
	dia.setValue(85);

	_ui->graphicsView->setStyleSheet("background:none; color:#000;");
		
	QImage image(fileList[0]);
	double w = (double) image.width();
	double h = (double) image.height();
	double f = (w+h) / 2.;
	double ppx = w / 2.;
	double ppy = h / 2.;
	_ui->lineEdit_7->setText(QString().setNum(f));
	_ui->doubleSpinBox_2->setValue(ppx);
	_ui->doubleSpinBox_3->setValue(ppy);

	_ui->spinBox_3->setMaximum(fileList.size());
	_ui->spinBox_8->setMaximum(fileList.size());
	_ui->spinBox_9->setMaximum(fileList.size());
	_ui->spinBox_7->setMaximum(fileList.size());

	dia.setValue(100);

}


void SfmGui::drawCorrs(){

	imgCorrs.clear();

	for(size_t j=0; j<fileList.size(); j++)
	{

		QImage image(imgList[j]);
		QPainter painter(&image);

		auto it = corrList.get<imgIndex>().lower_bound(j) ;
		auto itEnd = corrList.get<imgIndex>().upper_bound(j) ;
		for(; it != itEnd ; it++)
		{	

			// Get list of all points with that point id
			std::vector<const Correspondence<T>*> vec = 
				corrList.getListByPointId(it->pointIndex);

			if( vec.size() <= 1 )
				continue;

			for(size_t i=0; i<vec.size(); i++)
			{

				if(vec[i]->outlier)
					continue;

				// If point belongs to current image
				if(vec[i]->imgIndex == j)
				{
					painter.setPen(QPen(QColor(254,0,0), 2., Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
					painter.setBrush( QColor(254,0,0) );
					painter.drawEllipse (vec[i]->x(),vec[i]->y(),5,5);
				}

				// If point is the subsequent point to the prior one in this loop
				if( i > 0  &&  vec[i]->imgIndex == vec[i-1]->imgIndex+1 && !vec[i-1]->outlier)
				{
					painter.setPen(QPen(QColor(254,254,0), 2., Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
					painter.setBrush( QColor(254,254,0) );
					painter.drawLine(vec[i-1]->x(),vec[i-1]->y(),vec[i]->x(),vec[i]->y());		
				}

			}

		}

		imgCorrs.push_back( image );

	}

}

void SfmGui::drawBackP(const CorrespondenceList<T> &corrs, const CorrespondenceList<T> &corrsBack){
	
	imgBackP.clear();

	for(size_t i=0; i<fileList.size(); i++)
	{

		QImage image(imgList[i]);
		QPainter painter(&image);

		painter.setPen(QPen(QColor(0,254,254), 4., Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
		painter.setBrush( QColor(0,254,254) );

		auto it = corrs.get<imgIndex>().lower_bound(i) ;
		auto itEnd = corrs.get<imgIndex>().upper_bound(i) ;
		for(int j=0; it != itEnd ; it++, j++){
			try{
				const Correspondence<T> &x = corrsBack(i,it->pointIndex);
				painter.drawLine(it->x(),it->y(),x.x(),x.y());
			}
			catch(...){}
		}

		imgBackP.push_back( image );

	}

}

void SfmGui::drawFeatures(const CorrespondenceList<T> &corrs, std::vector<QImage> &list, QColor color){

	list.clear();

	//std::vector<QImage> imgFeatures, imgCorrs, imgBackP;
	sout << "corrs.sizeImages() = " << corrs.sizeImages() << std::endl;
	sout << "fileList.size() = " << fileList.size() << std::endl;
	for(size_t i=0; i<fileList.size(); i++)
	{


		QImage image(imgList[i]);
		image.convertToFormat(QImage::Format_ARGB32);
		QPainter painter(&image);

		painter.setPen(QPen(color, 2., Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
		painter.setBrush( color );

		auto it = corrs.get<imgIndex>().lower_bound(i) ;
		auto itEnd = corrs.get<imgIndex>().upper_bound(i) ;
		for(int j=0; it != itEnd ; it++, j++){
			painter.drawEllipse (it->x(),it->y(),5,5);
		}

		list.push_back( image );
	}

}


void SfmGui::viewFeatures(bool checked){

	if(checked && calculatedCorrs)
	{
		loadImage( imgFeatures[currentID], currentID );
		_ui->checkBox_9->setChecked(0);
		_ui->checkBox_10->setChecked(0);
	}
	else
	{
		loadImage( imgList[currentID], currentID );
	}

}


void SfmGui::viewCorrs(bool checked){

	if(checked && calculatedCorrs)
	{
		loadImage( imgCorrs[currentID], currentID );
		_ui->checkBox_8->setChecked(0);
		_ui->checkBox_10->setChecked(0);
	}
	else
	{
		loadImage( imgList[currentID], currentID );
	}
}


void SfmGui::viewBackP(bool checked){

	if(checked && calculated3D)
	{
		loadImage( imgBackP[currentID], currentID );
		_ui->checkBox_8->setChecked(0);
		_ui->checkBox_9->setChecked(0);
	}
	else
	{
		loadImage( imgList[currentID], currentID );
	}
}


// Load image to label
void SfmGui::loadImage( const QImage &image, const int id ){

	int width = _ui->graphicsView->geometry().width();
	int height = _ui->graphicsView->geometry().height();

	QGraphicsScene *viewScene = new QGraphicsScene(QRectF(0, 0, width, height), 0);
	QGraphicsPixmapItem *item = viewScene->addPixmap(QPixmap().fromImage(image).scaled(QSize(
			(int)viewScene->width(), (int)viewScene->height()),
			Qt::KeepAspectRatio, Qt::SmoothTransformation));

	_ui->graphicsView->fitInView(QRectF(0, 0, width, height),
	Qt::KeepAspectRatio);
	_ui->graphicsView->setScene(viewScene);
	_ui->graphicsView->show();
		
	_ui->label_13->setText( QFileInfo(fileList[id]).fileName() + 
		" (" + QString().setNum(id+1) + "/" + QString().setNum(fileList.size()) + ")");

	*currentImg = image;

}


void SfmGui::updateGraphcisView(){
	if(loadedImages)
		loadImage(*currentImg,currentID);
}


void SfmGui::updateProgressBarCorr(){
	sout << "\n\n\n\nUPDATE\n\n\n\n" << std::endl;
	//progress.setValue(asift.calcCurrent);
}


void SfmGui::loadPrevious(){
	if(currentID > 0)
	{
		currentID --;
		if(currentID <= 0)
			_ui->pushButton_6->setDisabled(1);
		if(currentID < fileList.size()-1)
			_ui->pushButton_5->setEnabled(1);

		if( _ui->checkBox_8->isChecked() )
			viewFeatures(1);
		else if( _ui->checkBox_9->isChecked() )
			viewCorrs(1);
		else if( _ui->checkBox_10->isChecked() )
			viewBackP(1);
		else
			loadImage( imgList[currentID], currentID );
	}
}


void SfmGui::loadNext(){
	if(currentID < fileList.size()-1)
	{
		currentID ++;
		if(currentID >= fileList.size()-1)
			_ui->pushButton_5->setDisabled(1);
		if(currentID > 0)
			_ui->pushButton_6->setEnabled(1);

		if( _ui->checkBox_8->isChecked() )
			viewFeatures(1);
		else if( _ui->checkBox_9->isChecked() )
			viewCorrs(1);
		else if( _ui->checkBox_10->isChecked() )
			viewBackP(1);
		else
			loadImage( imgList[currentID], currentID );
	}
}


// Load image to label
void SfmGui::loadSelectedImage(const QModelIndex & index){

	//_ui->tableWidget->currentIndex()
	QMessageBox::information(this, tr("Unable to open file"),
			"werew");

	sout << "lalalal" << std::endl;
	//loadImage( index.row() );
	//sout << index.row() << std::endl;
	//QModelIndexList idlist = _ui->tableWidget;
	//loadImage( id );

}


void SfmGui::resizeEvent(QResizeEvent * event ){
	updateGraphcisView();
}


void SfmGui::closeEvent(QCloseEvent *event)
{
	settings.setValue("geometry", saveGeometry());
	settings.setValue("windowState", saveState());
	QMainWindow::closeEvent(event);
}


void SfmGui::readSettings()
{
	restoreGeometry(settings.value("geometry").toByteArray());
	restoreState(settings.value("windowState").toByteArray());
}


void SfmGui::on_pushButton_3_clicked(){
	pcl::visualization::CloudViewer *viewer;
	reconstruct->updatePCLViewer(viewer);
}


void SfmGui::on_toolButton_3_clicked(){
	save2D();
}


void SfmGui::on_toolButton_4_clicked(){
	save3D();
}


void SfmGui::save2D(){
	
	QString dir = settings.value( "dirSaveCorrs", settings.value("dirCorrs") ).toString();

	// Open dialog window
	QString filename = QFileDialog::getSaveFileName(this,
		tr("Save 2D Correspondences"), dir,
		tr("Correspondences (*.txt)"));

	if(filename.isNull())
		return;
	
	settings.setValue("dirSaveCorrs", QDir(filename).absolutePath() );

	QProgressDialog dia(tr("Save 2D Correspondences..."),tr("Cancel"),0,100,this);
	dia.setWindowModality(Qt::WindowModal);
	dia.show();
	dia.setValue(5);

	writer2D.resetExecuted();
	writer2D._out().clear();
	writer2D._in.disconnect();

	Roi<int> test;
	OutputSlot< CorrespondenceList<T> > slotCorrs;
	slotCorrs.init(&test, "pseudoCorrs", "CorrespondenceList");		
	slotCorrs = corrList;	
	writer2D._in.connect( slotCorrs );
	writer2D._filename = filename.toStdString();

	writer2D.execute();

	dia.setValue(100);

}


void SfmGui::save3D(){
	
	QString dir = settings.value( "dirSave3D", settings.value("dirCorrs") ).toString();

	// Open dialog window
	QString filename = QFileDialog::getSaveFileName(this,
		tr("Save 3D Point Cloud"), dir,
		tr("3D Point Cloud (*.txt)"));

	if(filename.isNull())
		return;
	
	settings.setValue("dirSave3D", QDir(filename).absolutePath() );

	QProgressDialog dia(tr("Save 3D Point Cloud..."),tr("Cancel"),0,100,this);
	dia.setWindowModality(Qt::WindowModal);
	dia.show();
	dia.setValue(5);

	writer3D.resetExecuted();
	writer3D._out().clear();
	writer3D._in.disconnect();

	Roi<int> test;
	OutputSlot< CorrespondenceList<T> > slotCorrs;
	slotCorrs.init(&test, "pseudoCorrs", "CorrespondenceList");		
	slotCorrs = reconstruct->pointCloud();	
	writer3D._in.connect( slotCorrs );
	writer3D._filename = filename.toStdString();

	writer3D.execute();

	dia.setValue(100);

}


void SfmGui::saveImage(){

	// file name
	QString filename2;
	if( _ui->checkBox_8->isChecked() )
		filename2 = fileList[currentID] + "_features_" + QString().setNum(currentID) + ".png";
	else if( _ui->checkBox_9->isChecked() )
		filename2 = fileList[currentID] + "_correspondences_" + QString().setNum(currentID) + ".png";
	else if( _ui->checkBox_10->isChecked() )
		filename2 = fileList[currentID] + "_backprojection_" + QString().setNum(currentID) + ".png";
	else
		filename2 =  fileList[currentID] + "_" + QString().setNum(currentID) + ".png";

	QString dir = settings.value( "dirSaveImage", settings.value("dirFrames") ).toString();

	// Open dialog window
	QString filename = QFileDialog::getSaveFileName(this,
		tr("Save Image"), dir+filename2,
		tr("PNG (*.png)"));

	if(filename.isNull())
		return;
	
	settings.setValue("dirSaveImage", QDir(filename).absolutePath() );
	
	
	QProgressDialog progress("Save Image...",0,0,100,this);
	progress.setValue(99);
	progress.show();


	if( _ui->checkBox_8->isChecked() )
		imgFeatures[currentID].save(filename);
	else if( _ui->checkBox_9->isChecked() )
		imgCorrs[currentID].save(filename);
	else if( _ui->checkBox_10->isChecked() )
		imgBackP[currentID].save(filename);
	else
		imgList[currentID].save(filename);

}


void SfmGui::saveImages(){

}


void SfmGui::on_toolButton_6_clicked(){
	saveImage();
}


void SfmGui::on_toolButton_7_clicked(){
	saveImages();
}


void SfmGui::on_checkBox_6_clicked(){
	_ui->checkBox_14->setChecked(!_ui->checkBox_6->isChecked());

	if(_ui->checkBox_6->isChecked())
		updatePCLViewer(reconstruct->viewer, cloud1);
	else
		updatePCLViewer(reconstruct->viewer, cloud2);
	
}

void SfmGui::on_checkBox_14_clicked(){
	_ui->checkBox_6->setChecked(!_ui->checkBox_14->isChecked());

	if(_ui->checkBox_14->isChecked())
		updatePCLViewer(reconstruct->viewer, cloud2);
	else
		updatePCLViewer(reconstruct->viewer, cloud1);
	
}




/// Save WRL file
void SfmGui::on_toolButton_5_clicked(){

	sout << "Save WRL" << std::endl;

	QString dir = settings.value( "dirSave3D", settings.value("dirCorrs") ).toString();

	// Open dialog window
	QString filename = QFileDialog::getSaveFileName(this,
		tr("Export WRL File"), dir,
		tr("Virtual Reality Modeling Language (*.wrl)"));

	if(filename.isNull())
		return;
	
	settings.setValue("dirSave3D", QDir(filename).absolutePath() );
		
	QProgressDialog progress("Export WRL file...",0,0,100,this);
	progress.setValue(99);
	progress.show();
	

	filewriterWRL<T> wrl;
	wrl.resetExecuted();
	
	Roi<int> test;
	OutputSlot< CorrespondenceList<T> > slot3D;
	OutputSlot< IndexedMatrixList > slotCam;
	slot3D.init(&test, "pseudo3D", "CorrespondenceList");	
	slotCam.init(&test, "pseudoCam", "IndexedMatrixList");		
	slot3D = reconstruct->pointCloud();		
	slotCam = reconstruct->cameraMatrices();	
	wrl.pointCloud.connect( slot3D );
	wrl.cameraMatrices.connect( slotCam );
	wrl.filename = filename.toStdString();

	wrl.execute();

}


/// ADQ
void SfmGui::performADQ(){
	
	UpgradeMetricADQ<T> adq;
	adq.resetExecuted();
	
	Roi<int> test;
	OutputSlot< CorrespondenceList<T> > slot3D;
	OutputSlot< IndexedMatrixList > slotCam;
	OutputSlot< CImgList<T> > slotCali;
	slot3D.init(&test, "pseudo3D", "CorrespondenceList");	
	slotCam.init(&test, "pseudoCam", "IndexedMatrixList");		
	slotCali.init(&test, "pseudoCam", "CImgList<double>");		
	slot3D = reconstruct->pointCloud();		
	slotCam = reconstruct->cameraMatrices();	
	slotCali = reconstruct->calibrationMatrixOut();	
	adq.calibrationMatrix.connect(slotCali);
	adq.cameraMatrices.connect(slotCam);
	adq.input3dPointCloud.connect(slot3D);

	adq.execute();

	cloud2 = adq.output3dPointCloud();

	//updatePCLViewer(reconstruct->viewer, adq.output3dPointCloud() );

}


void setPointSize(pcl::visualization::PCLVisualizer &viewer)
{
	viewer.setPointCloudRenderingProperties(pcl::visualization::PCL_VISUALIZER_POINT_SIZE, 4.0);
}

// Update PCL Point cloud and show it in PCL Viewer
void SfmGui::updatePCLViewer(pcl::visualization::CloudViewer *view, const CorrespondenceList<T> pointCloud)
{
	view->runOnVisualizationThreadOnce(setPointSize);
	

	sout << "Updating Point Cloud Viewer2" << std::endl;
	pcl::PointCloud<pcl::PointXYZRGBA>::Ptr cloud (new pcl::PointCloud<pcl::PointXYZRGBA>);
	auto it = pointCloud.begin();
	auto itEnd = pointCloud.end();
	for(; it != itEnd; it++)
	{
		if(!it->outlier)
		{
			pcl::PointXYZRGBA pt;
			pt.x = it->x() ;
			pt.y = it->y() ;
			pt.z = it->z() ;
			if(_ui->checkBox_13->isChecked()){
				pt.r = it->R;
				pt.g = it->G;
				pt.b = it->B;
			}
			else{
				pt.r = 254.;
				pt.g = 254.;
				pt.b = 254.;
			}
			cloud->push_back(pt);
		}
	}
	sout << "Drawing " << cloud->size() << " points in 3d point cloud viewer2" << std::endl;
	
	view->showCloud(cloud);
	
}


SfmGui::~SfmGui(){
	delete reconstruct;
 	//if(viewer2)
	//	delete viewer2;
	delete _ui;
}

//#include "SfmGui.moc"
