/*  Copyright (C) 2012 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** @file SfmGui.h
 *
 */

#ifndef SFMGUI_H_
#define SFMGUI_H_

#include <charon-utils/ImgTool.hxx>
#include <QtGui>
#include <string>
#include <stdio.h>
#include <charon-utils/CImg.h>
#include <hekate/Correspondence.hxx>
#include <hekate/IndexedMatrix.h>
#include <features/FeatureSequenceGPUASIFT.h>
#include <hekate/CorrespondenceReadWrite.h>
#include <hekate/FundamentalMatrix.h>
#include <hekate/Create3DPointCloud.h>
#include <hekate/FilewriterWRL.h>
#include <pcl/visualization/cloud_viewer.h>
#include <pcl/io/io.h>
#include <pcl/io/pcd_io.h>
#include <hekate/UpgradeMetricADQ.h>

using namespace hekate;

namespace Ui {
	class SfmGui;
}


class SfmGui : public QMainWindow
{
	Q_OBJECT
	typedef double T;

public:
	/// default constructor
	explicit SfmGui(QWidget* parent = 0);
	virtual ~SfmGui();
	/// initialize page
	//virtual void initializePage();

private slots:
	/// Load images from files using a QDialog
	void loadImagesFromFiles();
	/// Open dialog box and read in correspondences
	void loadCorrespondences();
	/// Toggle between load/calculate correspondence groups
	void enableGroup1(bool checked);
	void enableGroup2(bool checked);
	/// Calculate Correspondences for opened frames
	void calculateCorrespondences();
	/// Calculate fundamental matrices
	void calculateFundamentalMatrices();
	/// Calculate 3D structure and cameras
	void calculate3DPoints();
	void viewFeatures(bool checked);
	void viewCorrs(bool checked);
	void viewBackP(bool checked);

	void loadImage( const QImage &image, const int id);
	void loadSelectedImage(const QModelIndex & index);
	void updateBar1();
	void loadPrevious();
	void loadNext();
	
	void resetImages();
	void resetCorrs();
	void resetPointCloud();
	void resetFM();

	void setCorrs(const CorrespondenceList<T> &corrs, const int size);
	void setPointCloud(const int sizePoints, const int sizeCams);
	void setFM(const int size);

	void updateGraphcisView();
	void updateProgressBarCorr();
	
	void on_pushButton_3_clicked();
	void on_toolButton_3_clicked();
	void on_toolButton_4_clicked();
	void on_toolButton_6_clicked();
	void on_toolButton_7_clicked();
	void on_toolButton_5_clicked();
	void on_checkBox_6_clicked();
	void on_checkBox_14_clicked();

	void save2D();
	void save3D();

	void saveImage();
	void saveImages();
	
	void SfmGui::performADQ();


private:
	/// designer GUI
	Ui::SfmGui* _ui;
	/// list of file paths of the selected images
	QStringList fileList;
	/// Stylesheet for different labels states
	QString cssSuccess;
	QString cssDefault;
	QString cssError;
	
	// Show Point Cloud in PCL CloudViewer
	//pcl::visualization::CloudViewer *viewer2;

	/// CorrespondenceList for 2D correspondences
	CorrespondenceList<T> corrList;

	/// CorrespondenceList for 2D correspondences
	CorrespondenceList<T> cloud1, cloud2;

	/// Class to read in CorrespondenceList<T> from file
	CorrespondenceReadWrite<T> reader;		

	/// Class to write CorrespondenceList<T> to file
	CorrespondenceReadWrite<T> writer2D;	

	/// Class to write CorrespondenceList<T> to file
	CorrespondenceReadWrite<T> writer3D;		

	/// Class to determine correspondences
	FeatureSequenceGPUASIFT<T> asift;

	/// Class to reconstruct 3D points and cameras
	Create3DPointCloud<T> *reconstruct;

	/// Fundamental Matrices
	FundamentalMatrix<T> fmatrix;
	//QProgressDialog *dia;

	std::vector<QImage> imgList, imgFeatures, imgCorrs, imgBackP;
	QImage *currentImg;
	
	bool loadedImages;
	bool calculatedCorrs;
	bool calculated3D;
	bool calculatedFM;

	int currentID;

	/// Write number of correspondences per frame to tableView
	void updateTableView(const std::vector< int > &nPointsPerImage, const int nImages);

	QSettings settings;
		
	QGraphicsScene scene;

	QTimer timer;

	QProgressDialog *prog;

	void closeEvent(QCloseEvent *event);
	void resizeEvent(QResizeEvent * event);
	void readSettings();
	
	void drawFeatures(const CorrespondenceList<T> &corrs, std::vector<QImage> &list, QColor color);
	void drawCorrs();
	void drawBackP(const CorrespondenceList<T> &corrs, const CorrespondenceList<T> &corrsBack);
	void updatePCLViewer(pcl::visualization::CloudViewer *viewer, const CorrespondenceList<T> pointCloud);

};


#endif /* SFMGUI_H_ */
