#ifndef _QtOGPCV_H
#define _QtOGPCV_H

#define sout std::cout

#include <windows.h>
#include <gl/gl.h>
#include <QtOpenGL/QGLWidget>
#include <QtOpenGL/QGLBuffer>
#include "ArcBall.h"
#include <qtimer.h>
#include <QElapsedTimer>
#include <map>
#include <QImage>
#include <hekate/IndexedMatrix.h>
#include <hekate/Correspondence.hxx>

class QtOGPCV : public QGLWidget {

    Q_OBJECT // must include this if you use Qt signals/slots

    typedef double T;

	enum DISPLAYMODE{
		VBO,
		DISPLAYLIST,
		SLOW,
	};

	struct point{
		float x;
		float y;
		float z;

		float r;
		float g;
		float b;

		int number;
	};

	struct camera{
		Tuple3fT  	position;
		Matrix3fT 	rotation;
		Matrix4fT 	calibration;
		int 		width;
		int 		height;
		QImage		image;
		GLuint		textur;
		int 		originalNr;
	};

	private:
		//Points & real Cameras
		std::vector<point> points;
		std::vector<camera> cameras;
		
		int currentCamera;

		std::map<Qt::Key, QElapsedTimer> keys;

		//Information about the cameras view
		Tuple3fT position;

		//----ArcBall

		// Finale Transformation
		Matrix4fT	Transform;
		Matrix3fT	LastRot;
		Matrix3fT	ThisRot;

		ArcBallT	ArcBall;					
		Point2fT	MousePt;								
		bool		isClicked;							
		bool		isRClicked;							
		bool		isDragging;			

		//Settings
		int 	width;
		int 	height;

		float 	pointSize;
		bool 	invertBGColor;
		bool	drawPoints;
		bool	useCalibration;
		bool	drawCamBG;
		float	moveSpeed;

		//Optimize Rendering with a VBO or Displaylist
		DISPLAYMODE mode;

		//VBO
		QGLBuffer vertexVBO;
		QGLBuffer colorVBO;

		//Displaylist
		bool 	updateList;
		GLuint 	indexList;

		//Timer to process Keyboard events
		QTimer *timer;

		void paintPoints();
		void drawPointsNr();
		void freeCameras();

	private slots:
		void timerTimeout();

	public:
		QtOGPCV(QWidget *parent = NULL);
		void setPoints(const hekate::CorrespondenceList<T> &pointCloud);
		void setPointSize(float size);
		void setCameras(const IndexedMatrixList &cameras, const std::vector<QImage> &images);
		virtual ~QtOGPCV();

	protected:
		void initializeGL();
		void resizeGL(int w, int h);
		void paintGL();
		void mousePressEvent(QMouseEvent *event);
		void mouseMoveEvent(QMouseEvent *event);
		void mouseReleaseEvent(QMouseEvent *event);
		void keyPressEvent(QKeyEvent *event);
		void keyReleaseEvent(QKeyEvent *event);
	
};

#endif  /* _GLWIDGET_H */
