
#define LoadIdentyMatrix(matrix, width, height) \
	for(int x = 0; x < width; x++){				\
		for(int y = 0; y < height; y++){		\
			if(x == y){							\
				matrix.M[y*width+x] = 1;		\
			}else{								\
				matrix.M[y*width+x] = 0;		\
			}									\
		}										\
	}	


double detMatrix(const double m[16]){

	double inverse0, inverse4, inverse8, inverse12;

	inverse0   = m[5]  * m[10] * m[15] - m[5]  * m[11] * m[14] - 
				 m[9]  * m[6]  * m[15] + m[9]  * m[7]  * m[14] +
				 m[13] * m[6]  * m[11] - m[13] * m[7]  * m[10];


	inverse4   = -m[4]  * m[10] * m[15] + m[4]  * m[11] * m[14] + 
				  m[8]  * m[6]  * m[15] - m[8]  * m[7]  * m[14] - 
				  m[12] * m[6]  * m[11] + m[12] * m[7]  * m[10];


	inverse8   = m[4]  * m[9] * m[15] -  m[4]  * m[11] * m[13] - 
				 m[8]  * m[5] * m[15] +  m[8]  * m[7]  * m[13] + 
				 m[12] * m[5] * m[11] -  m[12] * m[7]  * m[9];

	inverse12  = -m[4]  * m[9] * m[14] + m[4]  * m[10] * m[13] +
				  m[8]  * m[5] * m[14] - m[8]  * m[6]  * m[13] - 
				  m[12] * m[5] * m[10] + m[12] * m[6]  * m[9];

	return m[0] * inverse0 + m[1] * inverse4 + m[2] * inverse8 + m[3] * inverse12;
}

bool invertMatrix4x4(const double m[16], double inverse[16]){

	inverse[0] = m[5]  * m[10] * m[15] - m[5]  * m[11] * m[14] - 
				 m[9]  * m[6]  * m[15] + m[9]  * m[7]  * m[14] +
				 m[13] * m[6]  * m[11] - m[13] * m[7]  * m[10];

	inverse[1] = -m[1]  * m[10] * m[15] + m[1]  * m[11] * m[14] + 
				  m[9]  * m[2]  * m[15] - m[9]  * m[3]  * m[14] - 
				  m[13] * m[2]  * m[11] + m[13] * m[3]  * m[10];

	inverse[2] = m[1]  * m[6] * m[15] - m[1]   * m[7] * m[14] - 
				 m[5]  * m[2] * m[15] + m[5]   * m[3] * m[14] + 
				 m[13] * m[2] * m[7]  -  m[13] * m[3] * m[6];

	inverse[3] = -m[1] * m[6] * m[11] + m[1] * m[7] * m[10] + 
				  m[5] * m[2] * m[11] - m[5] * m[3] * m[10] - 
				  m[9] * m[2] * m[7]  + m[9] * m[3] * m[6];

	inverse[4] = -m[4]  * m[10] * m[15] + m[4]  * m[11] * m[14] + 
				  m[8]  * m[6]  * m[15] - m[8]  * m[7]  * m[14] - 
				  m[12] * m[6]  * m[11] + m[12] * m[7]  * m[10];

	inverse[5] = m[0]  * m[10] * m[15] - m[0]  * m[11] * m[14] - 
				 m[8]  * m[2]  * m[15] + m[8]  * m[3]  * m[14] + 
				 m[12] * m[2]  * m[11] - m[12] * m[3]  * m[10];

	inverse[6] = -m[0]  * m[6] * m[15] + m[0]   * m[7] * m[14] + 
				  m[4]  * m[2] * m[15] - m[4]   * m[3] * m[14] - 
				  m[12] * m[2] * m[7]  +  m[12] * m[3] * m[6];

	inverse[7] = m[0] * m[6] * m[11] - m[0] * m[7] * m[10] - 
				 m[4] * m[2] * m[11] + m[4] * m[3] * m[10] + 
				 m[8] * m[2] * m[7] -  m[8] * m[3] * m[6];

	inverse[8] = m[4]  * m[9] * m[15] -  m[4]  * m[11] * m[13] - 
				 m[8]  * m[5] * m[15] +  m[8]  * m[7]  * m[13] + 
				 m[12] * m[5] * m[11] -  m[12] * m[7]  * m[9];

	inverse[9] = -m[0]  * m[9] * m[15] + m[0]  * m[11] * m[13] + 
				  m[8]  * m[1] * m[15] - m[8]  * m[3]  * m[13] - 
				  m[12] * m[1] * m[11] + m[12] * m[3]  * m[9];

	inverse[10] = m[0]  * m[5] * m[15] - m[0]  * m[7] * m[13] - 
				  m[4]  * m[1] * m[15] + m[4]  * m[3] * m[13] + 
				  m[12] * m[1] * m[7] -  m[12] * m[3] * m[5];

	inverse[11] = -m[0] * m[5] * m[11] + m[0] * m[7] * m[9] + 
				   m[4] * m[1] * m[11] - m[4] * m[3] * m[9] - 
				   m[8] * m[1] * m[7] +  m[8] * m[3] * m[5];

	inverse[12] = -m[4]  * m[9] * m[14] + m[4]  * m[10] * m[13] +
				   m[8]  * m[5] * m[14] - m[8]  * m[6]  * m[13] - 
				   m[12] * m[5] * m[10] + m[12] * m[6]  * m[9];

	inverse[13] = m[0]  * m[9] * m[14] - m[0]  * m[10] * m[13] - 
				  m[8]  * m[1] * m[14] + m[8]  * m[2]  * m[13] + 
				  m[12] * m[1] * m[10] - m[12] * m[2]  * m[9];

	inverse[14] = -m[0]  * m[5] * m[14] + m[0]  * m[6] * m[13] + 
				   m[4]  * m[1] * m[14] - m[4]  * m[2] * m[13] - 
				   m[12] * m[1] * m[6] +  m[12] * m[2] * m[5];

	inverse[15] = m[0] * m[5] * m[10] - m[0] * m[6] * m[9] - 
				  m[4] * m[1] * m[10] + m[4] * m[2] * m[9] + 
				  m[8] * m[1] * m[6] -  m[8] * m[2] * m[5];

	double det;

	det = m[0] * inverse[0] + m[1] * inverse[4] + m[2] * inverse[8] + m[3] * inverse[12];

	if (det){ 

		for (int i = 0; i < 16; i++)
			inverse[i] /= det;
		
		return true;
	}

	return false;
	
}

bool invertMatrix3x3(const double m[9], double inverse[9]){

	inverse[0] = m[4]*m[8] - m[5]*m[7];
	inverse[1] = m[2]*m[7] - m[1]*m[8];
	inverse[2] = m[1]*m[5] - m[2]*m[4];

	inverse[3] = m[5]*m[6] - m[3]*m[8];
	inverse[4] = m[0]*m[8] - m[2]*m[6];
	inverse[5] = m[2]*m[3] - m[0]*m[5];

	inverse[6] = m[3]*m[7] - m[4]*m[6];
	inverse[7] = m[1]*m[6] - m[0]*m[7];
	inverse[8] = m[0]*m[4] - m[1]*m[3];

	double det;
	det = m[0] * inverse[0] + m[1] * inverse[3] + m[2] * inverse[6];

	if (det){ 

		for (int i = 0; i < 9; i++)
			inverse[i] /= det;
		
		return true;
	}

	return false;

}

void multMatrix4x4(const double left[16], const double right[16], double result[16]){

	for (int i = 0; i < 4; i++){
		for(int j = 0; j < 4; j++){

			double temp = 0.0;

			for(int k = 0; k < 4; k++)
				temp += left[i*4+k] * right[k*4+j];

			result[i*4+j] = temp;

		}
	}
	
}

void multMatrix3x3(const double left[9], const double right[9], double result[9]){

	for (int i = 0; i < 3; i++){
		for(int j = 0; j < 3; j++){

			double temp = 0.0;

			for(int k = 0; k < 3; k++)
				temp += left[i*3+k] * right[k*3+j];

			result[i*3+j] = temp;

		}
	}
	
}

void decomposeQR3x3(double matrix[9], double q[9]){

	// Set q to the unit matrix
	for(int i = 0; i < 3; i++){
		for(int j = 0; j < 3; j++){
			q[i*3+j] = (i == j) ? 1.0 : 0.0;
		}
	}

	// Start the iteration
	for(int k = 0; k < 3; k++){

	
		// Check if we have to rotate this column?
		bool rowIsZero = true;
		for(int y = k+1; y < 3; y++){
			if( matrix[y * 3 + k] != 0.0 ){
				rowIsZero = false;
				break;
			}
		}
		if( rowIsZero ) continue;


		// Temporary u vector
		double u[3];

		// Copy a part of the current row into this vector
		for(int y = 0; y < k; y++){
			u[y] = 0.0;
		}
		for(int y = k; y < 3; y++){
			u[y] = matrix[y * 3 + k];
		}

		double d = u[k];
		if( d != 0.0 ){
		double unorm = sqrt( u[0]*u[0] + u[1]*u[1] + u[2]*u[2]);
		double sgn = (d >= 0) ? 1.0 : -1.0;

			u[k] += unorm * sgn;
		}

		// Normalize the u vector
		double unorm = sqrt( u[0]*u[0] + u[1]*u[1] + u[2]*u[2]);
		for(int i = 0; i < 3; i++){        
			u[i] /= unorm;
		}

		// Householder-Matrix
		double h[9];

		for(int i = 0; i < 3; i++){
			for(int j = 0; j < 3; j++){
				h[i*3+j] = ((i == j) ? 1.0 : 0.0) - 2.0 * u[i] * u[j];
			}
		}

		double temp[9];
		multMatrix3x3(q, h, temp);
		for(int i = 0; i < 9; i++){
			q[i] = temp[i];
		}

		multMatrix3x3(h, matrix, temp); // Update the current matrix for the next iteration
		for(int i = 0; i < 9; i++){
			matrix[i] = temp[i];
		}

		// Ensure that this is really zero!
		for(int y = k+1; y < 3; y++){
			matrix[ y * 3 + k ] = 0.0;
		}

	}
	
	for(int k =0; k< 3; k++){
		if(matrix[k*3+k] < 0){
			for(int i = 0; i < 3; i++){
				matrix[k*3+i] *= -1;
				q[i*3+k] *= -1;    			
			}
		}
	}
	
}
