/*  Copyright (C) 2013 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file Value2String.cpp
 *  Implementation of parameter class Value2String.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 27.06.2013
 */

#include <charon-core/ParameteredObject.hxx>
#include <hekate/Value2String.h>
#include <iostream>
#include <sstream>


Value2String::Value2String(const std::string& name) :
		ParameteredObject(
			"Value2String", name,
			"<h2>Converts a value to a vector of strings</h2><br>"
			"This module can be used, to assign numbers to the filewriter "
			"names."
		)
{

	ParameteredObject::_addInputSlot(
		value, "value",
		"Value",
		"uint");

	ParameteredObject::_addOutputSlot(
		names, "names",
		"Value saved as a vector of string",
		"vector<string>");

}

void Value2String::execute() {
	
	//double val1 = this->value();
	//std::string val = std::to_string( val1 ) ;

	int number = this->value();       // number to be converted to a string

	std::string result;          // string which will contain the result

	std::ostringstream convert;   // stream used for the conversion

	convert << number;      // insert the textual representation of 'Number' in the characters in the stream

	result = convert.str(); 

	this->names().push_back( result );

}

// the following functions are needed
// for class Value2String to work as a charon plugin.
extern "C" value2string_DECLDIR ParameteredObject*
		create(const std::string& name, ParameteredObject::template_type) {
	return new Value2String(name);
}

extern "C" value2string_DECLDIR void destroy(ParameteredObject* b) {
	delete b;
}

/// Report build configuration to prevent linking of incompatibel runtime libs
extern "C" value2string_DECLDIR ParameteredObject::build_type getBuildType() {
#ifdef _DEBUG
	return ParameteredObject::DEBUG_BUILD;
#else
	return ParameteredObject::RELEASE_BUILD;
#endif
}
