//Source file for the implementation of GPU (=CUDA) Powered
//rotating and tilting function by Martin Treiber, 26.03.2012

#include "hekate/features/gpu_rotation.h"
#include <iostream>
#include <cstdlib>

//Cuda Kernel
//Operate on a single Target Pixel and set them to the final value
//doesnt blur, so no longer used
__global__ void setThisTargetPixel(int width, int height,int targetwidth, int targetheight,int shiftx,int shifty,float coss,float coss2, float sins,float sins2, float reverse_tilt, CUdeviceptr d_dest, CUdeviceptr d_source)
{
	//Save the target coordinates:
	int x= blockIdx.x;
	int y= blockIdx.y;
	if(x<targetwidth && y<targetheight)
	{
		unsigned char* simage= (unsigned char*)d_source;
		unsigned char* dimage= (unsigned char*)d_dest;
		//Save the coordinates again to manipulate them:
		float mx=x;
		float my=y;
		//first, revert the tilt:
		my=my/reverse_tilt;
		//second, revert the shift
		mx = mx - shiftx;
		my = my - shifty;
		//third, revert the rotation
		float tmpx=mx;
		mx = coss*mx-sins*my;
		my = coss*my+sins*tmpx;
		//last, retrieve the pixel information from up to four neighboring pixels
		int lx=mx;
		int ly=my;
		/*if(lx>=0 && ly>=0 && lx<width && ly < height)
			dimage[targetwidth*y+x]=simage[width*ly+lx];
		else
			dimage[targetwidth*y+x]=0;*/
		int ux=(mx+1);
		int uy=(my+1);
		unsigned char ll,lu,ul,uu;
		if(lx>=0 && lx <width)
		{
			if(ly>=0 && ly<height)
				ll=simage[width*ly+lx];
			else
				ll=0;
			if(uy>=0 && uy<height)
				lu=simage[width*uy+lx];
			else
				lu=0;
		}else
		{
			ll=0;
			lu=0;
		}
		if(ux>=0 && ux <width)
		{
			if(ly>=0 && ly<height)
				ul=simage[width*ly+ux];
			else
				ul=0;
			if(uy>=0 && uy<height)
				uu=simage[width*uy+ux];
			else
				uu=0;
		}else
		{
			ul=0;
			uu=0;
		}
		float dll,dlu,dul,duu;
		dll=(mx-lx)*(mx-lx)+(my-ly)*(my-ly);
		dlu=(mx-lx)*(mx-lx)+(my-uy)*(my-uy);
		dul=(mx-ux)*(mx-ux)+(my-ly)*(my-ly);
		duu=(mx-ux)*(mx-ux)+(my-uy)*(my-uy);
		/*dll=abs((mx-lx))+abs((my-ly));
		dul=abs((mx-ux))+abs((my-ly));
		dlu=abs((mx-lx))+abs((my-uy));
		duu=abs((mx-ux))+abs((my-uy));*/
		if(dll==0)
		{
			dimage[targetwidth*y+x]=ll;
		}else if(dlu==0)
		{
			dimage[targetwidth*y+x]=lu;
		}else if(dul==0)
		{
			dimage[targetwidth*y+x]=ul;
		}else if(duu==0)
		{
			dimage[targetwidth*y+x]=uu;
		}else 
		{
			dll = 1/dll;dlu=1/dlu;dul=1/dul;duu=1/duu;
			float dsum=dll+dlu+dul+duu;
			dll/=dsum;
			dlu/=dsum;
			dul/=dsum;
			duu/=dsum;
			dimage[targetwidth*y+x]=(unsigned char)((float)ll*dll+(float)lu*dlu+(float)ul*dul+(float)uu*duu);
		}
	}
}

//Rotates the sourceimage
__global__ void rotate(int width, int height, int targetwidth, int targetheight, float coss, float sins, int shiftx, int shifty,CUdeviceptr d_dest, CUdeviceptr d_source)
{
	int x= blockIdx.x;
	int y =blockIdx.y;
	float mx=x;
	float my=y;
	mx = mx - shiftx;
	my = my - shifty;

	float tmpx=mx;
	mx = coss*mx-sins*my;
	my = coss*my+sins*tmpx;
	unsigned char* source= (unsigned char*)d_source;
	unsigned char* dest=(unsigned char*)d_dest;
	int lx=mx;
		int ly=my;
		if(lx>=0 && ly>=0 && lx<width && ly < height)
			dest[targetwidth*y+x]=source[width*ly+lx];
		else
			dest[targetwidth*y+x]=0;
		/*int ux=(mx+1);
		int uy=(my+1);
		unsigned char ll,lu,ul,uu;
		if(lx>=0 && lx <width)
		{
			if(ly>=0 && ly<height)
				ll=source[width*ly+lx];
			else
				ll=0;
			if(uy>=0 && uy<height)
				lu=source[width*uy+lx];
			else
				lu=0;
		}else
		{
			ll=0;
			lu=0;
		}
		if(ux>=0 && ux <width)
		{
			if(ly>=0 && ly<height)
				ul=source[width*ly+ux];
			else
				ul=0;
			if(uy>=0 && uy<height)
				uu=source[width*uy+ux];
			else
				uu=0;
		}else
		{
			ul=0;
			uu=0;
		}
		float dll,dlu,dul,duu;
		dll=abs((mx-lx))+abs((my-ly));
		dul=abs((mx-ux))+abs((my-ly));
		dlu=abs((mx-lx))+abs((my-uy));
		duu=abs((mx-ux))+abs((my-uy));
		if(dll==0)
		{
			dest[targetwidth*y+x]=ll;
		}else if(dlu==0)
		{
			dest[targetwidth*y+x]=lu;
		}else if(dul==0)
		{
			dest[targetwidth*y+x]=ul;
		}else if(duu==0)
		{
			dest[targetwidth*y+x]=uu;
		}else 
		{
			dll = 1/dll;dlu=1/dlu;dul=1/dul;duu=1/duu;
			float dsum=dll+dlu+dul+duu;
			dll/=dsum;
			dlu/=dsum;
			dul/=dsum;
			duu/=dsum;
			dest[targetwidth*y+x]=(unsigned char)((float)ll*dll+(float)lu*dlu+(float)ul*dul+(float)uu*duu);
		}*/

}

//blurs thesourceimage in y-direction
__global__ void yBlur(int width, int height,int blurSize,CUdeviceptr d_kernel,CUdeviceptr d_source, CUdeviceptr d_dest)
{
	int x=blockIdx.x;
	int y=blockIdx.y;
	unsigned char* kernel=(unsigned char*)d_kernel;
	unsigned char* source=(unsigned char*)d_source;
	unsigned char* dest=(unsigned char*)d_dest;
	int bl=blurSize/2;
	if(y-bl>=0 && y+bl<height)
	{
		int value=0;
		for(int i=0;i<blurSize;++i)
			value += (int)kernel[i]*(int)source[width*(y-bl+i)+x];

		value /=255;
		dest[width*y+x]=(unsigned char)value;
	}else
	{
		dest[width*y+x]=source[width*y+x];
	}
}

//resizes the image in y direction
__global__ void ytilt(int width, int height, int targetwidth, int targetheight, float tilt, CUdeviceptr d_dest, CUdeviceptr d_source)
{
	int x=blockIdx.x;
	int y=blockIdx.y;
	int y2 = y*tilt;
	((unsigned char*)d_dest)[targetwidth*y+x]=((unsigned char*)d_source)[width*y2+x];
}

//copies an image to the gpu
CUdeviceptr copyImageToDevice(int width, int height, unsigned char* h_image)
{
	CUdeviceptr d_target;
	cuMemAlloc(&d_target, width*height);
	cuMemcpyHtoD(d_target,h_image,width*height);
	return d_target;
}

//copies an image from the gpu into ram
void copyDeviceToImage(int width, int height, unsigned char* h_dest, CUdeviceptr d_source)
{
	cuMemcpyDtoH(h_dest,d_source,width*height);

}


//rotates,tilts and blurs an image, copy the result with copyDeviceToImage before you use it
CUdeviceptr rotateAndTilt(int width, int height, int targetwidth,int targetheight,float angle, float tilt, CUdeviceptr d_source)
{
	
	CUdeviceptr d_target;
		
		//calculate shift to apply after rotation
		int x,y;
		x=y=0;
		if(angle<90)
		{
			y=width*sin(angle*M_PI/180);
		}else if(angle<180)
		{
			y=targetheight*tilt;
			x=width*cos((180-angle)*M_PI/180);
		}else if(angle<270)
		{
			x=targetwidth;
			y=height*sin((angle-90)*M_PI/180);
		}else
		{
			x=height*cos((angle-270)*M_PI/180);
		}

	dim3 blocks(targetwidth,targetheight*tilt,1);
	cuMemAlloc(&d_target, targetwidth*targetheight*tilt);
	rotate<<<blocks,1>>>(width,height,targetwidth,targetheight*tilt,cos(angle*M_PI/180),sin(angle*M_PI/180),x,y,d_target,d_source);
	int blurSize=6.4*tilt;
	if(blurSize%2==0)
		blurSize++;
	long double* kernel_float= new long double[blurSize];
	
	long double sum=0;
	for(int i=0;i<blurSize;++i)
	{
		kernel_float[i]=blurSize*pow(M_E,-(double)(i-blurSize/2)*(double)(i-blurSize/2)/(blurSize/2.0));
		//std::cout<<kernel_float[i]<<" ";
		sum +=kernel_float[i];
	}
	//std::cout<<std::endl;
	unsigned char* kernel = new unsigned char[blurSize];
	for(int i=0;i<blurSize;++i)
	{
		kernel[i]=(unsigned char)(kernel_float[i]/sum*255);
		//std::cout<<(int) kernel[i]<<" ";
	}
	//std::cout<<std::endl;
	CUdeviceptr d_kernel;
	cuMemAlloc(&d_kernel,blurSize);
	cuMemcpyHtoD(d_kernel,kernel,blurSize);
	delete[] kernel_float;
	delete[] kernel;

	CUdeviceptr d_target2;
	cuMemAlloc(&d_target2,targetwidth*targetheight*tilt);
	
	
	yBlur<<<blocks,1>>>(targetwidth,targetheight*tilt,blurSize,d_kernel,d_target,d_target2);
	cudaFree((void*)d_target);
	cudaFree((void*)d_kernel);
	CUdeviceptr d_target3;


	dim3 blocks2(targetwidth,targetheight,1);
	
	cuMemAlloc(&d_target3,targetwidth*targetheight);
	ytilt<<<blocks2,1>>>(targetwidth,targetheight*tilt,targetwidth,targetheight,tilt,d_target3,d_target2);
	cudaFree((void*)d_target2);
	return d_target3;
}

void freeMems(unsigned char* d_ptr)
{
	cudaFree(d_ptr);
}