#include <QTest>
#include "cImg2Pixmap.h"
#include <AdjustBaseDisplay/convToPixmap.hxx>

using namespace cimg_library;

void CImg2Pixmap::testSimple() {
	testCImg.assign(1,1);
	testCImg.fill(5);
	convToPixmap(testCImg, pixmap);
	testQImg = pixmap.toImage();
	QCOMPARE(testQImg.width(),1);
	QCOMPARE(testQImg.height(),1);
	QVERIFY(testQImg.pixel(0,0) == QColor(5,5,5).rgb());
	QVERIFY(testQImg.isGrayscale());

	testCImg.assign(3,2);
	testCImg.fill(3);
	convToPixmap(testCImg,pixmap);
	testQImg = pixmap.toImage();
	QCOMPARE(testQImg.width(),3);
	QCOMPARE(testQImg.height(),2);
	QVERIFY(testQImg.pixel(0,0) == QColor(3,3,3).rgb());
	QVERIFY(testQImg.pixel(1,1) == QColor(3,3,3).rgb());
	QVERIFY(testQImg.pixel(2,1) == QColor(3,3,3).rgb());
	QVERIFY(testQImg.isGrayscale());
}

void CImg2Pixmap::testGrayscale() {
	testCImg.assign(2,4);
	testCImg.fill(0);
	testCImg(0,0) = 1;
	testCImg(1,3) = 5;
	convToPixmap(testCImg, pixmap);
	testQImg = pixmap.toImage();
	QCOMPARE(testQImg.width(),2);
	QCOMPARE(testQImg.height(),4);
	QVERIFY(testQImg.pixel(0,0) == QColor(1,1,1).rgb());
	QVERIFY(testQImg.pixel(1,1) == QColor(0,0,0).rgb());
	QVERIFY(testQImg.pixel(1,3) == QColor(5,5,5).rgb());
	QVERIFY(testQImg.isGrayscale());
}

void CImg2Pixmap::testColor() {
	testCImg.assign(2,4,1,3);
	testCImg.fill(0);
	testCImg.fillC(0,0,0,1,2,3);
	QCOMPARE(testCImg(0,0,0,0), (uchar) 1u);
	QCOMPARE(testCImg(0,0,0,1), (uchar) 2u);
	QCOMPARE(testCImg(0,0,0,2), (uchar) 3u);
	testCImg.fillC(1,3,0,3,2,1);
	QCOMPARE(testCImg(1,3,0,0), (uchar) 3u);
	QCOMPARE(testCImg(1,3,0,1), (uchar) 2u);
	QCOMPARE(testCImg(1,3,0,2), (uchar) 1u);
	convToPixmap(testCImg, pixmap);
	testQImg = pixmap.toImage();
	QCOMPARE(testQImg.width(),2);
	QCOMPARE(testQImg.height(),4);
	QVERIFY(testQImg.pixel(0,0) == QColor(1,2,3).rgb());
	QVERIFY(testQImg.pixel(1,1) == QColor(0,0,0).rgb());
	QVERIFY(testQImg.pixel(1,3) == QColor(3,2,1).rgb());
	QVERIFY(!testQImg.isGrayscale());
}

QTEST_MAIN(CImg2Pixmap)
#include "cImg2Pixmap.moc"
